@@ -1,7 +1,7 @@
 #ifndef SCM_LIBGUILE_H
 #define SCM_LIBGUILE_H
 
-/* Copyright (C) 1995,1996,1997,1998,2000,2001, 2002, 2003 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1997,1998,2000,2001, 2002, 2003, 2004 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,6 +43,10 @@
  * If you do not wish that, delete this exception notice.  */
 
 
+/* This needs to be included outside of the extern "C" block.
+ */
+#include <gmp.h>
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -123,6 +127,7 @@ extern "C" {
 #include "libguile/values.h"
 #include "libguile/variable.h"
 #include "libguile/vectors.h"
+#include "libguile/srfi-4.h"
 #include "libguile/version.h"
 #include "libguile/vports.h"
 #include "libguile/weaks.h"
