@@ -2,22 +2,23 @@
 
 #ifndef SCM___SCM_H
 #define SCM___SCM_H
-/* Copyright (C) 1995,1996,1998,1999,2000,2001 Free Software Foundation, Inc.
- * 
+
+/* Copyright (C) 1995,1996,1998,1999,2000,2001,2002,2003 Free Software Foundation, Inc.
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301 USA
+ * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307 USA
  *
  * As a special exception, the Free Software Foundation gives permission
  * for additional uses of the text contained in its release of GUILE.
@@ -42,18 +43,39 @@
  * If you write modifications of your own for GUILE, it is your choice
  * whether to permit this exception to apply to your modifications.
  * If you do not wish that, delete this exception notice.  */
+
 
 
-/* "What's the difference between _scm.h and __scm.h?"
+/**********************************************************************
+ This file is Guile's central public header.
+
+ When included by other files, this file should preceed any include
+ other than __scm.h.
+
+ Under *NO* circumstances should new items be added to the global
+ namespace (via adding #define, typedef, or similar to this file) with
+ generic names.  This usually means that any new names should be
+ prefixed by either SCM_ or GUILE_.  i.e. do *not* #define HAVE_FOO or
+ SIZEOF_BAR.  See configure.in, gen-scmconfig.h.in, and
+ gen-scmconfig.c for examples of how to properly handle this issue.
+ The main documentation is in gen-scmconfig.c.
+
+ "What's the difference between _scm.h and __scm.h?"
 
    _scm.h is not installed; it's only visible to the libguile sources
-   themselves.
+   themselves, and it includes config.h, the private config header.
 
    __scm.h is installed, and is #included by <libguile.h>.  If both
    the client and libguile need some piece of information, and it
    doesn't fit well into the header file for any particular module, it
-   should go in __scm.h.  */
+   should go in __scm.h.  __scm.h includes scmconfig.h, the public
+   config header.
+ **********************************************************************/
 
+/* What did the configure script discover about the outside world?  */
+#include "libguile/scmconfig.h"
+
+
 
 /* {Compiler hints}
  *
@@ -92,28 +114,8 @@
  * These may be defined or undefined.
  */
 
-/* Old async mechanism */
-/* #define GUILE_OLD_ASYNC_CLICK */
-
 /* #define GUILE_DEBUG_FREELIST */
 
-/* If the compile FLAG `SCM_CAUTIOUS' is #defined then the number of
- * arguments is always checked for application of closures.  If the
- * compile FLAG `SCM_RECKLESS' is #defined then they are not checked.
- * Otherwise, number of argument checks for closures are made only when
- * the function position (whose value is the closure) of a combination is
- * not an ILOC or GLOC.  When the function position of a combination is a
- * symbol it will be checked only the first time it is evaluated because
- * it will then be replaced with an ILOC or GLOC.
- */
-#undef SCM_RECKLESS
-#define SCM_CAUTIOUS
-
-/* After looking up a local for the first time, rewrite the
- * code graph, caching its position.
- */
-#define MEMOIZE_LOCALS
-
 /* All the number support there is.
  */
 #define BIGNUMS
@@ -154,8 +156,17 @@
 
 
 
-/* What did the configure script discover about the outside world?  */
-#include "libguile/scmconfig.h"
+/* SCM_API is a macro prepended to all function and data definitions
+   which should be exported or imported in the resulting dynamic link
+   library (DLL) in the Win32 port. */
+
+#if defined (SCM_IMPORT)
+# define SCM_API __declspec (dllimport) extern
+#elif defined (SCM_EXPORT) || defined (DLL_EXPORT)
+# define SCM_API __declspec (dllexport) extern
+#else
+# define SCM_API extern
+#endif
 
 
 
@@ -164,9 +175,10 @@
  * These compile time options determine whether to include code that is only
  * useful for debugging guile itself or C level extensions to guile.  The
  * common prefix for all option macros of this kind is "SCM_DEBUG_".  It is
- * guaranteed that a macro named SCM_DEBUG_XXX is defined to be either 0 or 1,
- * i. e. there is no need to test for the undefined case.  This allows to use
- * these definitions comfortably in macro code, as in the following example:
+ * guaranteed that a macro named SCM_DEBUG_XXX is always defined (typically to
+ * either 0 or 1), i. e. there is no need to test for the undefined case.
+ * This allows to use these definitions comfortably within code, as in the
+ * following example:
  *   #define FOO do { if (SCM_DEBUG_XXX) bar(); else baz(); } while (0)
  * Any sane compiler will remove the unused branch without any performance
  * penalty for the resulting code.
@@ -195,13 +207,6 @@
 #define SCM_DEBUG_CELL_ACCESSES SCM_DEBUG
 #endif
 
-/* If SCM_DEBUG_DEPRECATED is set to 1, deprecated code is not compiled.  This
- * can be used by developers to get rid of references to deprecated code.
- */
-#ifndef SCM_DEBUG_DEPRECATED
-#define SCM_DEBUG_DEPRECATED SCM_DEBUG
-#endif
-
 /* If SCM_DEBUG_INTERRUPTS is set to 1, with every deferring and allowing of
  * interrupts a consistency check will be performed.
  */
@@ -226,67 +231,110 @@
 #define SCM_DEBUG_REST_ARGUMENT SCM_DEBUG
 #endif
 
-/* Use this for _compile time_ type checking only, since the compiled result
- * will be quite inefficient.  The right way to make use of this option is to
- * do a 'make clean; make CFLAGS=-DSCM_DEBUG_TYPING_STRICTNESS=1', fix your
+/* The macro SCM_DEBUG_TYPING_STRICTNESS indicates what level of type checking
+ * shall be performed with respect to the use of the SCM datatype.  The macro
+ * may be defined to one of the values 0, 1 and 2.
+ *
+ * A value of 0 means that there will be no compile time type checking, since
+ * the SCM datatype will be declared as an integral type.  This setting should
+ * only be used on systems, where casting from integral types to pointers may
+ * lead to loss of bit information.
+ *
+ * A value of 1 means that there will an intermediate level of compile time
+ * type checking, since the SCM datatype will be declared as a pointer to an
+ * undefined struct.  This setting is the default, since it does not cost
+ * anything in terms of performance or code size.
+ *
+ * A value of 2 provides a maximum level of compile time type checking since
+ * the SCM datatype will be declared as a struct.  This setting should be used
+ * for _compile time_ type checking only, since the compiled result is likely
+ * to be quite inefficient.  The right way to make use of this option is to do
+ * a 'make clean; make CFLAGS=-DSCM_DEBUG_TYPING_STRICTNESS=2', fix your
  * errors, and then do 'make clean; make'.
  */
 #ifndef SCM_DEBUG_TYPING_STRICTNESS
 #define SCM_DEBUG_TYPING_STRICTNESS 1
 #endif
 
-/* If SCM_ENABLE_VCELLS is set to 1, a couple of functions that deal
- * with vcells are defined for compatability reasons.  Supporting
- * vcells reduces performance however.
- *
- * We use a dedicated macro instead of just SCM_DEBUG_DEPRECATED so
- * that code the belongs to the `vcell' feature is easier to find.
+/* If SCM_DEBUG_DEBUGGING_SUPPORT is set to 1, guile will provide a set of
+ * special functions that support debugging with a debugger like gdb or
+ * debugging of guile internals on the scheme level.  The behaviour of guile
+ * is not changed by this macro, only the set of functions that are available
+ * will differ.  All functions that are introduced this way have the prefix
+ * 'scm_dbg_' on the C level and the prefix 'dbg-' on the scheme level.  This
+ * allows to easily determine the set of support functions, given that your
+ * debugger or repl provide automatic name completion.  Note that these
+ * functions are intended to be used during interactive debugging sessions
+ * only.  They are not considered part of guile's official API.  They may
+ * change or disappear without notice or deprecation phase.
  */
-#define SCM_ENABLE_VCELLS !SCM_DEBUG_DEPRECATED
+#ifndef SCM_DEBUG_DEBUGGING_SUPPORT
+#define SCM_DEBUG_DEBUGGING_SUPPORT SCM_DEBUG
+#endif
 
 
 
-#ifdef HAVE_LONG_LONGS
-
-/* Some auto-generated .h files contain unused prototypes
- * that need these typedefs.
+/* {Feature Options}
+ *
+ * These compile time options determine whether code for certain features
+ * should be compiled into guile.  The common prefix for all option macros
+ * of this kind is "SCM_ENABLE_".  It is guaranteed that a macro named
+ * SCM_ENABLE_XXX is defined to be either 0 or 1, i. e. there is no need to
+ * test for the undefined case.  This allows to use these definitions
+ * comfortably within code, as in the following example:
+ *   #define FOO do { if (SCM_ENABLE_XXX) bar(); else baz(); } while (0)
+ * Any sane compiler will remove the unused branch without any performance
+ * penalty for the resulting code.
+ *
+ * Note:  Some SCM_ENABLE_XXX options are not settable at configure time.
+ * To change the value of such options you will have to edit this header
+ * file or give suitable options to make, like:
+ *   make all CFLAGS="-DSCM_ENABLE_XXX=1 ..."
  */
 
-#if (SCM_DEBUG_DEPRECATED == 0)
-typedef long long long_long;
-typedef unsigned long long ulong_long;
+/* If SCM_ENABLE_DEPRECATED is set to 1, deprecated code will be included in
+ * guile, as well as some functions to issue run-time warnings about uses of
+ * deprecated functions.
+ */
+#ifndef SCM_ENABLE_DEPRECATED
+#define SCM_ENABLE_DEPRECATED 0
 #endif
 
-#endif /* HAVE_LONG_LONGS */
-
 
 
 /* {Architecture and compiler properties}
  *
  * Guile as of today can only work on systems which fulfill at least the
  * following requirements:
- * - long ints have at least 32 bits.
+ *
+ * - scm_t_bits and SCM variables have at least 32 bits.
  *   Guile's type system is based on this assumption.
- * - long ints consist of at least four characters.
- *   It is assumed that cells, i. e. pairs of long ints, are eight character
- *   aligned, because three bits of a cell pointer are used for type data.
- * - sizeof (void*) == sizeof (long int)
- *   Pointers are stored in SCM objects, and sometimes SCM objects are passed
- *   as void*.  Thus, there has to be a one-to-one correspondence.
+ *
+ * - sizeof (scm_t_bits) >= sizeof (void*) and sizeof (SCM) >= sizeof (void*)
+ *   Guile's type system is based on this assumption, since it must be
+ *   possible to store pointers to cells on the heap in scm_t_bits and SCM
+ *   variables.
+ *
+ * - sizeof (scm_t_bits) >= 4 and sizeof (scm_t_bits) is a power of 2.
+ *   Guile's type system is based on this assumption.  In particular, it is
+ *   assumed that cells, i. e. pairs of scm_t_bits variables, are eight
+ *   character aligned.  This is because three bits of a scm_t_bits variable
+ *   that is holding a pointer to a cell on the heap must be available for
+ *   storing type data.
+ *
+ * - sizeof (scm_t_bits) <= sizeof (void*) and sizeof (SCM) <= sizeof (void*)
+ *   In some parts of guile, scm_t_bits and SCM variables are passed to
+ *   functions as void* arguments.  Together with the requirement above, this
+ *   requires a one-to-one correspondence between the size of a void* and the
+ *   sizes of scm_t_bits and SCM variables.
+ *
  * - numbers are encoded using two's complement.
  *   The implementation of the bitwise scheme level operations is based on
  *   this assumption.
+ *
  * - ... add more
  */
 
-#ifndef HAVE_PTRDIFF_T
-typedef long ptrdiff_t;
-#endif
-
-#ifdef HAVE_LIMITS_H
-# include <limits.h>
-#endif
-
 #ifdef CHAR_BIT
 # define SCM_CHAR_BIT CHAR_BIT
 #else
@@ -305,23 +353,42 @@ typedef long ptrdiff_t;
 # define SCM_CHAR_CODE_LIMIT 256L
 #endif
 
-
+#define SCM_I_UTYPE_MAX(type)      ((type)-1)
+#define SCM_I_TYPE_MAX(type,umax)  ((type)((umax)/2))
+#define SCM_I_TYPE_MIN(type,umax)  (-((type)((umax)/2))-1)
 
-#ifdef STDC_HEADERS
-# include <stdlib.h>
-# if HAVE_SYS_TYPES_H
-#  include <sys/types.h>
-# endif
-# if HAVE_SYS_STDTYPES_H
-#  include <sys/stdtypes.h>
-# endif
-#  include <stddef.h>
-#endif /* def STDC_HEADERS */
+#define SCM_T_UINT8_MAX   SCM_I_UTYPE_MAX(scm_t_uint8)
+#define SCM_T_INT8_MIN    SCM_I_TYPE_MIN(scm_t_int8,SCM_T_UINT8_MAX)
+#define SCM_T_INT8_MAX    SCM_I_TYPE_MAX(scm_t_int8,SCM_T_UINT8_MAX)
 
-#if (SCM_DEBUG_DEPRECATED == 0)
-# define scm_sizet size_t
+#define SCM_T_UINT16_MAX  SCM_I_UTYPE_MAX(scm_t_uint16)
+#define SCM_T_INT16_MIN   SCM_I_TYPE_MIN(scm_t_int16,SCM_T_UINT16_MAX)
+#define SCM_T_INT16_MAX   SCM_I_TYPE_MAX(scm_t_int16,SCM_T_UINT16_MAX)
+
+#define SCM_T_UINT32_MAX  SCM_I_UTYPE_MAX(scm_t_uint32)
+#define SCM_T_INT32_MIN   SCM_I_TYPE_MIN(scm_t_int32,SCM_T_UINT32_MAX)
+#define SCM_T_INT32_MAX   SCM_I_TYPE_MAX(scm_t_int32,SCM_T_UINT32_MAX)
+
+#if SCM_HAVE_T_INT64
+#define SCM_T_UINT64_MAX  SCM_I_UTYPE_MAX(scm_t_uint64)
+#define SCM_T_INT64_MIN   SCM_I_TYPE_MIN(scm_t_int64,SCM_T_UINT64_MAX)
+#define SCM_T_INT64_MAX   SCM_I_TYPE_MAX(scm_t_int64,SCM_T_UINT64_MAX)
 #endif
 
+#if SCM_SIZEOF_LONG_LONG
+#define SCM_I_ULLONG_MAX  SCM_I_UTYPE_MAX(unsigned long long)
+#define SCM_I_LLONG_MIN   SCM_I_TYPE_MIN(long long,SCM_I_ULLONG_MAX)
+#define SCM_I_LLONG_MAX   SCM_I_TYPE_MAX(long long,SCM_I_ULLONG_MAX)
+#endif
+
+#define SCM_T_UINTMAX_MAX SCM_I_UTYPE_MAX(scm_t_uintmax)
+#define SCM_T_INTMAX_MIN  SCM_I_TYPE_MIN(scm_t_intmax,SCM_T_UINTMAX_MAX)
+#define SCM_T_INTMAX_MAX  SCM_I_TYPE_MAX(scm_t_intmax,SCM_T_UINTMAX_MAX)
+
+#define SCM_I_SIZE_MAX    SCM_I_UTYPE_MAX(size_t)
+#define SCM_I_SSIZE_MIN   SCM_I_TYPE_MIN(ssize_t,SCM_I_SIZE_MAX)
+#define SCM_I_SSIZE_MAX   SCM_I_TYPE_MAX(ssize_t,SCM_I_SIZE_MAX)
+
 
 
 #include "libguile/tags.h"
@@ -351,7 +418,7 @@ typedef long ptrdiff_t;
 
 /* James Clark came up with this neat one instruction fix for
  * continuations on the SPARC.  It flushes the register windows so
- * that all the state of the process is contained in the stack. 
+ * that all the state of the process is contained in the stack.
  */
 
 #if defined (sparc) || defined (__sparc__) || defined (__sparc)
@@ -360,7 +427,7 @@ typedef long ptrdiff_t;
 # define SCM_FLUSH_REGISTER_WINDOWS /* empty */
 #endif
 
-/* If stack is not longword aligned then 
+/* If stack is not longword aligned then
  */
 
 /* #define SHORT_ALIGN */
@@ -379,52 +446,17 @@ typedef short SCM_STACKITEM;
 #else
 typedef long SCM_STACKITEM;
 #endif
+
+/* Cast pointer through (void *) in order to avoid compiler warnings
+   when strict aliasing is enabled */
+#define SCM_STACK_PTR(ptr) ((SCM_STACKITEM *) (void *) (ptr))
 
 
-#ifndef USE_THREADS
-#define SCM_CRITICAL_SECTION_START 
-#define SCM_CRITICAL_SECTION_END 
-#define SCM_THREAD_SWITCHING_CODE
-#endif
-
-#ifdef GUILE_OLD_ASYNC_CLICK
-extern unsigned int scm_async_clock;
-
-#define SCM_ASYNC_TICK \
-do { \
-  if (0 == --scm_async_clock) \
-    scm_async_click (); \
-} while(0)
-#else
-extern int scm_asyncs_pending_p;
-
 #define SCM_ASYNC_TICK /*fixme* should change names */ \
 do { \
-  if (scm_asyncs_pending_p) \
+  if (SCM_I_CURRENT_THREAD->pending_asyncs) \
     scm_async_click (); \
 } while (0)
-#endif
-
-#if (SCM_DEBUG_INTERRUPTS == 1)
-#include <stdio.h>
-#define SCM_CHECK_NOT_DISABLED \
-  do { \
-    if (scm_ints_disabled) \
-      fprintf(stderr, "ints already disabled (at %s:%d)\n", \
-              __FILE__, __LINE__); \
-  } while (0)
-
-#define SCM_CHECK_NOT_ENABLED \
-  do { \
-    if (!scm_ints_disabled) \
-      fprintf(stderr, "ints already enabled (at %s:%d)\n", \
-              __FILE__, __LINE__); \
-  } while (0)
-
-#else
-#define SCM_CHECK_NOT_DISABLED
-#define SCM_CHECK_NOT_ENABLED
-#endif
 
 
 /* Anthony Green writes:
@@ -444,60 +476,12 @@ do { \
    are implicitly volatile. */
 #ifdef __GNUC__
 #define SCM_FENCE asm /* volatile */ ("")
+#elif defined (__INTEL_COMPILER) && defined (__ia64)
+#define SCM_FENCE __memory_barrier()
 #else
 #define SCM_FENCE
 #endif
 
-#define SCM_DEFER_INTS \
-do { \
-  SCM_FENCE; \
-  SCM_CHECK_NOT_DISABLED; \
-  SCM_CRITICAL_SECTION_START; \
-  SCM_FENCE; \
-  scm_ints_disabled = 1; \
-  SCM_FENCE; \
-} while (0)
-
-
-#define SCM_ALLOW_INTS_ONLY \
-do { \
-  SCM_CRITICAL_SECTION_END; \
-  scm_ints_disabled = 0; \
-} while (0)
-
-
-#define SCM_ALLOW_INTS \
-do { \
-  SCM_FENCE; \
-  SCM_CHECK_NOT_ENABLED; \
-  SCM_CRITICAL_SECTION_END; \
-  SCM_FENCE; \
-  scm_ints_disabled = 0; \
-  SCM_FENCE; \
-  SCM_THREAD_SWITCHING_CODE; \
-  SCM_FENCE; \
-} while (0)
-
-
-#define SCM_REDEFER_INTS  \
-do { \
-  SCM_FENCE; \
-  SCM_CRITICAL_SECTION_START; \
-  ++scm_ints_disabled; \
-  SCM_FENCE; \
-} while (0)
-
-
-#define SCM_REALLOW_INTS \
-do { \
-  SCM_FENCE; \
-  SCM_CRITICAL_SECTION_END; \
-  SCM_FENCE; \
-  --scm_ints_disabled; \
-  SCM_FENCE; \
-} while (0)
-
-
 #define SCM_TICK \
 do { \
   SCM_ASYNC_TICK; \
@@ -506,41 +490,8 @@ do { \
 
 
 
-/* Classification of critical sections
- *
- * When Guile moves to POSIX threads, it won't be possible to prevent
- * context switching.  In fact, the whole idea of context switching is
- * bogus if threads are run by different processors.  Therefore, we
- * must ultimately eliminate all critical sections or enforce them by
- * use of mutecis.
- *
- * All instances of SCM_DEFER_INTS and SCM_ALLOW_INTS should therefore
- * be classified and replaced by one of the delimiters below.  If you
- * understand what this is all about, I'd like to encourage you to
- * help with this task.  The set of classes below must of course be
- * incrementally augmented.
- *
- * MDJ 980419 <djurfeldt@nada.kth.se>
- */
-
-/* A sections
- *
- * Allocation of a cell with type tag in the CAR.
- *
- * With POSIX threads, each thread will have a private pool of free
- * cells.  Therefore, this type of section can be removed.  But!  It
- * is important that the CDR is initialized first (with the CAR still
- * indicating a free cell) so that we can guarantee a consistent heap
- * at all times.
- */
-
-#define SCM_ENTER_A_SECTION SCM_CRITICAL_SECTION_START
-#define SCM_EXIT_A_SECTION SCM_CRITICAL_SECTION_END
-
-
-
 /** SCM_ASSERT
- ** 
+ **
  **/
 
 
@@ -550,14 +501,14 @@ do { \
 #define SCM_ASRTGO(_cond, _label)
 #else
 #define SCM_ASSERT(_cond, _arg, _pos, _subr) \
-	if (!(_cond)) \
-          scm_wrong_type_arg (_subr, _pos, _arg)
+	do { if (!(_cond)) \
+          scm_wrong_type_arg (_subr, _pos, _arg); } while (0)
 #define SCM_ASSERT_TYPE(_cond, _arg, _pos, _subr, _msg) \
-	if (!(_cond)) \
-          scm_wrong_type_arg_msg(_subr, _pos, _arg, _msg)
+	do { if (!(_cond)) \
+          scm_wrong_type_arg_msg(_subr, _pos, _arg, _msg);  } while (0)
 #define SCM_ASRTGO(_cond, _label) \
-        if (!(_cond)) \
-          goto _label
+        do {  if (!(_cond)) \
+          goto _label; } while (0)
 #endif
 
 /*
@@ -569,7 +520,7 @@ do { \
  * like SCM_BOOL_F or SCM_UNDEFINED was chosen.
  */
 
-extern SCM scm_call_generic_0 (SCM gf);
+SCM_API SCM scm_call_generic_0 (SCM gf);
 
 #define SCM_WTA_DISPATCH_0(gf, subr)			        \
   return (SCM_UNPACK (gf)					\
@@ -578,7 +529,7 @@ extern SCM scm_call_generic_0 (SCM gf);
 #define SCM_GASSERT0(cond, gf, subr) \
   if (!(cond)) SCM_WTA_DISPATCH_0((gf), (subr))
 
-extern SCM scm_call_generic_1 (SCM gf, SCM a1);
+SCM_API SCM scm_call_generic_1 (SCM gf, SCM a1);
 
 #define SCM_WTA_DISPATCH_1(gf, a1, pos, subr)			\
   return (SCM_UNPACK (gf)					\
@@ -587,7 +538,7 @@ extern SCM scm_call_generic_1 (SCM gf, SCM a1);
 #define SCM_GASSERT1(cond, gf, a1, pos, subr) \
   if (!(cond)) SCM_WTA_DISPATCH_1((gf), (a1), (pos), (subr))
 
-extern SCM scm_call_generic_2 (SCM gf, SCM a1, SCM a2);
+SCM_API SCM scm_call_generic_2 (SCM gf, SCM a1, SCM a2);
 
 #define SCM_WTA_DISPATCH_2(gf, a1, a2, pos, subr)			\
   return (SCM_UNPACK (gf)						\
@@ -598,14 +549,14 @@ extern SCM scm_call_generic_2 (SCM gf, SCM a1, SCM a2);
 #define SCM_GASSERT2(cond, gf, a1, a2, pos, subr) \
   if (!(cond)) SCM_WTA_DISPATCH_2((gf), (a1), (a2), (pos), (subr))
 
-extern SCM scm_apply_generic (SCM gf, SCM args);
+SCM_API SCM scm_apply_generic (SCM gf, SCM args);
 
 #define SCM_WTA_DISPATCH_n(gf, args, pos, subr)				  \
   return (SCM_UNPACK (gf)						  \
 	  ? scm_apply_generic ((gf), (args))				  \
 	  : (scm_wrong_type_arg ((subr), (pos),				  \
 				 scm_list_ref ((args),			  \
-					       SCM_MAKINUM ((pos) - 1))), \
+					       scm_from_int ((pos) - 1))), \
 	     SCM_UNSPECIFIED))
 #define SCM_GASSERTn(cond, gf, args, pos, subr) \
   if (!(cond)) SCM_WTA_DISPATCH_n((gf), (args), (pos), (subr))
@@ -626,32 +577,7 @@ extern SCM scm_apply_generic (SCM gf, SCM args);
 #define SCM_ARG4 		4
 #define SCM_ARG5 		5
 #define SCM_ARG6 		6
-#define SCM_ARG7 		7 
-
-#if (SCM_DEBUG_DEPRECATED == 0)
-
-/* Use SCM_WRONG_NUM_ARGS instead of: */
-#define SCM_WNA 		8
-
-/* Use SCM_ASSERT_RANGE or SCM_VALIDATE_XXX_RANGE instead of: */
-#define SCM_OUTOFRANGE         10
-
-/* Use scm_memory_error instead of: */
-#define SCM_NALLOC             11
-
-#define SCM_HUP_SIGNAL         14
-#define SCM_INT_SIGNAL         15
-#define SCM_FPE_SIGNAL         16
-#define SCM_BUS_SIGNAL         17
-#define SCM_SEGV_SIGNAL        18
-#define SCM_ALRM_SIGNAL        19
-#define SCM_GC_SIGNAL          20
-#define SCM_TICK_SIGNAL        21
-#define SCM_SIG_ORD(X)         ((X) - SCM_HUP_SIGNAL)
-#define SCM_ORD_SIG(X)         ((X) + SCM_HUP_SIGNAL)
-#define SCM_NUM_SIGS           (SCM_SIG_ORD (SCM_TICK_SIGNAL) + 1)
-
-#endif  /* SCM_DEBUG_DEPRECATED == 0 */
+#define SCM_ARG7 		7
 
 #endif /* SCM_MAGIC_SNARFER */
 
@@ -678,7 +604,16 @@ extern SCM scm_apply_generic (SCM gf, SCM args);
 #endif /* def vms */
 #endif /* ndef SCM_EXIT_FAILURE */
 
-
+/* Define SCM_C_INLINE_KEYWORD so that it can be used as a replacement
+   for the "inline" keyword, expanding to nothing when "inline" is not
+   available.
+*/
+
+#ifdef SCM_C_INLINE
+#define SCM_C_INLINE_KEYWORD SCM_C_INLINE
+#else
+#define SCM_C_INLINE_KEYWORD
+#endif
 
 #endif  /* SCM___SCM_H */
 
