@@ -1,43 +1,19 @@
 /*	Copyright (C) 1995,1996,1998, 2000, 2001, 2004 Free Software Foundation, Inc.
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- * 
- * This program is distributed in the hope that it will be useful,
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301 USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * As a special exception, the Free Software Foundation gives permission
- * for additional uses of the text contained in its release of GUILE.
- *
- * The exception is that, if you link the GUILE library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the GUILE library code into it.
- *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by the
- * Free Software Foundation under the name GUILE.  If you copy
- * code from other Free Software Foundation releases into a copy of
- * GUILE, as the General Public License permits, the exception does
- * not apply to the code that you add in this way.  To avoid misleading
- * anyone as to the status of such modified files, you must delete
- * this exception notice from them.
- *
- * If you write modifications of your own for GUILE, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.  */
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ */
 
 
 
@@ -48,6 +24,8 @@
 #include "libguile/validate.h"
 
 #include "libguile/chars.h"
+#include "libguile/srfi-14.h"
+
 
 
 SCM_DEFINE (scm_char_p, "char?", 1, 0, 0, 
@@ -55,7 +33,7 @@ SCM_DEFINE (scm_char_p, "char?", 1, 0, 0,
 	    "Return @code{#t} iff @var{x} is a character, else @code{#f}.")
 #define FUNC_NAME s_scm_char_p
 {
-  return SCM_BOOL(SCM_CHARP(x));
+  return scm_from_bool (SCM_CHARP(x));
 }
 #undef FUNC_NAME
 
@@ -66,7 +44,7 @@ SCM_DEFINE1 (scm_char_eq_p, "char=?", scm_tc7_rpsubr,
 {
   SCM_VALIDATE_CHAR (1, x);
   SCM_VALIDATE_CHAR (2, y);
-  return SCM_BOOL (SCM_EQ_P (x, y));
+  return scm_from_bool (scm_is_eq (x, y));
 }
 #undef FUNC_NAME
 
@@ -77,9 +55,9 @@ SCM_DEFINE1 (scm_char_less_p, "char<?", scm_tc7_rpsubr,
 	     "else @code{#f}.")
 #define FUNC_NAME s_scm_char_less_p
 {
-  SCM_VALIDATE_CHAR (1,x);
-  SCM_VALIDATE_CHAR (2,y);
-  return SCM_BOOL(SCM_CHAR(x) < SCM_CHAR(y));
+  SCM_VALIDATE_CHAR (1, x);
+  SCM_VALIDATE_CHAR (2, y);
+  return scm_from_bool (SCM_CHAR(x) < SCM_CHAR(y));
 }
 #undef FUNC_NAME
 
@@ -89,9 +67,9 @@ SCM_DEFINE1 (scm_char_leq_p, "char<=?", scm_tc7_rpsubr,
 	     "ASCII sequence, else @code{#f}.")
 #define FUNC_NAME s_scm_char_leq_p
 {
-  SCM_VALIDATE_CHAR (1,x);
-  SCM_VALIDATE_CHAR (2,y);
-  return SCM_BOOL(SCM_CHAR(x) <= SCM_CHAR(y));
+  SCM_VALIDATE_CHAR (1, x);
+  SCM_VALIDATE_CHAR (2, y);
+  return scm_from_bool (SCM_CHAR(x) <= SCM_CHAR(y));
 }
 #undef FUNC_NAME
 
@@ -101,9 +79,9 @@ SCM_DEFINE1 (scm_char_gr_p, "char>?", scm_tc7_rpsubr,
 	     "sequence, else @code{#f}.")
 #define FUNC_NAME s_scm_char_gr_p
 {
-  SCM_VALIDATE_CHAR (1,x);
-  SCM_VALIDATE_CHAR (2,y);
-  return SCM_BOOL(SCM_CHAR(x) > SCM_CHAR(y));
+  SCM_VALIDATE_CHAR (1, x);
+  SCM_VALIDATE_CHAR (2, y);
+  return scm_from_bool (SCM_CHAR(x) > SCM_CHAR(y));
 }
 #undef FUNC_NAME
 
@@ -113,9 +91,9 @@ SCM_DEFINE1 (scm_char_geq_p, "char>=?", scm_tc7_rpsubr,
 	     "ASCII sequence, else @code{#f}.")
 #define FUNC_NAME s_scm_char_geq_p
 {
-  SCM_VALIDATE_CHAR (1,x);
-  SCM_VALIDATE_CHAR (2,y);
-  return SCM_BOOL(SCM_CHAR(x) >= SCM_CHAR(y));
+  SCM_VALIDATE_CHAR (1, x);
+  SCM_VALIDATE_CHAR (2, y);
+  return scm_from_bool (SCM_CHAR(x) >= SCM_CHAR(y));
 }
 #undef FUNC_NAME
 
@@ -125,9 +103,9 @@ SCM_DEFINE1 (scm_char_ci_eq_p, "char-ci=?", scm_tc7_rpsubr,
 	     "case, else @code{#f}.")
 #define FUNC_NAME s_scm_char_ci_eq_p
 {
-  SCM_VALIDATE_CHAR (1,x);
-  SCM_VALIDATE_CHAR (2,y);
-  return SCM_BOOL(scm_upcase(SCM_CHAR(x))==scm_upcase(SCM_CHAR(y)));
+  SCM_VALIDATE_CHAR (1, x);
+  SCM_VALIDATE_CHAR (2, y);
+  return scm_from_bool (scm_c_upcase(SCM_CHAR(x))==scm_c_upcase(SCM_CHAR(y)));
 }
 #undef FUNC_NAME
 
@@ -137,9 +115,9 @@ SCM_DEFINE1 (scm_char_ci_less_p, "char-ci<?", scm_tc7_rpsubr,
 	     "ignoring case, else @code{#f}.")
 #define FUNC_NAME s_scm_char_ci_less_p
 {
-  SCM_VALIDATE_CHAR (1,x);
-  SCM_VALIDATE_CHAR (2,y);
-  return SCM_BOOL((scm_upcase(SCM_CHAR(x))) < scm_upcase(SCM_CHAR(y)));
+  SCM_VALIDATE_CHAR (1, x);
+  SCM_VALIDATE_CHAR (2, y);
+  return scm_from_bool ((scm_c_upcase(SCM_CHAR(x))) < scm_c_upcase(SCM_CHAR(y)));
 }
 #undef FUNC_NAME
 
@@ -149,9 +127,9 @@ SCM_DEFINE1 (scm_char_ci_leq_p, "char-ci<=?", scm_tc7_rpsubr,
 	     "ASCII sequence ignoring case, else @code{#f}.")
 #define FUNC_NAME s_scm_char_ci_leq_p
 {
-  SCM_VALIDATE_CHAR (1,x);
-  SCM_VALIDATE_CHAR (2,y);
-  return SCM_BOOL(scm_upcase(SCM_CHAR(x)) <= scm_upcase(SCM_CHAR(y)));
+  SCM_VALIDATE_CHAR (1, x);
+  SCM_VALIDATE_CHAR (2, y);
+  return scm_from_bool (scm_c_upcase(SCM_CHAR(x)) <= scm_c_upcase(SCM_CHAR(y)));
 }
 #undef FUNC_NAME
 
@@ -161,9 +139,9 @@ SCM_DEFINE1 (scm_char_ci_gr_p, "char-ci>?", scm_tc7_rpsubr,
 	     "sequence ignoring case, else @code{#f}.")
 #define FUNC_NAME s_scm_char_ci_gr_p
 {
-  SCM_VALIDATE_CHAR (1,x);
-  SCM_VALIDATE_CHAR (2,y);
-  return SCM_BOOL(scm_upcase(SCM_CHAR(x)) > scm_upcase(SCM_CHAR(y)));
+  SCM_VALIDATE_CHAR (1, x);
+  SCM_VALIDATE_CHAR (2, y);
+  return scm_from_bool (scm_c_upcase(SCM_CHAR(x)) > scm_c_upcase(SCM_CHAR(y)));
 }
 #undef FUNC_NAME
 
@@ -173,43 +151,37 @@ SCM_DEFINE1 (scm_char_ci_geq_p, "char-ci>=?", scm_tc7_rpsubr,
 	     "ASCII sequence ignoring case, else @code{#f}.")
 #define FUNC_NAME s_scm_char_ci_geq_p
 {
-  SCM_VALIDATE_CHAR (1,x);
-  SCM_VALIDATE_CHAR (2,y);
-  return SCM_BOOL(scm_upcase(SCM_CHAR(x)) >= scm_upcase(SCM_CHAR(y)));
+  SCM_VALIDATE_CHAR (1, x);
+  SCM_VALIDATE_CHAR (2, y);
+  return scm_from_bool (scm_c_upcase(SCM_CHAR(x)) >= scm_c_upcase(SCM_CHAR(y)));
 }
 #undef FUNC_NAME
 
 
 SCM_DEFINE (scm_char_alphabetic_p, "char-alphabetic?", 1, 0, 0,
            (SCM chr),
-	    "Return @code{#t} iff @var{chr} is alphabetic, else @code{#f}.\n"
-	    "Alphabetic means the same thing as the isalpha C library function.")
+	    "Return @code{#t} iff @var{chr} is alphabetic, else @code{#f}.\n")
 #define FUNC_NAME s_scm_char_alphabetic_p
 {
-  SCM_VALIDATE_CHAR (1,chr);
-  return SCM_BOOL(isalpha(SCM_CHAR(chr)));
+  return scm_char_set_contains_p (scm_char_set_letter, chr);
 }
 #undef FUNC_NAME
 
 SCM_DEFINE (scm_char_numeric_p, "char-numeric?", 1, 0, 0, 
            (SCM chr),
-	    "Return @code{#t} iff @var{chr} is numeric, else @code{#f}.\n"
-	    "Numeric means the same thing as the isdigit C library function.")
+	    "Return @code{#t} iff @var{chr} is numeric, else @code{#f}.\n")
 #define FUNC_NAME s_scm_char_numeric_p
 {
-  SCM_VALIDATE_CHAR (1,chr);
-  return SCM_BOOL(isdigit(SCM_CHAR(chr)));
+  return scm_char_set_contains_p (scm_char_set_digit, chr);
 }
 #undef FUNC_NAME
 
 SCM_DEFINE (scm_char_whitespace_p, "char-whitespace?", 1, 0, 0, 
            (SCM chr),
-	    "Return @code{#t} iff @var{chr} is whitespace, else @code{#f}.\n"
-	    "Whitespace means the same thing as the isspace C library function.")
+	    "Return @code{#t} iff @var{chr} is whitespace, else @code{#f}.\n")
 #define FUNC_NAME s_scm_char_whitespace_p
 {
-  SCM_VALIDATE_CHAR (1,chr);
-  return SCM_BOOL(isspace(SCM_CHAR(chr)));
+  return scm_char_set_contains_p (scm_char_set_whitespace, chr);
 }
 #undef FUNC_NAME
 
@@ -217,24 +189,20 @@ SCM_DEFINE (scm_char_whitespace_p, "char-whitespace?", 1, 0, 0,
 
 SCM_DEFINE (scm_char_upper_case_p, "char-upper-case?", 1, 0, 0, 
            (SCM chr),
-	    "Return @code{#t} iff @var{chr} is uppercase, else @code{#f}.\n"
-	    "Uppercase means the same thing as the isupper C library function.")
+	    "Return @code{#t} iff @var{chr} is uppercase, else @code{#f}.\n")
 #define FUNC_NAME s_scm_char_upper_case_p
 {
-  SCM_VALIDATE_CHAR (1,chr);
-  return SCM_BOOL(isupper(SCM_CHAR(chr)));
+  return scm_char_set_contains_p (scm_char_set_upper_case, chr);
 }
 #undef FUNC_NAME
 
 
 SCM_DEFINE (scm_char_lower_case_p, "char-lower-case?", 1, 0, 0, 
            (SCM chr),
-	    "Return @code{#t} iff @var{chr} is lowercase, else @code{#f}.\n"
-	    "Lowercase means the same thing as the islower C library function.")
+	    "Return @code{#t} iff @var{chr} is lowercase, else @code{#f}.\n")
 #define FUNC_NAME s_scm_char_lower_case_p
 {
-  SCM_VALIDATE_CHAR (1,chr);
-  return SCM_BOOL(islower(SCM_CHAR(chr)));
+  return scm_char_set_contains_p (scm_char_set_lower_case, chr);
 }
 #undef FUNC_NAME
 
@@ -242,13 +210,12 @@ SCM_DEFINE (scm_char_lower_case_p, "char-lower-case?", 1, 0, 0,
 
 SCM_DEFINE (scm_char_is_both_p, "char-is-both?", 1, 0, 0, 
             (SCM chr),
-	    "Return @code{#t} iff @var{chr} is either uppercase or lowercase, else @code{#f}.\n"
-	    "Uppercase and lowercase are as defined by the isupper and islower\n"
-	    "C library functions.")
+	    "Return @code{#t} iff @var{chr} is either uppercase or lowercase, else @code{#f}.\n")
 #define FUNC_NAME s_scm_char_is_both_p
 {
-  SCM_VALIDATE_CHAR (1,chr);
-  return SCM_BOOL((isupper(SCM_CHAR(chr)) || islower(SCM_CHAR(chr))));
+  if (scm_is_true (scm_char_set_contains_p (scm_char_set_lower_case, chr)))
+    return SCM_BOOL_T;
+  return scm_char_set_contains_p (scm_char_set_upper_case, chr);
 }
 #undef FUNC_NAME
 
@@ -261,8 +228,8 @@ SCM_DEFINE (scm_char_to_integer, "char->integer", 1, 0, 0,
 	    "ASCII sequence.")
 #define FUNC_NAME s_scm_char_to_integer
 {
-  SCM_VALIDATE_CHAR (1,chr);
-  return scm_ulong2num((unsigned long)SCM_CHAR(chr));
+  SCM_VALIDATE_CHAR (1, chr);
+  return scm_from_ulong (SCM_CHAR(chr));
 }
 #undef FUNC_NAME
 
@@ -273,8 +240,7 @@ SCM_DEFINE (scm_integer_to_char, "integer->char", 1, 0, 0,
 	    "Return the character at position @var{n} in the ASCII sequence.")
 #define FUNC_NAME s_scm_integer_to_char
 {
-  SCM_VALIDATE_INUM_RANGE (1, n, 0, 256);
-  return SCM_MAKE_CHAR (SCM_INUM (n));
+  return SCM_MAKE_CHAR (scm_to_uchar (n));
 }
 #undef FUNC_NAME
 
@@ -284,8 +250,8 @@ SCM_DEFINE (scm_char_upcase, "char-upcase", 1, 0, 0,
 	    "Return the uppercase character version of @var{chr}.")
 #define FUNC_NAME s_scm_char_upcase
 {
-  SCM_VALIDATE_CHAR (1,chr);
-  return SCM_MAKE_CHAR(toupper(SCM_CHAR(chr)));
+  SCM_VALIDATE_CHAR (1, chr);
+  return SCM_MAKE_CHAR (toupper (SCM_CHAR (chr)));
 }
 #undef FUNC_NAME
 
@@ -295,16 +261,22 @@ SCM_DEFINE (scm_char_downcase, "char-downcase", 1, 0, 0,
 	    "Return the lowercase character version of @var{chr}.")
 #define FUNC_NAME s_scm_char_downcase
 {
-  SCM_VALIDATE_CHAR (1,chr);
-  return SCM_MAKE_CHAR(tolower(SCM_CHAR(chr)));
+  SCM_VALIDATE_CHAR (1, chr);
+  return SCM_MAKE_CHAR (tolower (SCM_CHAR(chr)));
 }
 #undef FUNC_NAME
 
 
 
 
+
+/*
+TODO: change name  to scm_i_.. ? --hwn
+*/
+
+
 int
-scm_upcase (unsigned int c)
+scm_c_upcase (unsigned int c)
 {
   if (c <= UCHAR_MAX)
     return toupper (c);
@@ -314,7 +286,7 @@ scm_upcase (unsigned int c)
 
 
 int
-scm_downcase (unsigned int c)
+scm_c_downcase (unsigned int c)
 {
   if (c <= UCHAR_MAX)
     return tolower (c);
@@ -338,14 +310,14 @@ scm_downcase (unsigned int c)
 #ifdef EBCDIC
 char *const scm_charnames[] =
 {
-  "nul","soh","stx","etx", "pf", "ht", "lc","del",
-   0   , 0   ,"smm", "vt", "ff", "cr", "so", "si",
-  "dle","dc1","dc2","dc3","res", "nl", "bs", "il",
-  "can", "em", "cc", 0   ,"ifs","igs","irs","ius",
-   "ds","sos", "fs", 0   ,"byp", "lf","eob","pre",
-   0   , 0   , "sm", 0   , 0   ,"enq","ack","bel",
-   0   , 0   ,"syn", 0   , "pn", "rs", "uc","eot",
-   0   , 0   , 0   , 0   ,"dc4","nak", 0   ,"sub",
+  "nul", "soh", "stx", "etx", "pf", "ht", "lc", "del",
+   0   , 0   , "smm", "vt", "ff", "cr", "so", "si",
+  "dle", "dc1", "dc2", "dc3", "res", "nl", "bs", "il",
+  "can", "em", "cc", 0   , "ifs", "igs", "irs", "ius",
+   "ds", "sos", "fs", 0   , "byp", "lf", "eob", "pre",
+   0   , 0   , "sm", 0   , 0   , "enq", "ack", "bel",
+   0   , 0   , "syn", 0   , "pn", "rs", "uc", "eot",
+   0   , 0   , 0   , 0   , "dc4", "nak", 0   , "sub",
    "space", scm_s_newline, "tab", "backspace", "return", "page", "null"};
 
 const char scm_charnums[] =
