@@ -19,6 +19,7 @@
 
 
 #include "libguile/_scm.h"
+#include "libguile/async.h"
 #include "libguile/eval.h"
 #include "libguile/list.h"
 #include "libguile/stackchk.h"
@@ -54,7 +55,10 @@ SCM_DEFINE (scm_debug_options, "debug-options-interface", 0, 1, 0,
 #define FUNC_NAME s_scm_debug_options
 {
   SCM ans;
-  SCM_DEFER_INTS;
+
+  scm_frame_begin (0);
+  scm_frame_critical_section (SCM_BOOL_F);
+
   ans = scm_options (setting, scm_debug_opts, SCM_N_DEBUG_OPTIONS, FUNC_NAME);
   if (!(1 <= SCM_N_FRAMES && SCM_N_FRAMES <= SCM_MAX_FRAME_SIZE))
     {
@@ -64,7 +68,8 @@ SCM_DEFINE (scm_debug_options, "debug-options-interface", 0, 1, 0,
   SCM_RESET_DEBUG_MODE;
   scm_stack_checking_enabled_p = SCM_STACK_CHECKING_P;
   scm_debug_eframe_size = 2 * SCM_N_FRAMES;
-  SCM_ALLOW_INTS;
+
+  scm_frame_end ();
   return ans;
 }
 #undef FUNC_NAME
@@ -143,10 +148,10 @@ scm_make_memoized (SCM exp, SCM env)
 {
   /* *fixme* Check that env is a valid environment. */
   register SCM z, ans;
-  SCM_ENTER_A_SECTION;
+  SCM_CRITICAL_SECTION_START;
   SCM_NEWSMOB (z, SCM_UNPACK (exp), SCM_UNPACK (env));
   SCM_NEWSMOB (ans, scm_tc16_memoized, SCM_UNPACK (z));
-  SCM_EXIT_A_SECTION;
+  SCM_CRITICAL_SECTION_END;
   return ans;
 }
 
@@ -422,18 +427,18 @@ SCM_REGISTER_PROC (s_reverse_lookup, "reverse-lookup", 2, 0, 0, scm_reverse_look
 SCM
 scm_reverse_lookup (SCM env, SCM data)
 {
-  while (SCM_CONSP (env) && SCM_CONSP (SCM_CAR (env)))
+  while (scm_is_pair (env) && scm_is_pair (SCM_CAR (env)))
     {
       SCM names = SCM_CAAR (env);
       SCM values = SCM_CDAR (env);
-      while (SCM_CONSP (names))
+      while (scm_is_pair (names))
 	{
 	  if (scm_is_eq (SCM_CAR (values), data))
 	    return SCM_CAR (names);
 	  names = SCM_CDR (names);
 	  values = SCM_CDR (values);
 	}
-      if (!SCM_NULLP (names) && scm_is_eq (values, data))
+      if (!scm_is_null (names) && scm_is_eq (values, data))
 	return names;
       env = SCM_CDR (env);
     }
@@ -446,13 +451,13 @@ scm_start_stack (SCM id, SCM exp, SCM env)
   SCM answer;
   scm_t_debug_frame vframe;
   scm_t_debug_info vframe_vect_body;
-  vframe.prev = scm_last_debug_frame;
+  vframe.prev = scm_i_last_debug_frame ();
   vframe.status = SCM_VOIDFRAME;
   vframe.vect = &vframe_vect_body;
   vframe.vect[0].id = id;
-  scm_last_debug_frame = &vframe;
+  scm_i_set_last_debug_frame (&vframe);
   answer = scm_i_eval (exp, env);
-  scm_last_debug_frame = vframe.prev;
+  scm_i_set_last_debug_frame (vframe.prev);
   return answer;
 }
 
@@ -463,9 +468,9 @@ scm_m_start_stack (SCM exp, SCM env)
 #define FUNC_NAME s_start_stack
 {
   exp = SCM_CDR (exp);
-  if (!SCM_CONSP (exp) 
-      || !SCM_CONSP (SCM_CDR (exp))
-      || !SCM_NULLP (SCM_CDDR (exp)))
+  if (!scm_is_pair (exp) 
+      || !scm_is_pair (SCM_CDR (exp))
+      || !scm_is_null (SCM_CDDR (exp)))
     SCM_WRONG_NUM_ARGS ();
   return scm_start_stack (scm_eval_car (exp, env), SCM_CADR (exp), env);
 }
