@@ -20,7 +20,9 @@
  */
 
 #include "libguile/_scm.h"
+#include "libguile/async.h"
 #include "libguile/deprecated.h"
+#include "libguile/discouraged.h"
 #include "libguile/deprecation.h"
 #include "libguile/snarf.h"
 #include "libguile/validate.h"
@@ -41,6 +43,8 @@
 #include "libguile/strports.h"
 #include "libguile/smob.h"
 #include "libguile/alist.h"
+#include "libguile/keywords.h"
+#include "libguile/feature.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -197,7 +201,7 @@ SCM_DEFINE (scm_clear_registered_modules, "c-clear-registered-modules", 0, 0, 0,
 {
   struct moddata *md1, *md2;
 
-  SCM_DEFER_INTS;
+  SCM_CRITICAL_SECTION_START;
 
   for (md1 = registered_mods; md1; md1 = md2)
     {
@@ -206,7 +210,7 @@ SCM_DEFINE (scm_clear_registered_modules, "c-clear-registered-modules", 0, 0, 0,
     }
   registered_mods = NULL;
 
-  SCM_ALLOW_INTS;
+  SCM_CRITICAL_SECTION_END;
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
@@ -317,7 +321,7 @@ maybe_close_port (void *data, SCM port)
 {
   SCM except = (SCM)data;
 
-  while (!SCM_NULLP (except))
+  while (!scm_is_null (except))
     {
       SCM p = SCM_COERCE_OUTPORT (SCM_CAR (except));
       if (scm_is_eq (p, port))
@@ -341,7 +345,7 @@ SCM_DEFINE (scm_close_all_ports_except, "close-all-ports-except", 0, 0, 1,
   SCM p;
   SCM_VALIDATE_REST_ARGUMENT (ports);
   
-  for (p = ports; !SCM_NULLP (p); p = SCM_CDR (p))
+  for (p = ports; !scm_is_null (p); p = SCM_CDR (p))
     SCM_VALIDATE_OPPORT (SCM_ARG1, SCM_COERCE_OUTPORT (SCM_CAR (p)));
 
   scm_c_port_for_each (maybe_close_port, ports);
@@ -445,7 +449,7 @@ SCM_DEFINE (scm_sloppy_memq, "sloppy-memq", 2, 0, 0,
   scm_c_issue_deprecation_warning
     ("'sloppy-memq' is deprecated.  Use 'memq' instead.");
 
-  for(;  SCM_CONSP (lst);  lst = SCM_CDR(lst))
+  for(;  scm_is_pair (lst);  lst = SCM_CDR(lst))
     {
       if (scm_is_eq (SCM_CAR (lst), x))
 	return lst;
@@ -465,7 +469,7 @@ SCM_DEFINE (scm_sloppy_memv, "sloppy-memv", 2, 0, 0,
   scm_c_issue_deprecation_warning
     ("'sloppy-memv' is deprecated.  Use 'memv' instead.");
 
-  for(;  SCM_CONSP (lst);  lst = SCM_CDR(lst))
+  for(;  scm_is_pair (lst);  lst = SCM_CDR(lst))
     {
       if (! scm_is_false (scm_eqv_p (SCM_CAR (lst), x)))
 	return lst;
@@ -485,7 +489,7 @@ SCM_DEFINE (scm_sloppy_member, "sloppy-member", 2, 0, 0,
   scm_c_issue_deprecation_warning
     ("'sloppy-member' is deprecated.  Use 'member' instead.");
 
-  for(;  SCM_CONSP (lst);  lst = SCM_CDR(lst))
+  for(;  scm_is_pair (lst);  lst = SCM_CDR(lst))
     {
       if (! scm_is_false (scm_equal_p (SCM_CAR (lst), x)))
 	return lst;
@@ -649,6 +653,33 @@ scm_strprint_obj (SCM obj)
   return scm_object_to_string (obj, SCM_UNDEFINED);
 }
 
+char *
+scm_i_object_chars (SCM obj)
+{
+  scm_c_issue_deprecation_warning 
+    ("SCM_CHARS is deprecated.  See the manual for alternatives.");
+  if (SCM_STRINGP (obj))
+    return SCM_STRING_CHARS (obj);
+  if (SCM_SYMBOLP (obj))
+    return SCM_SYMBOL_CHARS (obj);
+  abort ();
+}
+
+long
+scm_i_object_length (SCM obj)
+{
+  scm_c_issue_deprecation_warning 
+    ("SCM_LENGTH is deprecated.  "
+     "Use scm_c_string_length instead, for example, or see the manual.");
+  if (SCM_STRINGP (obj))
+    return SCM_STRING_LENGTH (obj);
+  if (SCM_SYMBOLP (obj))
+    return SCM_SYMBOL_LENGTH (obj);
+  if (SCM_VECTORP (obj))
+    return SCM_VECTOR_LENGTH (obj);
+  abort ();
+}
+
 SCM 
 scm_sym2ovcell_soft (SCM sym, SCM obarray)
 {
@@ -658,7 +689,7 @@ scm_sym2ovcell_soft (SCM sym, SCM obarray)
   scm_c_issue_deprecation_warning ("`scm_sym2ovcell_soft' is deprecated. "
 				   "Use hashtables instead.");
 
-  SCM_REDEFER_INTS;
+  SCM_CRITICAL_SECTION_START;
   for (lsym = SCM_VECTOR_REF (obarray, hash);
        SCM_NIMP (lsym);
        lsym = SCM_CDR (lsym))
@@ -666,11 +697,11 @@ scm_sym2ovcell_soft (SCM sym, SCM obarray)
       z = SCM_CAR (lsym);
       if (scm_is_eq (SCM_CAR (z), sym))
 	{
-	  SCM_REALLOW_INTS;
+	  SCM_CRITICAL_SECTION_END;
 	  return z;
 	}
     }
-  SCM_REALLOW_INTS;
+  SCM_CRITICAL_SECTION_END;
   return SCM_BOOL_F;
 }
 
@@ -843,7 +874,7 @@ SCM_DEFINE (scm_intern_symbol, "intern-symbol", 2, 0, 0,
   SCM_VALIDATE_VECTOR (1,o);
   hval = scm_i_symbol_hash (s) % SCM_VECTOR_LENGTH (o);
   /* If the symbol is already interned, simply return. */
-  SCM_REDEFER_INTS;
+  SCM_CRITICAL_SECTION_START;
   {
     SCM lsym;
     SCM sym;
@@ -854,7 +885,7 @@ SCM_DEFINE (scm_intern_symbol, "intern-symbol", 2, 0, 0,
 	sym = SCM_CAR (lsym);
 	if (scm_is_eq (SCM_CAR (sym), s))
 	  {
-	    SCM_REALLOW_INTS;
+	    SCM_CRITICAL_SECTION_END;
 	    return SCM_UNSPECIFIED;
 	  }
       }
@@ -862,7 +893,7 @@ SCM_DEFINE (scm_intern_symbol, "intern-symbol", 2, 0, 0,
 		    scm_acons (s, SCM_UNDEFINED,
 			       SCM_VECTOR_REF (o, hval)));
   }
-  SCM_REALLOW_INTS;
+  SCM_CRITICAL_SECTION_END;
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
@@ -884,7 +915,7 @@ SCM_DEFINE (scm_unintern_symbol, "unintern-symbol", 2, 0, 0,
     return SCM_BOOL_F;
   SCM_VALIDATE_VECTOR (1,o);
   hval = scm_i_symbol_hash (s) % SCM_VECTOR_LENGTH (o);
-  SCM_DEFER_INTS;
+  SCM_CRITICAL_SECTION_START;
   {
     SCM lsym_follow;
     SCM lsym;
@@ -901,12 +932,12 @@ SCM_DEFINE (scm_unintern_symbol, "unintern-symbol", 2, 0, 0,
 	      SCM_VECTOR_SET (o, hval, lsym);
 	    else
 	      SCM_SETCDR (lsym_follow, SCM_CDR(lsym));
-	    SCM_ALLOW_INTS;
+	    SCM_CRITICAL_SECTION_END;
 	    return SCM_BOOL_T;
 	  }
       }
   }
-  SCM_ALLOW_INTS;
+  SCM_CRITICAL_SECTION_END;
   return SCM_BOOL_F;
 }
 #undef FUNC_NAME
@@ -1056,7 +1087,7 @@ SCM_DEFINE (scm_gentemp, "gentemp", 0, 2, 0,
   if (SCM_UNBNDP (obarray))
     return scm_gensym (prefix);
   else
-    SCM_ASSERT ((SCM_VECTORP (obarray) || SCM_WVECTP (obarray)),
+    SCM_ASSERT ((scm_is_vector (obarray) || SCM_I_WVECTP (obarray)),
 		obarray,
 		SCM_ARG2,
 		FUNC_NAME);
@@ -1079,7 +1110,7 @@ SCM_DEFINE (scm_gentemp, "gentemp", 0, 2, 0,
 #undef FUNC_NAME
 
 SCM
-SCM_MAKINUM (scm_t_signed_bits val)
+scm_i_makinum (scm_t_signed_bits val)
 {
   scm_c_issue_deprecation_warning
     ("SCM_MAKINUM is deprecated.  Use scm_from_int or similar instead.");
@@ -1087,7 +1118,7 @@ SCM_MAKINUM (scm_t_signed_bits val)
 }
 
 int
-SCM_INUMP (SCM obj)
+scm_i_inump (SCM obj)
 {
   scm_c_issue_deprecation_warning
     ("SCM_INUMP is deprecated.  Use scm_is_integer or similar instead.");
@@ -1095,7 +1126,7 @@ SCM_INUMP (SCM obj)
 }
 
 scm_t_signed_bits
-SCM_INUM (SCM obj)
+scm_i_inum (SCM obj)
 {
   scm_c_issue_deprecation_warning
     ("SCM_INUM is deprecated.  Use scm_to_int or similar instead.");
@@ -1176,7 +1207,7 @@ scm_round (double x)
 }
 
 char *
-SCM_SYMBOL_CHARS (SCM sym)
+scm_i_deprecated_symbol_chars (SCM sym)
 {
   scm_c_issue_deprecation_warning
     ("SCM_SYMBOL_CHARS is deprecated.  Use scm_symbol_to_string.");
@@ -1185,13 +1216,229 @@ SCM_SYMBOL_CHARS (SCM sym)
 }
 
 size_t
-SCM_SYMBOL_LENGTH (SCM sym)
+scm_i_deprecated_symbol_length (SCM sym)
 {
   scm_c_issue_deprecation_warning
     ("SCM_SYMBOL_LENGTH is deprecated.  Use scm_symbol_to_string.");
   return scm_i_symbol_length (sym);
 }
 
+int
+scm_i_keywordp (SCM obj)
+{
+  scm_c_issue_deprecation_warning
+    ("SCM_KEYWORDP is deprecated.  Use scm_is_keyword instead.");
+  return scm_is_keyword (obj);
+}
+
+SCM
+scm_i_keywordsym (SCM keyword)
+{
+  scm_c_issue_deprecation_warning
+    ("SCM_KEYWORDSYM is deprecated.  See scm_keyword_to_symbol instead.");
+  return scm_keyword_dash_symbol (keyword);
+}
+
+int
+scm_i_vectorp (SCM x)
+{
+  scm_c_issue_deprecation_warning
+    ("SCM_VECTORP is deprecated.  Use scm_is_vector instead.");
+  return SCM_I_IS_VECTOR (x);
+}
+
+unsigned long
+scm_i_vector_length (SCM x)
+{
+  scm_c_issue_deprecation_warning
+    ("SCM_VECTOR_LENGTH is deprecated.  Use scm_c_vector_length instead.");
+  return SCM_I_VECTOR_LENGTH (x);
+}
+
+const SCM *
+scm_i_velts (SCM x)
+{
+  scm_c_issue_deprecation_warning
+    ("SCM_VELTS is deprecated.  Use scm_vector_elements instead.");
+  return SCM_I_VECTOR_ELTS (x);
+}
+
+SCM *
+scm_i_writable_velts (SCM x)
+{
+  scm_c_issue_deprecation_warning
+    ("SCM_WRITABLE_VELTS is deprecated.  "
+     "Use scm_vector_writable_elements instead.");
+  return SCM_I_VECTOR_WELTS (x);
+}
+
+SCM
+scm_i_vector_ref (SCM x, size_t idx)
+{
+  scm_c_issue_deprecation_warning
+    ("SCM_VECTOR_REF is deprecated.  "
+     "Use scm_c_vector_ref or scm_vector_elements instead.");
+  return scm_c_vector_ref (x, idx);
+}
+
+void
+scm_i_vector_set (SCM x, size_t idx, SCM val)
+{
+  scm_c_issue_deprecation_warning
+    ("SCM_VECTOR_SET is deprecated.  "
+     "Use scm_c_vector_set_x or scm_vector_writable_elements instead.");
+  scm_c_vector_set_x (x, idx, val);
+}
+
+SCM
+scm_vector_equal_p (SCM x, SCM y)
+{
+  scm_c_issue_deprecation_warning
+    ("scm_vector_euqal_p is deprecated.  "
+     "Use scm_equal_p instead.");
+  return scm_equal_p (x, y);
+}
+
+int
+scm_i_arrayp (SCM a)
+{
+  scm_c_issue_deprecation_warning
+    ("SCM_ARRAYP is deprecated.  Use scm_is_array instead.");
+  return SCM_I_ARRAYP(a) || SCM_I_ENCLOSED_ARRAYP(a);
+}
+
+size_t
+scm_i_array_ndim (SCM a)
+{
+  scm_c_issue_deprecation_warning
+    ("SCM_ARRAY_NDIM is deprecated.  "
+     "Use scm_c_array_rank or scm_array_handle_rank instead.");
+  return scm_c_array_rank (a);
+}
+
+int
+scm_i_array_contp (SCM a)
+{
+  scm_c_issue_deprecation_warning
+    ("SCM_ARRAY_CONTP is deprecated.  Do not use it.");
+  return SCM_I_ARRAY_CONTP (a);
+}
+
+scm_t_array *
+scm_i_array_mem (SCM a)
+{
+  scm_c_issue_deprecation_warning
+    ("SCM_ARRAY_MEM is deprecated.  Do not use it.");
+  return (scm_t_array *)SCM_I_ARRAY_MEM (a);
+}
+
+SCM
+scm_i_array_v (SCM a)
+{
+  /* We could use scm_shared_array_root here, but it is better to move
+     them away from expecting vectors as the basic storage for arrays.
+  */
+  scm_c_issue_deprecation_warning
+    ("SCM_ARRAY_V is deprecated.  Do not use it.");
+  return SCM_I_ARRAY_V (a);
+}
+
+size_t
+scm_i_array_base (SCM a)
+{
+  scm_c_issue_deprecation_warning
+    ("SCM_ARRAY_BASE is deprecated.  Do not use it.");
+  return SCM_I_ARRAY_BASE (a);
+}
+
+scm_t_array_dim *
+scm_i_array_dims (SCM a)
+{
+  scm_c_issue_deprecation_warning
+    ("SCM_ARRAY_DIMS is deprecated.  Use scm_array_handle_dims instead.");
+  return SCM_I_ARRAY_DIMS (a);
+}
+
+SCM
+scm_i_cur_inp (void)
+{
+  scm_c_issue_deprecation_warning
+    ("scm_cur_inp is deprecated.  Use scm_current_input_port instead.");
+  return scm_current_input_port ();
+}
+
+SCM
+scm_i_cur_outp (void)
+{
+  scm_c_issue_deprecation_warning
+    ("scm_cur_outp is deprecated.  Use scm_current_output_port instead.");
+  return scm_current_output_port ();
+}
+
+SCM
+scm_i_cur_errp (void)
+{
+  scm_c_issue_deprecation_warning
+    ("scm_cur_errp is deprecated.  Use scm_current_error_port instead.");
+  return scm_current_error_port ();
+}
+
+SCM
+scm_i_cur_loadp (void)
+{
+  scm_c_issue_deprecation_warning
+    ("scm_cur_loadp is deprecated.  Use scm_current_load_port instead.");
+  return scm_current_load_port ();
+}
+
+SCM
+scm_i_progargs (void)
+{
+  scm_c_issue_deprecation_warning
+    ("scm_progargs is deprecated.  Use scm_program_arguments instead.");
+  return scm_program_arguments ();
+}
+
+SCM
+scm_i_deprecated_dynwinds (void)
+{
+  scm_c_issue_deprecation_warning
+    ("scm_dynwinds is deprecated.  Do not use it.");
+  return scm_i_dynwinds ();
+}
+
+scm_t_debug_frame *
+scm_i_deprecated_last_debug_frame (void)
+{
+  scm_c_issue_deprecation_warning
+    ("scm_last_debug_frame is deprecated.  Do not use it.");
+  return scm_i_last_debug_frame ();
+}
+
+SCM_STACKITEM *
+scm_i_stack_base (void)
+{
+  scm_c_issue_deprecation_warning
+    ("scm_stack_base is deprecated.  Do not use it.");
+  return SCM_I_CURRENT_THREAD->base;
+}
+
+int
+scm_i_fluidp (SCM x)
+{
+  scm_c_issue_deprecation_warning
+    ("SCM_FLUIDP is deprecated.  Use scm_is_fluid instead.");
+  return scm_is_fluid (x);
+}
+
+void
+scm_i_defer_ints_etc ()
+{
+  scm_c_issue_deprecation_warning
+    ("SCM_CRITICAL_SECTION_START etc are deprecated.  "
+     "Use a mutex instead if appropriate.");
+}
+
 void
 scm_i_init_deprecated ()
 {
