@@ -5,7 +5,7 @@
 #ifndef SCM_DEPRECATED_H
 #define SCM_DEPRECATED_H
 
-/* Copyright (C) 2003,2004 Free Software Foundation, Inc.
+/* Copyright (C) 2003,2004, 2005 Free Software Foundation, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -226,6 +226,15 @@ SCM_API SCM scm_strprint_obj (SCM obj);
 SCM_API SCM scm_read_0str (char *expr);
 SCM_API SCM scm_eval_0str (const char *expr);
 
+SCM_API char *scm_i_object_chars (SCM);
+
+#define SCM_CHARS(x)   scm_i_object_chars(x)
+#define SCM_UCHARS(x)  ((unsigned char *)SCM_CHARS(x))
+
+SCM_API long scm_i_object_length (SCM);
+
+#define SCM_LENGTH(x) scm_i_object_length(x)
+
 #define scm_strhash(str, len, n) (scm_string_hash ((str), (len)) % (n))
 
 SCM_API SCM scm_sym2ovcell_soft (SCM sym, SCM obarray);
@@ -299,10 +308,14 @@ SCM_API SCM scm_gentemp (SCM prefix, SCM obarray);
 /* Users shouldn't know about INUMs.
  */
 
-SCM_API SCM SCM_MAKINUM (scm_t_signed_bits val);
-SCM_API int SCM_INUMP (SCM obj);
-#define SCM_NINUMP(x) (!SCM_INUMP(x))
-SCM_API scm_t_signed_bits SCM_INUM (SCM obj);
+SCM_API SCM scm_i_makinum (scm_t_signed_bits val);
+SCM_API int scm_i_inump (SCM obj);
+SCM_API scm_t_signed_bits scm_i_inum (SCM obj);
+
+#define SCM_MAKINUM(x)   scm_i_makinum(x)
+#define SCM_INUM(x)      scm_i_inum(x)
+#define SCM_INUMP(x)     scm_i_inump(x)
+#define SCM_NINUMP(x)    (!SCM_INUMP(x))
 
 #define SCM_VALIDATE_INUM(pos, k) SCM_MAKE_VALIDATE_MSG (pos, k, INUMP, "exact integer")
 
@@ -444,8 +457,115 @@ SCM_API double scm_round (double x);
    symbols directly.
 */
 
-SCM_API char *SCM_SYMBOL_CHARS (SCM sym);
-SCM_API size_t SCM_SYMBOL_LENGTH (SCM sym);
+SCM_API char *scm_i_deprecated_symbol_chars (SCM sym);
+SCM_API size_t scm_i_deprecated_symbol_length (SCM sym);
+
+#define SCM_SYMBOL_CHARS(x)  scm_i_deprecated_symbol_chars(x)
+#define SCM_SYMBOL_LENGTH(x) scm_i_deprecated_symbol_length(x)
+
+/* Deprecated because the macros used to evaluate the arguments more
+   than once and because the symbol of a keyword now has no dash.
+*/
+
+SCM_API int scm_i_keywordp (SCM obj);
+SCM_API SCM scm_i_keywordsym (SCM keyword);
+
+#define SCM_KEYWORDP(x)   scm_i_keywordp(x)
+#define SCM_KEYWORDSYM(x) scm_i_keywordsym(x)
+
+/* Deprecated because we don't want to hand out unprotected pointers
+   to arrays, vectors, etc. */
+
+#define SCM_VECTOR_MAX_LENGTH ((1L << 24) - 1)
+
+SCM_API int scm_i_vectorp (SCM x);
+SCM_API unsigned long scm_i_vector_length (SCM x);
+SCM_API const SCM *scm_i_velts (SCM x);
+SCM_API SCM *scm_i_writable_velts (SCM x);
+SCM_API SCM scm_i_vector_ref (SCM x, size_t idx);
+SCM_API void scm_i_vector_set (SCM x, size_t idx, SCM val);
+SCM_API SCM scm_vector_equal_p (SCM x, SCM y);
+
+#define SCM_VECTORP(x)         scm_i_vectorp(x)
+#define SCM_VECTOR_LENGTH(x)   scm_i_vector_length(x)
+#define SCM_VELTS(x)           scm_i_velts(x)
+#define SCM_WRITABLE_VELTS(x)  scm_i_writable_velts(x)
+#define SCM_VECTOR_REF(x,y)    scm_i_vector_ref(x,y)
+#define SCM_VECTOR_SET(x,y,z)  scm_i_vector_set(x,y,z)
+
+typedef scm_i_t_array scm_t_array;
+
+SCM_API int scm_i_arrayp (SCM a);
+SCM_API size_t scm_i_array_ndim (SCM a);
+SCM_API int scm_i_array_contp (SCM a);
+SCM_API scm_t_array *scm_i_array_mem (SCM a);
+SCM_API SCM scm_i_array_v (SCM a);
+SCM_API size_t scm_i_array_base (SCM a);
+SCM_API scm_t_array_dim *scm_i_array_dims (SCM a);
+
+#define SCM_ARRAYP(a)      scm_i_arrayp(a)
+#define SCM_ARRAY_NDIM(a)  scm_i_array_ndim(a)
+#define SCM_ARRAY_CONTP(a) scm_i_array_contp(a)
+#define SCM_ARRAY_MEM(a)   scm_i_array_mem(a)
+#define SCM_ARRAY_V(a)     scm_i_array_v(a)
+#define SCM_ARRAY_BASE(a)  scm_i_array_base(a)
+#define SCM_ARRAY_DIMS(a)  scm_i_array_dims(a)
+
+/* Deprecated because they should not be lvalues and we want people to
+   use the official interfaces.
+ */
+
+#define scm_cur_inp           scm_i_cur_inp ()
+#define scm_cur_outp          scm_i_cur_outp ()
+#define scm_cur_errp          scm_i_cur_errp ()
+#define scm_cur_loadp         scm_i_cur_loadp ()
+#define scm_progargs          scm_i_progargs ()
+#define scm_dynwinds          scm_i_deprecated_dynwinds ()
+#define scm_last_debug_frame  scm_i_deprecated_last_debug_frame ()
+#define scm_stack_base        scm_i_stack_base ()
+
+SCM_API SCM scm_i_cur_inp (void);
+SCM_API SCM scm_i_cur_outp (void);
+SCM_API SCM scm_i_cur_errp (void);
+SCM_API SCM scm_i_cur_loadp (void);
+SCM_API SCM scm_i_progargs (void);
+SCM_API SCM scm_i_deprecated_dynwinds (void);
+SCM_API scm_t_debug_frame *scm_i_deprecated_last_debug_frame (void);
+SCM_API SCM_STACKITEM *scm_i_stack_base (void);
+
+/* Deprecated because it evaluates its argument twice.
+ */
+#define SCM_FLUIDP(x) scm_i_fluidp (x)
+SCM_API int scm_i_fluidp (SCM x);
+
+/* In the old days, SCM_CRITICAL_SECTION_START stopped signal handlers from running,
+   since in those days the handler directly ran scheme code, and that had to
+   be avoided when the heap was not in a consistent state etc.  And since
+   the scheme code could do a stack swapping new continuation etc, signals
+   had to be deferred around various C library functions which were not safe
+   or not known to be safe to swap away, which was a lot of stuff.
+
+   These days signals are implemented with asyncs and don't directly run
+   scheme code in the handler, but hold it until an SCM_TICK etc where it
+   will be safe.  This means interrupt protection is not needed and
+   SCM_CRITICAL_SECTION_START / SCM_CRITICAL_SECTION_END is something of an anachronism.
+
+   What past SCM_CRITICAL_SECTION_START usage also did though was indicate code that was
+   not reentrant, ie. could not be reentered by signal handler code.  The
+   present definitions are a mutex lock, affording that reentrancy
+   protection against the new guile 1.8 free-running posix threads.
+
+   One big problem with the present defintions though is that code which
+   throws an error from within a DEFER/ALLOW region will leave the
+   defer_mutex locked and hence hang other threads that attempt to enter a
+   similar DEFER/ALLOW region.
+*/
+
+SCM_API void scm_i_defer_ints_etc (void);
+#define SCM_DEFER_INTS scm_i_defer_ints_etc ()
+#define SCM_ALLOW_INTS scm_i_defer_ints_etc ()
+#define SCM_REDEFER_INTS scm_i_defer_ints_etc ()
+#define SCM_REALLOW_INTS scm_i_defer_ints_etc ()
 
 void scm_i_init_deprecated (void);
 
