@@ -2,44 +2,24 @@
  *
  * Copyright (C) 2001 Free Software Foundation, Inc.
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- * 
- * This program is distributed in the hope that it will be useful,
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301 USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * As a special exception, the Free Software Foundation gives permission
- * for additional uses of the text contained in its release of GUILE.
- *
- * The exception is that, if you link the GUILE library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the GUILE library code into it.
- *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by the
- * Free Software Foundation under the name GUILE.  If you copy
- * code from other Free Software Foundation releases into a copy of
- * GUILE, as the General Public License permits, the exception does
- * not apply to the code that you add in this way.  To avoid misleading
- * anyone as to the status of such modified files, you must delete
- * this exception notice from them.
- *
- * If you write modifications of your own for GUILE, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.  */
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ */
+
+#if HAVE_CONFIG_H
+#  include <config.h>
+#endif
 
 #include <string.h>
 
@@ -47,6 +27,7 @@
 #include "libguile/strings.h"
 #include "libguile/gc.h"
 #include "libguile/dynl.h"
+#include "libguile/dynwind.h"
 
 #include "libguile/extensions.h"
 
@@ -74,13 +55,12 @@ void
 scm_c_register_extension (const char *lib, const char *init,
 			  void (*func) (void *), void *data)
 {
-  extension_t *ext = scm_must_malloc (sizeof(extension_t),
-					   "scm_register_extension");
+  extension_t *ext = scm_malloc (sizeof(extension_t));
   if (lib)
-    ext->lib = scm_must_strdup (lib);
+    ext->lib = scm_strdup (lib);
   else
     ext->lib = NULL;
-  ext->init = scm_must_strdup (init);
+  ext->init = scm_strdup (init);
   ext->func = func;
   ext->data = data;
 
@@ -92,27 +72,37 @@ static void
 load_extension (SCM lib, SCM init)
 {
   /* Search the registry. */
-  {
-    extension_t *ext;
+  if (registered_extensions != NULL)
+    {
+      extension_t *ext;
+      char *clib, *cinit;
 
-    for (ext = registered_extensions; ext; ext = ext->next)
-      if ((ext->lib == NULL || !strcmp (ext->lib, SCM_STRING_CHARS (lib)))
-	  && !strcmp (ext->init, SCM_STRING_CHARS (init)))
-	{
-	  ext->func (ext->data);
-	  return;
-	}
-  }
+      scm_frame_begin (0);
+
+      clib = scm_to_locale_string (lib);
+      scm_frame_free (clib);
+      cinit = scm_to_locale_string (init);
+      scm_frame_free (cinit);
+
+      for (ext = registered_extensions; ext; ext = ext->next)
+	if ((ext->lib == NULL || !strcmp (ext->lib, clib))
+	    && !strcmp (ext->init, cinit))
+	  {
+	    ext->func (ext->data);
+	    break;
+	  }
+
+      scm_frame_end ();
+    }
 
   /* Dynamically link the library. */
-
   scm_dynamic_call (init, scm_dynamic_link (lib));
 }
 
 void
 scm_c_load_extension (const char *lib, const char *init)
 {
-  load_extension (scm_makfrom0str (lib), scm_makfrom0str (init));
+  load_extension (scm_from_locale_string (lib), scm_from_locale_string (init));
 }
 
 SCM_DEFINE (scm_load_extension, "load-extension", 2, 0, 0,
@@ -153,8 +143,6 @@ SCM_DEFINE (scm_load_extension, "load-extension", 2, 0, 0,
 	    "@end lisp")
 #define FUNC_NAME s_scm_load_extension
 {
-  SCM_VALIDATE_STRING (1, lib);
-  SCM_VALIDATE_STRING (2, init);
   load_extension (lib, init);
   return SCM_UNSPECIFIED;
 }
