@@ -62,15 +62,18 @@
 
 scm_t_bits scm_tc16_hashtable;
 
-#define HASHTABLE_SIZE_N 25
-
 static unsigned long hashtable_size[] = {
   31, 61, 113, 223, 443, 883, 1759, 3517, 7027, 14051, 28099, 56197, 112363,
-  224717, 449419, 898823, 1797641, 3595271, 7190537, 14381041, 28762081,
-  57524111, 115048217, 230096423, 460192829 /* larger values can't be
-					       represented as INUMs */
+  224717, 449419, 898823, 1797641, 3595271, 7190537, 14381041
+#if 0
+  /* vectors are currently restricted to 2^24-1 = 16777215 elements. */
+  28762081, 57524111, 115048217, 230096423, 460192829
+  /* larger values can't be represented as INUMs */
+#endif
 };
 
+#define HASHTABLE_SIZE_N (sizeof(hashtable_size)/sizeof(unsigned long))
+
 /* Turn an empty vector hash table into an opaque resizable one. */
 
 static char *s_hashtable = "hashtable";
@@ -91,8 +94,7 @@ make_hash_table (int flags, unsigned long k, const char *func_name) {
        that ourselves in scan_weak_hashtables. */
     vector = scm_i_allocate_weak_vector (flags | SCM_WVECTF_NOSCAN,
 					 scm_from_int (n),
-					 SCM_EOL,
-					 func_name);
+					 SCM_EOL);
   else
     vector = scm_c_make_vector (n, SCM_EOL);
   t = scm_gc_malloc (sizeof (*t), s_hashtable);
@@ -156,28 +158,45 @@ scm_i_rehash (SCM table,
     new_buckets = scm_i_allocate_weak_vector (SCM_HASHTABLE_FLAGS (table)
 					      | SCM_WVECTF_NOSCAN,
 					      scm_from_ulong (new_size),
-					      SCM_EOL,
-					      func_name);
+					      SCM_EOL);
   else
     new_buckets = scm_c_make_vector (new_size, SCM_EOL);
 
-  old_size = SCM_VECTOR_LENGTH (buckets);
+  /* When this is a weak hashtable, running the GC might change it.
+     We need to cope with this while rehashing its elements.  We do
+     this by first installing the new, empty bucket vector and turning
+     the old bucket vector into a regularily scanned weak vector.
+     Then we remove the elements from the old bucket vector and insert
+     them into the new one.
+  */
+
+  SCM_SET_HASHTABLE_VECTOR (table, new_buckets);
+  SCM_SET_HASHTABLE_N_ITEMS (table, 0);
+  if (SCM_HASHTABLE_WEAK_P (table))
+    SCM_I_SET_WVECT_TYPE (buckets, (SCM_HASHTABLE_FLAGS (table)));
+
+  old_size = SCM_SIMPLE_VECTOR_LENGTH (buckets);
   for (i = 0; i < old_size; ++i)
     {
-      SCM ls = SCM_VELTS (buckets)[i], handle;
-      while (!SCM_NULLP (ls))
+      SCM ls, cell, handle;
+
+      ls = SCM_SIMPLE_VECTOR_REF (buckets, i);
+      SCM_SIMPLE_VECTOR_SET (buckets, i, SCM_EOL);
+
+      while (scm_is_pair (ls))
 	{
 	  unsigned long h;
-	  handle = SCM_CAR (ls);
+	  cell = ls;
+	  handle = SCM_CAR (cell);
+	  ls = SCM_CDR (ls);
 	  h = hash_fn (SCM_CAR (handle), new_size, closure);
 	  if (h >= new_size)
 	    scm_out_of_range (func_name, scm_from_ulong (h));
-	  SCM_VECTOR_SET (new_buckets, h,
-			  scm_cons (handle, SCM_VELTS (new_buckets)[h]));
-	  ls = SCM_CDR (ls);
+	  SCM_SETCDR (cell, SCM_SIMPLE_VECTOR_REF (new_buckets, h));
+	  SCM_SIMPLE_VECTOR_SET (new_buckets, h, cell);
+	  SCM_HASHTABLE_INCREMENT (table);
 	}
     }
-  SCM_SET_HASHTABLE_VECTOR (table, new_buckets);
 }
 
 
@@ -193,10 +212,10 @@ hashtable_print (SCM exp, SCM port, scm_print_state *pstate SCM_UNUSED)
   else if (SCM_HASHTABLE_DOUBLY_WEAK_P (exp))
     scm_puts ("doubly-weak-", port);
   scm_puts ("hash-table ", port);
-  scm_intprint ((unsigned long) t->n_items, 10, port);
+  scm_uintprint (t->n_items, 10, port);
   scm_putc ('/', port);
-  scm_intprint ((unsigned long) SCM_VECTOR_LENGTH (SCM_HASHTABLE_VECTOR (exp)),
-		10, port);
+  scm_uintprint (SCM_SIMPLE_VECTOR_LENGTH (SCM_HASHTABLE_VECTOR (exp)),
+		 10, port);
   scm_puts (">", port);
   return 1;
 }
@@ -215,7 +234,7 @@ scan_weak_hashtables (void *dummy1 SCM_UNUSED,
 {
   SCM *next = &weak_hashtables;
   SCM h = *next;
-  while (!SCM_NULLP (h))
+  while (!scm_is_null (h))
     {
       if (!SCM_GC_MARK_P (h))
 	*next = h = SCM_HASHTABLE_NEXT (h);
@@ -228,19 +247,24 @@ scan_weak_hashtables (void *dummy1 SCM_UNUSED,
 	  int check_size_p = 0;
 	  for (i = 0; i < n; ++i)
 	    {
-	      SCM *next_spine = (SCM *) &SCM_HASHTABLE_BUCKETS (h)[i];
-	      for (alist = *next_spine;
-		   !SCM_NULLP (alist);
-		   alist = SCM_CDR (alist))
-		if ((weak_car && UNMARKED_CELL_P (SCM_CAAR (alist)))
-		    || (weak_cdr && UNMARKED_CELL_P (SCM_CDAR (alist))))
-		  {
-		    *next_spine = SCM_CDR (alist);
-		    SCM_HASHTABLE_DECREMENT (h);
-		    check_size_p = 1;
-		  }
-		else
-		  next_spine = SCM_CDRLOC (alist);
+	      SCM *next_spine = NULL;
+	      alist = SCM_HASHTABLE_BUCKET (h, i);
+	      while (scm_is_pair (alist))
+		{
+		  if ((weak_car && UNMARKED_CELL_P (SCM_CAAR (alist)))
+		      || (weak_cdr && UNMARKED_CELL_P (SCM_CDAR (alist))))
+		    {
+		      if (next_spine)
+			*next_spine = SCM_CDR (alist);
+		      else
+			SCM_SET_HASHTABLE_BUCKET (h, i, SCM_CDR (alist));
+		      SCM_HASHTABLE_DECREMENT (h);
+		      check_size_p = 1;
+		    }
+		  else
+		    next_spine = SCM_CDRLOC (alist);
+		  alist = SCM_CDR (alist);
+		}
 	    }
 	  if (check_size_p
 	      && SCM_HASHTABLE_N_ITEMS (h) < SCM_HASHTABLE_LOWER (h))
@@ -266,7 +290,7 @@ rehash_after_gc (void *dummy1 SCM_UNUSED,
 		 void *dummy2 SCM_UNUSED,
 		 void *dummy3 SCM_UNUSED)
 {
-  if (!SCM_NULLP (to_rehash))
+  if (!scm_is_null (to_rehash))
     {
       SCM first = to_rehash, last, h;
       /* important to clear to_rehash here so that we don't get stuck
@@ -282,7 +306,7 @@ rehash_after_gc (void *dummy1 SCM_UNUSED,
 			"rehash_after_gc");
 	  last = h;
 	  h = SCM_HASHTABLE_NEXT (h);
-	} while (!SCM_NULLP (h));
+	} while (!scm_is_null (h));
       /* move tables back to weak_hashtables */
       SCM_SET_HASHTABLE_NEXT (last, weak_hashtables);
       weak_hashtables = first;
@@ -429,12 +453,12 @@ scm_hash_fn_get_handle (SCM table, SCM obj, unsigned long (*hash_fn)(), SCM (*as
     table = SCM_HASHTABLE_VECTOR (table);
   else
     SCM_VALIDATE_VECTOR (1, table);
-  if (SCM_VECTOR_LENGTH (table) == 0)
+  if (SCM_SIMPLE_VECTOR_LENGTH (table) == 0)
     return SCM_BOOL_F;
-  k = hash_fn (obj, SCM_VECTOR_LENGTH (table), closure);
-  if (k >= SCM_VECTOR_LENGTH (table))
+  k = hash_fn (obj, SCM_SIMPLE_VECTOR_LENGTH (table), closure);
+  if (k >= SCM_SIMPLE_VECTOR_LENGTH (table))
     scm_out_of_range ("hash_fn_get_handle", scm_from_ulong (k));
-  h = assoc_fn (obj, SCM_VELTS (table)[k], closure);
+  h = assoc_fn (obj, SCM_SIMPLE_VECTOR_REF (table, k), closure);
   return h;
 }
 #undef FUNC_NAME
@@ -452,25 +476,39 @@ scm_hash_fn_create_handle_x (SCM table, SCM obj, SCM init, unsigned long (*hash_
     buckets = SCM_HASHTABLE_VECTOR (table);
   else
     {
-      SCM_ASSERT (SCM_VECTORP (table),
+      SCM_ASSERT (scm_is_simple_vector (table),
 		  table, SCM_ARG1, "hash_fn_create_handle_x");
       buckets = table;
     }
-  if (SCM_VECTOR_LENGTH (buckets) == 0)
+  if (SCM_SIMPLE_VECTOR_LENGTH (buckets) == 0)
     SCM_MISC_ERROR ("void hashtable", SCM_EOL);
 
-  k = hash_fn (obj, SCM_VECTOR_LENGTH (buckets), closure);
-  if (k >= SCM_VECTOR_LENGTH (buckets))
+  k = hash_fn (obj, SCM_SIMPLE_VECTOR_LENGTH (buckets), closure);
+  if (k >= SCM_SIMPLE_VECTOR_LENGTH (buckets))
     scm_out_of_range ("hash_fn_create_handle_x", scm_from_ulong (k));
-  it = assoc_fn (obj, SCM_VELTS (buckets)[k], closure);
+  it = assoc_fn (obj, SCM_SIMPLE_VECTOR_REF (buckets, k), closure);
   if (scm_is_true (it))
     return it;
   else
     {
-      SCM old_bucket = SCM_VELTS (buckets)[k];
-      SCM new_bucket = scm_acons (obj, init, old_bucket);
-      SCM_VECTOR_SET (buckets, k, new_bucket);
-      if (table != buckets)
+      /* When this is a weak hashtable, running the GC can change it.
+	 Thus, we must allocate the new cells first and can only then
+	 access BUCKETS.  Also, we need to fetch the bucket vector
+	 again since the hashtable might have been rehashed.  This
+	 necessitates a new hash value as well.
+      */
+      SCM new_bucket = scm_acons (obj, init, SCM_EOL);
+      if (!scm_is_eq (table, buckets)
+	  && !scm_is_eq (SCM_HASHTABLE_VECTOR (table), buckets))
+	{
+	  buckets = SCM_HASHTABLE_VECTOR (table);
+	  k = hash_fn (obj, SCM_SIMPLE_VECTOR_LENGTH (buckets), closure);
+	  if (k >= SCM_SIMPLE_VECTOR_LENGTH (buckets))
+	    scm_out_of_range ("hash_fn_create_handle_x", scm_from_ulong (k));
+	}
+      SCM_SETCDR (new_bucket, SCM_SIMPLE_VECTOR_REF (buckets, k));
+      SCM_SIMPLE_VECTOR_SET (buckets, k, new_bucket);
+      if (!scm_is_eq (table, buckets))
 	{
 	  SCM_HASHTABLE_INCREMENT (table);
 	  if (SCM_HASHTABLE_N_ITEMS (table) > SCM_HASHTABLE_UPPER (table))
@@ -487,7 +525,7 @@ scm_hash_fn_ref (SCM table, SCM obj, SCM dflt, unsigned long (*hash_fn)(),
                  SCM (*assoc_fn)(), void * closure)
 {
   SCM it = scm_hash_fn_get_handle (table, obj, hash_fn, assoc_fn, closure);
-  if (SCM_CONSP (it))
+  if (scm_is_pair (it))
     return SCM_CDR (it);
   else
     return dflt;
@@ -522,20 +560,23 @@ scm_hash_fn_remove_x (SCM table, SCM obj, unsigned long (*hash_fn)(), SCM (*asso
     buckets = SCM_HASHTABLE_VECTOR (table);
   else
     {
-      SCM_ASSERT (SCM_VECTORP (table), table, SCM_ARG1, "hash_fn_remove_x");
+      SCM_ASSERT (scm_is_simple_vector (table), table,
+		  SCM_ARG1, "hash_fn_remove_x");
       buckets = table;
     }
-  if (SCM_VECTOR_LENGTH (table) == 0)
+  if (SCM_SIMPLE_VECTOR_LENGTH (table) == 0)
     return SCM_EOL;
 
-  k = hash_fn (obj, SCM_VECTOR_LENGTH (buckets), closure);
-  if (k >= SCM_VECTOR_LENGTH (buckets))
+  k = hash_fn (obj, SCM_SIMPLE_VECTOR_LENGTH (buckets), closure);
+  if (k >= SCM_SIMPLE_VECTOR_LENGTH (buckets))
     scm_out_of_range ("hash_fn_remove_x", scm_from_ulong (k));
-  h = assoc_fn (obj, SCM_VELTS (buckets)[k], closure);
+  h = assoc_fn (obj, SCM_SIMPLE_VECTOR_REF (buckets, k), closure);
   if (scm_is_true (h))
     {
-      SCM_VECTOR_SET (buckets, k, delete_fn (h, SCM_VELTS (buckets)[k]));
-      if (table != buckets)
+      SCM_SIMPLE_VECTOR_SET 
+	(buckets, k,
+	 delete_fn (h, SCM_SIMPLE_VECTOR_REF (buckets, k)));
+      if (!scm_is_eq (table, buckets))
 	{
 	  SCM_HASHTABLE_DECREMENT (table);
 	  if (SCM_HASHTABLE_N_ITEMS (table) < SCM_HASHTABLE_LOWER (table))
@@ -908,16 +949,16 @@ scm_internal_hash_fold (SCM (*fn) (), void *closure, SCM init, SCM table)
   else
     buckets = table;
   
-  n = SCM_VECTOR_LENGTH (buckets);
+  n = SCM_SIMPLE_VECTOR_LENGTH (buckets);
   for (i = 0; i < n; ++i)
     {
-      SCM ls = SCM_VELTS (buckets)[i], handle;
-      while (!SCM_NULLP (ls))
+      SCM ls = SCM_SIMPLE_VECTOR_REF (buckets, i), handle;
+      while (!scm_is_null (ls))
 	{
-	  if (!SCM_CONSP (ls))
+	  if (!scm_is_pair (ls))
 	    scm_wrong_type_arg (s_scm_hash_fold, SCM_ARG3, buckets);
 	  handle = SCM_CAR (ls);
-	  if (!SCM_CONSP (handle))
+	  if (!scm_is_pair (handle))
 	    scm_wrong_type_arg (s_scm_hash_fold, SCM_ARG3, buckets);
 	  result = fn (closure, SCM_CAR (handle), SCM_CDR (handle), result);
 	  ls = SCM_CDR (ls);
@@ -946,16 +987,16 @@ scm_internal_hash_for_each_handle (SCM (*fn) (), void *closure, SCM table)
   else
     buckets = table;
   
-  n = SCM_VECTOR_LENGTH (buckets);
+  n = SCM_SIMPLE_VECTOR_LENGTH (buckets);
   for (i = 0; i < n; ++i)
     {
-      SCM ls = SCM_VELTS (buckets)[i], handle;
-      while (!SCM_NULLP (ls))
+      SCM ls = SCM_SIMPLE_VECTOR_REF (buckets, i), handle;
+      while (!scm_is_null (ls))
 	{
-	  if (!SCM_CONSP (ls))
+	  if (!scm_is_pair (ls))
 	    scm_wrong_type_arg (s_scm_hash_for_each, SCM_ARG3, buckets);
 	  handle = SCM_CAR (ls);
-	  if (!SCM_CONSP (handle))
+	  if (!scm_is_pair (handle))
 	    scm_wrong_type_arg (s_scm_hash_for_each, SCM_ARG3, buckets);
 	  fn (closure, handle);
 	  ls = SCM_CDR (ls);
