@@ -1,47 +1,27 @@
-/* 	Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+/* 	Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2003 Free Software Foundation, Inc.
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- * 
- * This program is distributed in the hope that it will be useful,
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301 USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * As a special exception, the Free Software Foundation gives permission
- * for additional uses of the text contained in its release of GUILE.
- *
- * The exception is that, if you link the GUILE library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the GUILE library code into it.
- *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by the
- * Free Software Foundation under the name GUILE.  If you copy
- * code from other Free Software Foundation releases into a copy of
- * GUILE, as the General Public License permits, the exception does
- * not apply to the code that you add in this way.  To avoid misleading
- * anyone as to the status of such modified files, you must delete
- * this exception notice from them.
- *
- * If you write modifications of your own for GUILE, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.  */
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ */
 
 
 
 
+#if HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 #include <stdio.h>
 #include <errno.h>
 
@@ -52,9 +32,13 @@
 #include "libguile/ports.h"
 #include "libguile/strings.h"
 #include "libguile/validate.h"
+#include "libguile/dynwind.h"
 
 #include <fcntl.h>
 
+#ifdef HAVE_IO_H
+#include <io.h>
+#endif
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -70,7 +54,7 @@ SCM_DEFINE (scm_ftell, "ftell", 1, 0, 0,
 	    "@end lisp")
 #define FUNC_NAME s_scm_ftell
 {
-  return scm_seek (fd_port, SCM_INUM0, SCM_MAKINUM (SEEK_CUR));
+  return scm_seek (fd_port, SCM_INUM0, scm_from_int (SEEK_CUR));
 }
 #undef FUNC_NAME
 
@@ -94,8 +78,8 @@ SCM_DEFINE (scm_redirect_port, "redirect-port", 2, 0, 0,
   old = SCM_COERCE_OUTPORT (old);
   new = SCM_COERCE_OUTPORT (new);
   
-  SCM_VALIDATE_OPFPORT (1,old);
-  SCM_VALIDATE_OPFPORT (2,new);
+  SCM_VALIDATE_OPFPORT (1, old);
+  SCM_VALIDATE_OPFPORT (2, new);
   oldfd = SCM_FPORT_FDES (old);
   fp = SCM_FSTREAM (new);
   newfd = fp->fdes;
@@ -131,11 +115,11 @@ SCM_DEFINE (scm_dup_to_fdes, "dup->fdes", 1, 1, 0,
 
   fd_or_port = SCM_COERCE_OUTPORT (fd_or_port);
 
-  if (SCM_INUMP (fd_or_port))
-    oldfd = SCM_INUM (fd_or_port);
+  if (scm_is_integer (fd_or_port))
+    oldfd = scm_to_int (fd_or_port);
   else
     {
-      SCM_VALIDATE_OPFPORT (1,fd_or_port);
+      SCM_VALIDATE_OPFPORT (1, fd_or_port);
       oldfd = SCM_FPORT_FDES (fd_or_port);
     }
 
@@ -144,11 +128,11 @@ SCM_DEFINE (scm_dup_to_fdes, "dup->fdes", 1, 1, 0,
       newfd = dup (oldfd);
       if (newfd == -1)
 	SCM_SYSERROR;
-      fd = SCM_MAKINUM (newfd);
+      fd = scm_from_int (newfd);
     }
   else
     {
-      SCM_VALIDATE_INUM_COPY (2, fd, newfd);
+      newfd = scm_to_int (fd);
       if (oldfd != newfd)
 	{
 	  scm_evict_ports (newfd);	/* see scsh manual.  */
@@ -178,8 +162,8 @@ SCM_DEFINE (scm_dup2, "dup2", 2, 0, 0,
   int c_newfd;
   int rv;
 
-  SCM_VALIDATE_INUM_COPY (1, oldfd, c_oldfd);
-  SCM_VALIDATE_INUM_COPY (2, newfd, c_newfd);
+  c_oldfd = scm_to_int (oldfd);
+  c_newfd = scm_to_int (newfd);
   rv = dup2 (c_oldfd, c_newfd);
   if (rv == -1)
     SCM_SYSERROR;
@@ -194,8 +178,8 @@ SCM_DEFINE (scm_fileno, "fileno", 1, 0, 0,
 #define FUNC_NAME s_scm_fileno
 {
   port = SCM_COERCE_OUTPORT (port);
-  SCM_VALIDATE_OPFPORT (1,port);
-  return SCM_MAKINUM (SCM_FPORT_FDES (port));
+  SCM_VALIDATE_OPFPORT (1, port);
+  return scm_from_int (SCM_FPORT_FDES (port));
 }
 #undef FUNC_NAME
 
@@ -221,7 +205,7 @@ SCM_DEFINE (scm_isatty_p, "isatty?", 1, 0, 0,
     return SCM_BOOL_F;
   
   rv = isatty (SCM_FPORT_FDES (port));
-  return  SCM_BOOL(rv);
+  return  scm_from_bool(rv);
 }
 #undef FUNC_NAME
 
@@ -235,11 +219,8 @@ SCM_DEFINE (scm_fdopen, "fdopen", 2, 0, 0,
 	    "same as that accepted by @ref{File Ports, open-file}.")
 #define FUNC_NAME s_scm_fdopen
 {
-  SCM_VALIDATE_INUM (1,fdes);
-  SCM_VALIDATE_STRING (2, modes);
-  SCM_STRING_COERCE_0TERMINATION_X (modes);
-
-  return scm_fdes_to_port (SCM_INUM (fdes), SCM_STRING_CHARS (modes), SCM_BOOL_F);
+  return scm_i_fdes_to_port (scm_to_int (fdes),
+			     scm_i_mode_bits (modes), SCM_BOOL_F);
 }
 #undef FUNC_NAME
 
@@ -267,11 +248,10 @@ SCM_DEFINE (scm_primitive_move_to_fdes, "primitive-move->fdes", 2, 0, 0,
 
   port = SCM_COERCE_OUTPORT (port);
 
-  SCM_VALIDATE_OPFPORT (1,port);
-  SCM_VALIDATE_INUM (2,fd);
+  SCM_VALIDATE_OPFPORT (1, port);
   stream = SCM_FSTREAM (port);
   old_fd = stream->fdes;
-  new_fd = SCM_INUM (fd);
+  new_fd = scm_to_int (fd);
   if  (old_fd == new_fd)
     {
       return SCM_BOOL_F;
@@ -297,15 +277,17 @@ SCM_DEFINE (scm_fdes_to_ports, "fdes->ports", 1, 0, 0,
   SCM result = SCM_EOL;
   int int_fd;
   long i;
-  
-  SCM_VALIDATE_INUM_COPY (1,fd,int_fd);
 
-  for (i = 0; i < scm_port_table_size; i++)
+  int_fd = scm_to_int (fd);
+
+  scm_i_scm_pthread_mutex_lock (&scm_i_port_table_mutex);
+  for (i = 0; i < scm_i_port_table_size; i++)
     {
-      if (SCM_OPFPORTP (scm_port_table[i]->port)
-	  && ((scm_t_fport *) scm_port_table[i]->stream)->fdes == int_fd)
-	result = scm_cons (scm_port_table[i]->port, result);
+      if (SCM_OPFPORTP (scm_i_port_table[i]->port)
+	  && ((scm_t_fport *) scm_i_port_table[i]->stream)->fdes == int_fd)
+	result = scm_cons (scm_i_port_table[i]->port, result);
     }
+  scm_i_pthread_mutex_unlock (&scm_i_port_table_mutex);
   return result;
 }
 #undef FUNC_NAME    
