@@ -14,34 +14,15 @@
 
    You should have received a copy of the GNU Library General Public
    License along with the GNU C Library; see the file COPYING.LIB.  If not,
-   write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+   Boston, MA 02111-1307, USA. 
+*/
 
-   As a special exception, the Free Software Foundation gives permission
-   for additional uses of the text contained in its release of GUILE.
+#if HAVE_CONFIG_H
+#  include <config.h>
+#endif
 
-   The exception is that, if you link the GUILE library with other files
-   to produce an executable, this does not by itself cause the
-   resulting executable to be covered by the GNU General Public License.
-   Your use of that executable is in no way restricted on account of
-   linking the GUILE library code into it.
-
-   This exception does not however invalidate any other reasons why
-   the executable file might be covered by the GNU General Public License.
-
-   This exception applies only to the code released by the
-   Free Software Foundation under the name GUILE.  If you copy
-   code from other Free Software Foundation releases into a copy of
-   GUILE, as the General Public License permits, the exception does
-   not apply to the code that you add in this way.  To avoid misleading
-   anyone as to the status of such modified files, you must delete
-   this exception notice from them.
-
-   If you write modifications of your own for GUILE, it is your choice
-   whether to permit this exception to apply to your modifications.
-   If you do not wish that, delete this exception notice.  */
-
-#include "libguile/scmconfig.h"
+#include "libguile/__scm.h"
 
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
@@ -58,18 +39,19 @@
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
-
-/* We need to provide a type for gcc_uint64_t.  */
-#ifdef __GNUC__
-typedef unsigned long long gcc_uint64_t;
-#else
-typedef unsigned long gcc_uint64_t;
+#ifdef __MINGW32__
+#include <process.h>
 #endif
 
 #ifndef TMP_MAX
 #define TMP_MAX 16384
 #endif
 
+/* We provide this prototype to avoid compiler warnings.  If this ever
+   conflicts with a declaration in a system header file, we'll find
+   out, because we should include that header file here.  */
+int mkstemp (char *);
+
 /* Generate a unique temporary file name from TEMPLATE.
 
    TEMPLATE has the form:
@@ -86,7 +68,7 @@ mkstemp (template)
 {
   static const char letters[]
     = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
-  static gcc_uint64_t value;
+  static scm_t_uint64 value;
 #ifdef HAVE_GETTIMEOFDAY
   struct timeval tv;
 #endif
@@ -107,14 +89,14 @@ mkstemp (template)
 #ifdef HAVE_GETTIMEOFDAY
   /* Get some more or less random data.  */
   gettimeofday (&tv, NULL);
-  value += ((gcc_uint64_t) tv.tv_usec << 16) ^ tv.tv_sec ^ getpid ();
+  value += ((scm_t_uint64) tv.tv_usec << 16) ^ tv.tv_sec ^ getpid ();
 #else
   value += getpid ();
 #endif
 
   for (count = 0; count < TMP_MAX; ++count)
     {
-      gcc_uint64_t v = value;
+      scm_t_uint64 v = value;
       int fd;
 
       /* Fill in the random bits.  */
