@@ -22,6 +22,8 @@
 
 
 
+#include <gmp.h>
+
 #include "libguile/__scm.h"
 #include "libguile/print.h"
 
@@ -109,10 +111,12 @@
 #endif /* def FLT_MAX */
 
 
-/* SCM_INTBUFLEN is the maximum number of characters neccessary for the
- * printed or scm_string representation of an exact immediate.
+/* SCM_INTBUFLEN is the maximum number of characters neccessary for
+ * the printed or scm_string representation of an scm_t_intmax in
+ * radix 2.  The buffer passed to scm_iint2str and scm_iuint2str must
+ * be of this size, for example.
  */
-#define SCM_INTBUFLEN (5 + SCM_LONG_BIT)
+#define SCM_INTBUFLEN (5 + SCM_CHAR_BIT*sizeof(scm_t_intmax))
 
 
 
@@ -206,7 +210,8 @@ SCM_API SCM scm_bit_extract (SCM n, SCM start, SCM end);
 SCM_API SCM scm_logcount (SCM n);
 SCM_API SCM scm_integer_length (SCM n);
 
-SCM_API size_t scm_iint2str (long num, int rad, char *p);
+SCM_API size_t scm_iint2str (scm_t_intmax num, int rad, char *p);
+SCM_API size_t scm_iuint2str (scm_t_uintmax num, int rad, char *p);
 SCM_API SCM scm_number_to_string (SCM x, SCM radix);
 SCM_API int scm_print_real (SCM sexp, SCM port, scm_print_state *pstate);
 SCM_API int scm_print_complex (SCM sexp, SCM port, scm_print_state *pstate);
@@ -277,6 +282,10 @@ SCM_API double scm_i_fraction2double (SCM z);
 SCM_API SCM scm_i_fraction_equalp (SCM x, SCM y);
 SCM_API int scm_i_print_fraction (SCM sexp, SCM port, scm_print_state *pstate);
 
+/* general internal functions */
+SCM_API void scm_i_print_double (double val, SCM port);
+SCM_API void scm_i_print_complex (double real, double imag, SCM port);
+
 /* conversion functions for integers */
 
 SCM_API int scm_is_integer (SCM val);
@@ -323,6 +332,10 @@ SCM_API SCM          scm_from_uint64 (scm_t_uint64 x);
 
 #endif
 
+SCM_API void scm_to_mpz (SCM x, mpz_t rop);
+SCM_API SCM  scm_from_mpz (mpz_t rop);
+
+
 /* The conversion functions for other types are aliased to the
    appropriate ones from above.  We pick the right one based on the
    size of the type.
