@@ -38,163 +38,13 @@
 
 #include "libguile/validate.h"
 #include "libguile/objects.h"
+#include "libguile/goops.h"
+
 
 
 SCM scm_metaclass_standard;
 SCM scm_metaclass_operator;
 
-/* These variables are filled in by the object system when loaded. */
-SCM scm_class_boolean, scm_class_char, scm_class_pair;
-SCM scm_class_procedure, scm_class_string, scm_class_symbol;
-SCM scm_class_procedure_with_setter, scm_class_primitive_generic;
-SCM scm_class_vector, scm_class_null;
-SCM scm_class_integer, scm_class_real, scm_class_complex, scm_class_fraction;
-SCM scm_class_unknown;
-
-SCM *scm_port_class = 0;
-SCM *scm_smob_class = 0;
-
-SCM scm_no_applicable_method;
-
-/* This function is used for efficient type dispatch.  */
-SCM_DEFINE (scm_class_of, "class-of", 1, 0, 0,
-	    (SCM x),
-	    "Return the class of @var{x}.")
-#define FUNC_NAME s_scm_class_of
-{
-  switch (SCM_ITAG3 (x))
-    {
-    case scm_tc3_int_1:
-    case scm_tc3_int_2:
-      return scm_class_integer;
-
-    case scm_tc3_imm24:
-      if (SCM_CHARP (x))
-	return scm_class_char;
-      else if (scm_is_bool (x))
-        return scm_class_boolean;
-      else if (SCM_NULLP (x))
-        return scm_class_null;
-      else
-        return scm_class_unknown;
-
-    case scm_tc3_cons:
-      switch (SCM_TYP7 (x))
-	{
-	case scm_tcs_cons_nimcar:
-	  return scm_class_pair;
-	case scm_tcs_closures:
-	  return scm_class_procedure;
-	case scm_tc7_symbol:
-	  return scm_class_symbol;
-	case scm_tc7_vector:
-	case scm_tc7_wvect:
-#if SCM_HAVE_ARRAYS
-	case scm_tc7_bvect:
-	case scm_tc7_byvect:
-	case scm_tc7_svect:
-	case scm_tc7_ivect:
-	case scm_tc7_uvect:
-	case scm_tc7_fvect:
-	case scm_tc7_dvect:
-	case scm_tc7_cvect:
-#endif
-	  return scm_class_vector;
-	case scm_tc7_string:
-	  return scm_class_string;
-        case scm_tc7_number:
-          switch SCM_TYP16 (x) {
-          case scm_tc16_big:
-            return scm_class_integer;
-          case scm_tc16_real:
-            return scm_class_real;
-          case scm_tc16_complex:
-            return scm_class_complex;
-	  case scm_tc16_fraction:
-	    return scm_class_fraction;
-          }
-	case scm_tc7_asubr:
-	case scm_tc7_subr_0:
-	case scm_tc7_subr_1:
-	case scm_tc7_dsubr:
-	case scm_tc7_cxr:
-	case scm_tc7_subr_3:
-	case scm_tc7_subr_2:
-	case scm_tc7_rpsubr:
-	case scm_tc7_subr_1o:
-	case scm_tc7_subr_2o:
-	case scm_tc7_lsubr_2:
-	case scm_tc7_lsubr:
-	  if (SCM_SUBR_GENERIC (x) && *SCM_SUBR_GENERIC (x))
-	    return scm_class_primitive_generic;
-	  else
-	    return scm_class_procedure;
-	case scm_tc7_cclo:
-	  return scm_class_procedure;
-	case scm_tc7_pws:
-	  return scm_class_procedure_with_setter;
-
-	case scm_tc7_smob:
-	  {
-	    scm_t_bits type = SCM_TYP16 (x);
-	    if (type != scm_tc16_port_with_ps)
-	      return scm_smob_class[SCM_TC2SMOBNUM (type)];
-	    x = SCM_PORT_WITH_PS_PORT (x);
-	    /* fall through to ports */
-	  }
-	case scm_tc7_port:
-	  return scm_port_class[(SCM_WRTNG & SCM_CELL_WORD_0 (x)
-				 ? (SCM_RDNG & SCM_CELL_WORD_0 (x)
-				    ? SCM_INOUT_PCLASS_INDEX | SCM_PTOBNUM (x)
-				    : SCM_OUT_PCLASS_INDEX | SCM_PTOBNUM (x))
-				 : SCM_IN_PCLASS_INDEX | SCM_PTOBNUM (x))];
-	case scm_tcs_struct:
-	  if (SCM_OBJ_CLASS_FLAGS (x) & SCM_CLASSF_GOOPS_VALID)
-	    return SCM_CLASS_OF (x);
-	  else if (SCM_OBJ_CLASS_FLAGS (x) & SCM_CLASSF_GOOPS)
-	    {
-	      /* Goops object */
-	      if (! scm_is_false (SCM_OBJ_CLASS_REDEF (x)))
-		scm_change_object_class (x,
-					 SCM_CLASS_OF (x),         /* old */
-					 SCM_OBJ_CLASS_REDEF (x)); /* new */
-	      return SCM_CLASS_OF (x);
-	    }
-	  else
-	    {
-	      /* ordinary struct */
-	      SCM handle = scm_struct_create_handle (SCM_STRUCT_VTABLE (x));
-	      if (scm_is_true (SCM_STRUCT_TABLE_CLASS (SCM_CDR (handle))))
-		return SCM_STRUCT_TABLE_CLASS (SCM_CDR (handle));
-	      else
-		{
-		  SCM name = SCM_STRUCT_TABLE_NAME (SCM_CDR (handle));
-		  SCM class = scm_make_extended_class (scm_is_true (name)
-						       ? scm_i_symbol_chars (name)
-						       : 0,
-						       SCM_I_OPERATORP (x));
-		  SCM_SET_STRUCT_TABLE_CLASS (SCM_CDR (handle), class);
-		  return class;
-		}
-	    }
-	default:
-	  if (SCM_CONSP (x))
-	    return scm_class_pair;
-	  else
-	    return scm_class_unknown;
-	}
-
-    case scm_tc3_struct:
-    case scm_tc3_tc7_1:
-    case scm_tc3_tc7_2:
-    case scm_tc3_closure:
-      /* Never reached */
-      break;
-    }
-  return scm_class_unknown;
-}
-#undef FUNC_NAME
-
 /* The cache argument for scm_mcache_lookup_cmethod has one of two possible
  * formats:
  *
@@ -239,12 +89,12 @@ scm_mcache_lookup_cmethod (SCM cache, SCM args)
   n = scm_to_ulong (SCM_CAR (z)); /* maximum number of specializers */
   methods = SCM_CADR (z);
 
-  if (SCM_VECTORP (methods))
+  if (scm_is_simple_vector (methods))
     {
       /* cache format #1: prepare for linear search */
       mask = -1;
       i = 0;
-      end = SCM_VECTOR_LENGTH (methods);
+      end = SCM_SIMPLE_VECTOR_LENGTH (methods);
     }
   else
     {
@@ -256,14 +106,14 @@ scm_mcache_lookup_cmethod (SCM cache, SCM args)
       methods = SCM_CADR (z);
       i = 0;
       ls = args;
-      if (!SCM_NULLP (ls))
+      if (!scm_is_null (ls))
 	do
 	  {
 	    i += SCM_STRUCT_DATA (scm_class_of (SCM_CAR (ls)))
 		 [scm_si_hashsets + hashset];
 	    ls = SCM_CDR (ls);
 	  }
-	while (j-- && !SCM_NULLP (ls));
+	while (j-- && !scm_is_null (ls));
       i &= mask;
       end = i;
     }
@@ -272,9 +122,9 @@ scm_mcache_lookup_cmethod (SCM cache, SCM args)
   do
     {
       long j = n;
-      z = SCM_VELTS (methods)[i];
+      z = SCM_SIMPLE_VECTOR_REF (methods, i);
       ls = args; /* list of arguments */
-      if (!SCM_NULLP (ls))
+      if (!scm_is_null (ls))
 	do
 	  {
 	    /* More arguments than specifiers => CLASS != ENV */
@@ -283,9 +133,9 @@ scm_mcache_lookup_cmethod (SCM cache, SCM args)
 	    ls = SCM_CDR (ls);
 	    z = SCM_CDR (z);
 	  }
-	while (j-- && !SCM_NULLP (ls));
+	while (j-- && !scm_is_null (ls));
       /* Fewer arguments than specifiers => CAR != ENV */
-      if (SCM_NULLP (SCM_CAR (z)) || SCM_CONSP (SCM_CAR (z)))
+      if (scm_is_null (SCM_CAR (z)) || scm_is_pair (SCM_CAR (z)))
 	return z;
     next_method:
       i = (i + 1) & mask;
