@@ -3,46 +3,23 @@
 #ifndef SCM_OBJECTS_H
 #define SCM_OBJECTS_H
 
-/* Copyright (C) 1996, 1999, 2000, 2001, 2005 Free Software Foundation, Inc.
- * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- * 
- * This program is distributed in the hope that it will be useful,
+/* Copyright (C) 1996,1999,2000,2001, 2003 Free Software Foundation, Inc.
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301 USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * As a special exception, the Free Software Foundation gives permission
- * for additional uses of the text contained in its release of GUILE.
- *
- * The exception is that, if you link the GUILE library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the GUILE library code into it.
- *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by the
- * Free Software Foundation under the name GUILE.  If you copy
- * code from other Free Software Foundation releases into a copy of
- * GUILE, as the General Public License permits, the exception does
- * not apply to the code that you add in this way.  To avoid misleading
- * anyone as to the status of such modified files, you must delete
- * this exception notice from them.
- *
- * If you write modifications of your own for GUILE, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.  */
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ */
+
 
 
 /* This file and objects.c contains those minimal pieces of the Guile
@@ -91,7 +68,7 @@
 	((SCM_OBJ_CLASS_FLAGS (obj) & SCM_CLASSF_ENTITY) != 0)
 #define SCM_ENTITY_PROCEDURE(obj) \
         (SCM_PACK (SCM_STRUCT_DATA (obj) [scm_struct_i_procedure]))
-#define SCM_SET_ENTITY_PROCEDURE(obj,v) \
+#define SCM_SET_ENTITY_PROCEDURE(obj, v) \
         (SCM_STRUCT_DATA (obj) [scm_struct_i_procedure] = SCM_UNPACK (v))
 #define SCM_ENTITY_SETTER(obj) (SCM_PACK (SCM_STRUCT_DATA (obj)[scm_struct_i_setter]))
 #define SCM_SET_ENTITY_SETTER(obj, v) \
@@ -173,8 +150,8 @@ struct scm_metaclass_operator {
 #define SCM_CLASSF_PURE_GENERIC (0x010 << 20)
 #define SCM_CLASSF_GOOPS_VALID  (0x080 << 20)
 #define SCM_CLASSF_GOOPS        (0x100 << 20)
-#define scm_si_redefined         6
-#define scm_si_hashsets          7
+#define scm_si_redefined         5
+#define scm_si_hashsets          6
 #define SCM_CLASS_OF(x)         SCM_STRUCT_VTABLE (x)
 #define SCM_OBJ_CLASS_REDEF(x)  (SCM_PACK (SCM_STRUCT_VTABLE_DATA (x) [scm_si_redefined]))
 
@@ -189,6 +166,8 @@ typedef struct scm_effective_slot_definition {
 #define SCM_ESLOTDEF(x) ((scm_effective_slot_definition *) SCM_CDR (x))
 
 #define SCM_CMETHOD_CODE(cmethod) SCM_CDR (cmethod)
+#define SCM_CMETHOD_FORMALS(cmethod) SCM_CAR (SCM_CMETHOD_CODE (cmethod))
+#define SCM_CMETHOD_BODY(cmethod) SCM_CDR (SCM_CMETHOD_CODE (cmethod))
 #define SCM_CMETHOD_ENV(cmethod)  SCM_CAR (cmethod)
 
 /* Port classes */
@@ -197,51 +176,40 @@ typedef struct scm_effective_slot_definition {
 #define SCM_INOUT_PCLASS_INDEX 0x200
 
 /* Plugin proxy classes for basic types. */
-extern SCM scm_metaclass_standard;
-extern SCM scm_metaclass_operator;
-extern SCM scm_class_boolean, scm_class_char, scm_class_pair;
-extern SCM scm_class_procedure, scm_class_string, scm_class_symbol;
-extern SCM scm_class_procedure_with_setter, scm_class_primitive_generic;
-extern SCM scm_class_vector, scm_class_null;
-extern SCM scm_class_real, scm_class_complex, scm_class_integer;
-extern SCM scm_class_unknown;
-extern SCM *scm_port_class;
-extern SCM *scm_smob_class;
-extern int scm_classes_initialized;
-
-extern SCM scm_no_applicable_method;
+SCM_API SCM scm_metaclass_standard;
+SCM_API SCM scm_metaclass_operator;
 
 /* Goops functions. */
-extern SCM scm_make_extended_class (char *type_name);
-extern void scm_make_port_classes (long ptobnum, char *type_name);
-extern void scm_change_object_class (SCM, SCM, SCM);
-extern SCM scm_memoize_method (SCM x, SCM args);
+SCM_API SCM scm_make_extended_class (char const *type_name, int applicablep);
+SCM_API void scm_i_inherit_applicable (SCM c);
+SCM_API void scm_make_port_classes (long ptobnum, char *type_name);
+SCM_API void scm_change_object_class (SCM, SCM, SCM);
+SCM_API SCM scm_memoize_method (SCM x, SCM args);
 
-extern SCM scm_class_of (SCM obj);
-extern SCM scm_mcache_lookup_cmethod (SCM cache, SCM args);
-extern SCM scm_mcache_compute_cmethod (SCM cache, SCM args);
+SCM_API SCM scm_mcache_lookup_cmethod (SCM cache, SCM args);
+SCM_API SCM scm_mcache_compute_cmethod (SCM cache, SCM args);
 /* The following are declared in __scm.h
-extern SCM scm_call_generic_0 (SCM gf);
-extern SCM scm_call_generic_1 (SCM gf, SCM a1);
-extern SCM scm_call_generic_2 (SCM gf, SCM a1, SCM a2);
-extern SCM scm_apply_generic (SCM gf, SCM args);
+SCM_API SCM scm_call_generic_0 (SCM gf);
+SCM_API SCM scm_call_generic_1 (SCM gf, SCM a1);
+SCM_API SCM scm_call_generic_2 (SCM gf, SCM a1, SCM a2);
+SCM_API SCM scm_apply_generic (SCM gf, SCM args);
 */
-extern SCM scm_call_generic_3 (SCM gf, SCM a1, SCM a2, SCM a3);
-extern SCM scm_entity_p (SCM obj);
-extern SCM scm_operator_p (SCM obj);
-extern SCM scm_valid_object_procedure_p (SCM proc);
-extern SCM scm_set_object_procedure_x (SCM obj, SCM proc);
+SCM_API SCM scm_call_generic_3 (SCM gf, SCM a1, SCM a2, SCM a3);
+SCM_API SCM scm_entity_p (SCM obj);
+SCM_API SCM scm_operator_p (SCM obj);
+SCM_API SCM scm_valid_object_procedure_p (SCM proc);
+SCM_API SCM scm_set_object_procedure_x (SCM obj, SCM proc);
 #ifdef GUILE_DEBUG
-extern SCM scm_object_procedure (SCM obj);
+SCM_API SCM scm_object_procedure (SCM obj);
 #endif
-extern SCM scm_make_class_object (SCM metaclass, SCM layout);
-extern SCM scm_make_subclass_object (SCM c, SCM layout);
+SCM_API SCM scm_make_class_object (SCM metaclass, SCM layout);
+SCM_API SCM scm_make_subclass_object (SCM c, SCM layout);
 
-extern SCM scm_i_make_class_object (SCM metaclass, SCM layout_string,
-				    unsigned long flags);
-extern void scm_init_objects (void);
+SCM_API SCM scm_i_make_class_object (SCM metaclass, SCM layout_string,
+				     unsigned long flags);
+SCM_API void scm_init_objects (void);
 
-#endif /* SCM_OBJECTS_H */
+#endif  /* SCM_OBJECTS_H */
 
 /*
   Local Variables:
