@@ -111,7 +111,7 @@ typedef struct
 
 SCM_API scm_t_port **scm_i_port_table;
 SCM_API long scm_i_port_table_size; /* Number of ports in scm_i_port_table.  */
-SCM_API scm_t_mutex scm_i_port_table_mutex;
+SCM_API scm_i_pthread_mutex_t scm_i_port_table_mutex;
 
 #define SCM_READ_BUFFER_EMPTY_P(c_port) (c_port->read_pos >= c_port->read_end)
 
@@ -157,7 +157,9 @@ SCM_API scm_t_mutex scm_i_port_table_mutex;
 #define SCM_SETREVEALED(x, s)      (SCM_PTAB_ENTRY(x)->revealed = (s))
 
 #define SCM_INCLINE(port)  	{SCM_LINUM (port) += 1; SCM_COL (port) = 0;}
+#define SCM_ZEROCOL(port)  	{SCM_COL (port) = 0;}
 #define SCM_INCCOL(port)  	{SCM_COL (port) += 1;}
+#define SCM_DECCOL(port)  	{if (SCM_COL (port) > 0) SCM_COL (port) -= 1;}
 #define SCM_TABCOL(port)  	{SCM_COL (port) += 8 - SCM_COL (port) % 8;}
 
 
@@ -304,6 +306,7 @@ SCM_API SCM scm_pt_member (SCM member);
 /* internal */
 
 SCM_API long scm_i_mode_bits (SCM modes);
+SCM_API void scm_i_frame_current_load_port (SCM port);
 
 
 #endif  /* SCM_PORTS_H */
