@@ -1,4 +1,4 @@
-/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001, 2002, 2003, 2004 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001, 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -39,6 +39,8 @@
 
 #include "libguile/validate.h"
 #include "libguile/posix.h"
+#include "libguile/i18n.h"
+#include "libguile/threads.h"
 
 
 #ifdef HAVE_STRING_H
@@ -267,7 +269,7 @@ SCM_DEFINE (scm_getgroups, "getgroups", 0, 0, 0,
 
   result = scm_c_make_vector (ngroups, SCM_BOOL_F);
   while (--ngroups >= 0) 
-    SCM_VECTOR_SET (result, ngroups, scm_from_ulong (groups[ngroups]));
+    SCM_SIMPLE_VECTOR_SET (result, ngroups, scm_from_ulong (groups[ngroups]));
 
   free (groups);
   return result;
@@ -294,17 +296,18 @@ SCM_DEFINE (scm_setgroups, "setgroups", 1, 0, 0,
 
   SCM_VALIDATE_VECTOR (SCM_ARG1, group_vec);
 
-  ngroups = SCM_VECTOR_LENGTH (group_vec);
+  ngroups = SCM_SIMPLE_VECTOR_LENGTH (group_vec);
 
   /* validate before allocating, so we don't have to worry about leaks */
   for (i = 0; i < ngroups; i++)
     {
       unsigned long ulong_gid;
       GETGROUPS_T gid;
-      SCM_VALIDATE_ULONG_COPY (1, SCM_VECTOR_REF (group_vec, i), ulong_gid);
+      SCM_VALIDATE_ULONG_COPY (1, SCM_SIMPLE_VECTOR_REF (group_vec, i),
+			       ulong_gid);
       gid = ulong_gid;
       if (gid != ulong_gid)
-	SCM_OUT_OF_RANGE (1, SCM_VECTOR_REF (group_vec, i));
+	SCM_OUT_OF_RANGE (1, SCM_SIMPLE_VECTOR_REF (group_vec, i));
     }
 
   size = ngroups * sizeof (GETGROUPS_T);
@@ -312,7 +315,7 @@ SCM_DEFINE (scm_setgroups, "setgroups", 1, 0, 0,
     SCM_OUT_OF_RANGE (SCM_ARG1, scm_from_int (ngroups));
   groups = scm_malloc (size);
   for(i = 0; i < ngroups; i++)
-    groups [i] = SCM_NUM2ULONG (1, SCM_VECTOR_REF (group_vec, i));
+    groups [i] = SCM_NUM2ULONG (1, SCM_SIMPLE_VECTOR_REF (group_vec, i));
 
   result = setgroups (ngroups, groups);
   save_errno = errno; /* don't let free() touch errno */
@@ -356,19 +359,19 @@ SCM_DEFINE (scm_getpwuid, "getpw", 0, 1, 0,
   if (!entry)
     SCM_MISC_ERROR ("entry not found", SCM_EOL);
 
-  SCM_VECTOR_SET(result, 0, scm_from_locale_string (entry->pw_name));
-  SCM_VECTOR_SET(result, 1, scm_from_locale_string (entry->pw_passwd));
-  SCM_VECTOR_SET(result, 2, scm_from_ulong (entry->pw_uid));
-  SCM_VECTOR_SET(result, 3, scm_from_ulong (entry->pw_gid));
-  SCM_VECTOR_SET(result, 4, scm_from_locale_string (entry->pw_gecos));
+  SCM_SIMPLE_VECTOR_SET(result, 0, scm_from_locale_string (entry->pw_name));
+  SCM_SIMPLE_VECTOR_SET(result, 1, scm_from_locale_string (entry->pw_passwd));
+  SCM_SIMPLE_VECTOR_SET(result, 2, scm_from_ulong (entry->pw_uid));
+  SCM_SIMPLE_VECTOR_SET(result, 3, scm_from_ulong (entry->pw_gid));
+  SCM_SIMPLE_VECTOR_SET(result, 4, scm_from_locale_string (entry->pw_gecos));
   if (!entry->pw_dir)
-    SCM_VECTOR_SET(result, 5, scm_from_locale_string (""));
+    SCM_SIMPLE_VECTOR_SET(result, 5, scm_from_locale_string (""));
   else
-    SCM_VECTOR_SET(result, 5, scm_from_locale_string (entry->pw_dir));
+    SCM_SIMPLE_VECTOR_SET(result, 5, scm_from_locale_string (entry->pw_dir));
   if (!entry->pw_shell)
-    SCM_VECTOR_SET(result, 6, scm_from_locale_string (""));
+    SCM_SIMPLE_VECTOR_SET(result, 6, scm_from_locale_string (""));
   else
-    SCM_VECTOR_SET(result, 6, scm_from_locale_string (entry->pw_shell));
+    SCM_SIMPLE_VECTOR_SET(result, 6, scm_from_locale_string (entry->pw_shell));
   return result;
 }
 #undef FUNC_NAME
@@ -421,10 +424,10 @@ SCM_DEFINE (scm_getgrgid, "getgr", 0, 1, 0,
   if (!entry)
     SCM_SYSERROR;
 
-  SCM_VECTOR_SET(result, 0, scm_from_locale_string (entry->gr_name));
-  SCM_VECTOR_SET(result, 1, scm_from_locale_string (entry->gr_passwd));
-  SCM_VECTOR_SET(result, 2, scm_from_ulong  (entry->gr_gid));
-  SCM_VECTOR_SET(result, 3, scm_makfromstrs (-1, entry->gr_mem));
+  SCM_SIMPLE_VECTOR_SET(result, 0, scm_from_locale_string (entry->gr_name));
+  SCM_SIMPLE_VECTOR_SET(result, 1, scm_from_locale_string (entry->gr_passwd));
+  SCM_SIMPLE_VECTOR_SET(result, 2, scm_from_ulong  (entry->gr_gid));
+  SCM_SIMPLE_VECTOR_SET(result, 3, scm_makfromstrs (-1, entry->gr_mem));
   return result;
 }
 #undef FUNC_NAME
@@ -818,11 +821,11 @@ SCM_DEFINE (scm_ttyname, "ttyname", 1, 0, 0,
     return SCM_BOOL_F;
   fd = SCM_FPORT_FDES (port);
 
-  scm_mutex_lock (&scm_i_misc_mutex);
+  scm_i_scm_pthread_mutex_lock (&scm_i_misc_mutex);
   SCM_SYSCALL (result = ttyname (fd));
   err = errno;
   ret = scm_from_locale_string (result);
-  scm_mutex_unlock (&scm_i_misc_mutex);
+  scm_i_pthread_mutex_unlock (&scm_i_misc_mutex);
 
   if (!result)
     {
@@ -1052,14 +1055,14 @@ SCM_DEFINE (scm_uname, "uname", 0, 0, 0,
   SCM result = scm_c_make_vector (5, SCM_UNSPECIFIED);
   if (uname (&buf) < 0)
     SCM_SYSERROR;
-  SCM_VECTOR_SET(result, 0, scm_from_locale_string (buf.sysname));
-  SCM_VECTOR_SET(result, 1, scm_from_locale_string (buf.nodename));
-  SCM_VECTOR_SET(result, 2, scm_from_locale_string (buf.release));
-  SCM_VECTOR_SET(result, 3, scm_from_locale_string (buf.version));
-  SCM_VECTOR_SET(result, 4, scm_from_locale_string (buf.machine));
+  SCM_SIMPLE_VECTOR_SET(result, 0, scm_from_locale_string (buf.sysname));
+  SCM_SIMPLE_VECTOR_SET(result, 1, scm_from_locale_string (buf.nodename));
+  SCM_SIMPLE_VECTOR_SET(result, 2, scm_from_locale_string (buf.release));
+  SCM_SIMPLE_VECTOR_SET(result, 3, scm_from_locale_string (buf.version));
+  SCM_SIMPLE_VECTOR_SET(result, 4, scm_from_locale_string (buf.machine));
 /* 
    a linux special?
-  SCM_VECTOR_SET(result, 5, scm_from_locale_string (buf.domainname));
+  SCM_SIMPLE_VECTOR_SET(result, 5, scm_from_locale_string (buf.domainname));
 */
   return result;
 }
@@ -1131,9 +1134,14 @@ SCM_DEFINE (scm_mkstemp, "mkstemp!", 1, 0, 0,
 	    (SCM tmpl),
 	    "Create a new unique file in the file system and returns a new\n"
 	    "buffered port open for reading and writing to the file.\n"
+	    "\n"
 	    "@var{tmpl} is a string specifying where the file should be\n"
-	    "created: it must end with @code{XXXXXX} and will be changed in\n"
-	    "place to return the name of the temporary file.")
+	    "created: it must end with @samp{XXXXXX} and will be changed in\n"
+	    "place to return the name of the temporary file.\n"
+	    "\n"
+	    "The file is created with mode @code{0600}, which means read and\n"
+	    "write for the owner only.  @code{chmod} can be used to change\n"
+	    "this.")
 #define FUNC_NAME s_scm_mkstemp
 {
   char *c_tmpl;
@@ -1194,29 +1202,46 @@ SCM_DEFINE (scm_utime, "utime", 1, 2, 0,
 
 SCM_DEFINE (scm_access, "access?", 2, 0, 0,
             (SCM path, SCM how),
-	    "Return @code{#t} if @var{path} corresponds to an existing file\n"
-	    "and the current process has the type of access specified by\n"
-	    "@var{how}, otherwise @code{#f}.  @var{how} should be specified\n"
-	    "using the values of the variables listed below.  Multiple\n"
-	    "values can be combined using a bitwise or, in which case\n"
-	    "@code{#t} will only be returned if all accesses are granted.\n"
+	    "Test accessibility of a file under the real UID and GID of the\n"
+	    "calling process.  The return is @code{#t} if @var{path} exists\n"
+	    "and the permissions requested by @var{how} are all allowed, or\n"
+	    "@code{#f} if not.\n"
 	    "\n"
-	    "Permissions are checked using the real id of the current\n"
-	    "process, not the effective id, although it's the effective id\n"
-	    "which determines whether the access would actually be granted.\n"
+	    "@var{how} is an integer which is one of the following values,\n"
+	    "or a bitwise-OR (@code{logior}) of multiple values.\n"
 	    "\n"
 	    "@defvar R_OK\n"
-	    "test for read permission.\n"
+	    "Test for read permission.\n"
 	    "@end defvar\n"
 	    "@defvar W_OK\n"
-	    "test for write permission.\n"
+	    "Test for write permission.\n"
 	    "@end defvar\n"
 	    "@defvar X_OK\n"
-	    "test for execute permission.\n"
+	    "Test for execute permission.\n"
 	    "@end defvar\n"
 	    "@defvar F_OK\n"
-	    "test for existence of the file.\n"
-	    "@end defvar")
+	    "Test for existence of the file.  This is implied by each of the\n"
+	    "other tests, so there's no need to combine it with them.\n"
+	    "@end defvar\n"
+	    "\n"
+	    "It's important to note that @code{access?} does not simply\n"
+	    "indicate what will happen on attempting to read or write a\n"
+	    "file.  In normal circumstances it does, but in a set-UID or\n"
+	    "set-GID program it doesn't because @code{access?} tests the\n"
+	    "real ID, whereas an open or execute attempt uses the effective\n"
+	    "ID.\n"
+	    "\n"
+	    "A program which will never run set-UID/GID can ignore the\n"
+	    "difference between real and effective IDs, but for maximum\n"
+	    "generality, especially in library functions, it's best not to\n"
+	    "use @code{access?} to predict the result of an open or execute,\n"
+	    "instead simply attempt that and catch any exception.\n"
+	    "\n"
+	    "The main use for @code{access?} is to let a set-UID/GID program\n"
+	    "determine what the invoking user would have been allowed to do,\n"
+	    "without the greater (or perhaps lesser) privileges afforded by\n"
+	    "the effective ID.  For more on this, see ``Testing File\n"
+	    "Access'' in The GNU C Library Reference Manual.")
 #define FUNC_NAME s_scm_access
 {
   int rv;
@@ -1268,7 +1293,7 @@ SCM_DEFINE (scm_putenv, "putenv", 1, 0, 0,
       /* On e.g. Win32 hosts putenv() called with 'name=' removes the
 	 environment variable 'name'. */
       int e;
-      ptr = scm_malloc (len + 2);
+      char *ptr = scm_malloc (len + 2);
       strcpy (ptr, c_str);
       strcpy (ptr+len, "=");
       rv = putenv (ptr);
@@ -1348,9 +1373,15 @@ SCM_DEFINE (scm_setlocale, "setlocale", 1, 1, 0,
       scm_frame_free (clocale);
     }
 
-  rv = setlocale (scm_to_int (category), clocale);
+  rv = setlocale (scm_i_to_lc_category (category, 1), clocale);
   if (rv == NULL)
-    SCM_SYSERROR;
+    {
+      /* POSIX and C99 don't say anything about setlocale setting errno, so
+         force a sensible value here.  glibc leaves ENOENT, which would be
+         fine, but it's not a documented feature.  */
+      errno = EINVAL;
+      SCM_SYSERROR;
+    }
 
   scm_frame_end ();
   return scm_from_locale_string (rv);
@@ -1422,7 +1453,11 @@ SCM_DEFINE (scm_nice, "nice", 1, 0, 0,
 	    "The return value is unspecified.")
 #define FUNC_NAME s_scm_nice
 {
-  if (nice (scm_to_int (incr)) != 0)
+  /* nice() returns "prio-NZERO" on success or -1 on error, but -1 can arise
+     from "prio-NZERO", so an error must be detected from errno changed */
+  errno = 0;
+  nice (scm_to_int (incr));
+  if (errno != 0)
     SCM_SYSERROR;
   return SCM_UNSPECIFIED;
 }
@@ -1471,15 +1506,12 @@ SCM_DEFINE (scm_crypt, "crypt", 2, 0, 0,
   char *c_key, *c_salt;
 
   scm_frame_begin (0);
-  scm_frame_unwind_handler ((void(*)(void*)) scm_mutex_unlock,
-                            &scm_i_misc_mutex,
-                            SCM_F_WIND_EXPLICITLY);
-  scm_mutex_lock (&scm_i_misc_mutex);
+  scm_i_frame_pthread_mutex_lock (&scm_i_misc_mutex);
 
   c_key = scm_to_locale_string (key);
   scm_frame_free (c_key);
   c_salt = scm_to_locale_string (salt);
-  scm_frame_free (c_key);
+  scm_frame_free (c_salt);
 
   ret = scm_from_locale_string (crypt (c_key, c_salt));
 
@@ -1724,21 +1756,28 @@ SCM_DEFINE (scm_flock, "flock", 2, 0, 0,
             (SCM file, SCM operation),
 	    "Apply or remove an advisory lock on an open file.\n"
 	    "@var{operation} specifies the action to be done:\n"
-	    "@table @code\n"
-	    "@item LOCK_SH\n"
+	    "\n"
+	    "@defvar LOCK_SH\n"
 	    "Shared lock.  More than one process may hold a shared lock\n"
 	    "for a given file at a given time.\n"
-	    "@item LOCK_EX\n"
+	    "@end defvar\n"
+	    "@defvar LOCK_EX\n"
 	    "Exclusive lock.  Only one process may hold an exclusive lock\n"
 	    "for a given file at a given time.\n"
-	    "@item LOCK_UN\n"
+	    "@end defvar\n"
+	    "@defvar LOCK_UN\n"
 	    "Unlock the file.\n"
-	    "@item LOCK_NB\n"
-	    "Don't block when locking.  May be specified by bitwise OR'ing\n"
-	    "it to one of the other operations.\n"
-	    "@end table\n"
+	    "@end defvar\n"
+	    "@defvar LOCK_NB\n"
+	    "Don't block when locking.  This is combined with one of the\n"
+	    "other operations using @code{logior}.  If @code{flock} would\n"
+	    "block an @code{EWOULDBLOCK} error is thrown.\n"
+	    "@end defvar\n"
+	    "\n"
 	    "The return value is not specified. @var{file} may be an open\n"
-	    "file descriptor or an open file descriptor port.")
+	    "file descriptor or an open file descriptor port.\n"
+	    "\n"
+	    "Note that @code{flock} does not lock files across NFS.")
 #define FUNC_NAME s_scm_flock
 {
   int fdes;
@@ -1908,6 +1947,24 @@ scm_init_posix ()
 #ifdef LC_ALL
   scm_c_define ("LC_ALL", scm_from_int (LC_ALL));
 #endif
+#ifdef LC_PAPER
+  scm_c_define ("LC_PAPER", scm_from_int (LC_PAPER));
+#endif
+#ifdef LC_NAME
+  scm_c_define ("LC_NAME", scm_from_int (LC_NAME));
+#endif
+#ifdef LC_ADDRESS
+  scm_c_define ("LC_ADDRESS", scm_from_int (LC_ADDRESS));
+#endif
+#ifdef LC_TELEPHONE
+  scm_c_define ("LC_TELEPHONE", scm_from_int (LC_TELEPHONE));
+#endif
+#ifdef LC_MEASUREMENT
+  scm_c_define ("LC_MEASUREMENT", scm_from_int (LC_MEASUREMENT));
+#endif
+#ifdef LC_IDENTIFICATION
+  scm_c_define ("LC_IDENTIFICATION", scm_from_int (LC_IDENTIFICATION));
+#endif
 #ifdef PIPE_BUF
   scm_c_define ("PIPE_BUF", scm_from_long (PIPE_BUF));
 #endif
