@@ -3,7 +3,7 @@
 #ifndef SCM_PRINT_H
 #define SCM_PRINT_H
 
-/* Copyright (C) 1995,1996,1998,2000,2001, 2003 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1998,2000,2001, 2003, 2004 Free Software Foundation, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -28,9 +28,11 @@
 
 SCM_API scm_t_option scm_print_opts[];
 
-#define SCM_PRINT_CLOSURE	(SCM_PACK (scm_print_opts[0].val))
-#define SCM_PRINT_SOURCE_P	((int) scm_print_opts[1].val)
-#define SCM_N_PRINT_OPTIONS 2
+#define SCM_PRINT_CLOSURE	    (SCM_PACK (scm_print_opts[0].val))
+#define SCM_PRINT_SOURCE_P	    ((int) scm_print_opts[1].val)
+#define SCM_PRINT_HIGHLIGHT_PREFIX  (SCM_PACK (scm_print_opts[2].val))
+#define SCM_PRINT_HIGHLIGHT_SUFFIX  (SCM_PACK (scm_print_opts[3].val))
+#define SCM_N_PRINT_OPTIONS 4
 
 /* State information passed around during printing.
  */
@@ -55,7 +57,7 @@ do { \
 #define SCM_COERCE_OUTPORT(p) \
   (SCM_PORT_WITH_PS_P (p) ? SCM_PORT_WITH_PS_PORT (p) : p)
 
-#define SCM_PRINT_STATE_LAYOUT "sruwuwuwuwuwpwuwuwuruopr"
+#define SCM_PRINT_STATE_LAYOUT "sruwuwuwuwuwpwuwuwurprpw"
 typedef struct scm_print_state {
   SCM handle;			/* Struct handle */
   int revealed;                 /* Has the state escaped to Scheme? */
@@ -67,9 +69,10 @@ typedef struct scm_print_state {
   unsigned long list_offset;
   unsigned long top;		/* Top of reference stack */
   unsigned long ceiling;	/* Max size of reference stack */
-  SCM *ref_stack;		/* Stack of references used during
-				   circular reference detection */
-  SCM ref_vect;
+  SCM ref_vect;	 	        /* Stack of references used during
+				   circular reference detection;
+				   a simple vector. */
+  SCM highlight_objects;        /* List of objects to be highlighted */
 } scm_print_state;
 
 SCM_API SCM scm_print_state_vtable;
@@ -80,7 +83,8 @@ SCM_API SCM scm_print_options (SCM setting);
 SCM_API SCM scm_make_print_state (void);
 SCM_API void scm_free_print_state (SCM print_state);
 SCM scm_i_port_with_print_state (SCM port, SCM print_state);
-SCM_API void scm_intprint (long n, int radix, SCM port);
+SCM_API void scm_intprint (scm_t_intmax n, int radix, SCM port);
+SCM_API void scm_uintprint (scm_t_uintmax n, int radix, SCM port);
 SCM_API void scm_ipruk (char *hdr, SCM ptr, SCM port);
 SCM_API void scm_iprlist (char *hdr, SCM exp, int tlr, SCM port, scm_print_state *pstate);
 SCM_API void scm_print_symbol_name (const char *str, size_t len, SCM port);
