@@ -33,6 +33,7 @@
 #include "libguile/feature.h"
 #include "libguile/strings.h"
 #include "libguile/unif.h"
+#include "libguile/srfi-4.h"
 #include "libguile/vectors.h"
 
 #include "libguile/validate.h"
@@ -74,7 +75,7 @@ scm_t_rng scm_the_rng;
 #define M_PI 3.14159265359
 #endif
 
-#ifdef SCM_HAVE_T_INT64
+#if SCM_HAVE_T_INT64
 
 unsigned long
 scm_i_uniform32 (scm_t_i_rstate *state)
@@ -132,7 +133,7 @@ scm_i_init_rstate (scm_t_i_rstate *state, const char *seed, int n)
       else
         c += seed[i] << (8 * (m - 4));
     }
-  if ((w == 0 && c == 0) || (w == 0xffffffffUL && c == A - 1))
+  if ((w == 0 && c == 0) || (w == -1 && c == A - 1))
     ++c;
   state->w = w;
   state->c = c;
@@ -423,37 +424,65 @@ SCM_DEFINE (scm_random_normal, "random:normal", 0, 1, 0,
 }
 #undef FUNC_NAME
 
-#if SCM_HAVE_ARRAYS
-
 static void
-vector_scale (SCM v, double c)
+vector_scale_x (SCM v, double c)
 {
-  int n = scm_to_int (scm_uniform_vector_length (v));
-  if (SCM_VECTORP (v))
-    while (--n >= 0)
-      SCM_REAL_VALUE (SCM_VELTS (v)[n]) *= c;
+  size_t n;
+  if (scm_is_simple_vector (v))
+    {
+      n = SCM_SIMPLE_VECTOR_LENGTH (v);
+      while (n-- > 0)
+	SCM_REAL_VALUE (SCM_SIMPLE_VECTOR_REF (v, n)) *= c;
+    }
   else
-    while (--n >= 0)
-      ((double *) SCM_VELTS (v))[n] *= c;
+    {
+      /* must be a f64vector. */
+      scm_t_array_handle handle;
+      size_t i, len;
+      ssize_t inc;
+      double *elts;
+
+      elts = scm_f64vector_writable_elements (v, &handle, &len, &inc);
+
+      for (i = 0; i < len; i++, elts += inc)
+	*elts *= c;
+      
+      scm_array_handle_release (&handle);
+    }
 }
 
 static double
 vector_sum_squares (SCM v)
 {
   double x, sum = 0.0;
-  int n = scm_to_int (scm_uniform_vector_length (v));
-  if (SCM_VECTORP (v))
-    while (--n >= 0)
-      {
-	x = SCM_REAL_VALUE (SCM_VELTS (v)[n]);
-	sum += x * x;
-      }
+  size_t n;
+  if (scm_is_simple_vector (v))
+    {
+      n = SCM_SIMPLE_VECTOR_LENGTH (v);
+      while (n-- > 0)
+	{
+	  x = SCM_REAL_VALUE (SCM_SIMPLE_VECTOR_REF (v, n));
+	  sum += x * x;
+	}
+    }
   else
-    while (--n >= 0)
-      {
-	x = ((double *) SCM_VELTS (v))[n];
-	sum += x * x;
-      }
+    {
+      /* must be a f64vector. */
+      scm_t_array_handle handle;
+      size_t i, len;
+      ssize_t inc;
+      const double *elts;
+
+      elts = scm_f64vector_elements (v, &handle, &len, &inc);
+
+      for (i = 0; i < len; i++, elts += inc)
+	{
+	  x = *elts;
+	  sum += x * x;
+	}
+
+      scm_array_handle_release (&handle);
+    }
   return sum;
 }
 
@@ -464,23 +493,21 @@ vector_sum_squares (SCM v)
  */
 SCM_DEFINE (scm_random_solid_sphere_x, "random:solid-sphere!", 1, 1, 0, 
             (SCM v, SCM state),
-            "Fills vect with inexact real random numbers\n"
-            "the sum of whose squares is less than 1.0.\n"
-            "Thinking of vect as coordinates in space of\n"
-            "dimension n = (vector-length vect), the coordinates\n"
-            "are uniformly distributed within the unit n-sphere.\n"
-            "The sum of the squares of the numbers is returned.")
+	    "Fills @var{vect} with inexact real random numbers the sum of\n"
+	    "whose squares is less than 1.0.  Thinking of @var{vect} as\n"
+	    "coordinates in space of dimension @var{n} @math{=}\n"
+	    "@code{(vector-length @var{vect})}, the coordinates are\n"
+	    "uniformly distributed within the unit @var{n}-sphere.")
 #define FUNC_NAME s_scm_random_solid_sphere_x
 {
-  SCM_VALIDATE_VECTOR_OR_DVECTOR (1, v);
   if (SCM_UNBNDP (state))
     state = SCM_VARIABLE_REF (scm_var_random_state);
   SCM_VALIDATE_RSTATE (2, state);
   scm_random_normal_vector_x (v, state);
-  vector_scale (v,
-		pow (scm_c_uniform01 (SCM_RSTATE (state)),
-		     1.0 / scm_to_int (scm_uniform_vector_length (v)))
-		/ sqrt (vector_sum_squares (v)));
+  vector_scale_x (v,
+		  pow (scm_c_uniform01 (SCM_RSTATE (state)),
+		       1.0 / scm_c_generalized_vector_length (v))
+		  / sqrt (vector_sum_squares (v)));
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
@@ -495,12 +522,11 @@ SCM_DEFINE (scm_random_hollow_sphere_x, "random:hollow-sphere!", 1, 1, 0,
             "unit n-sphere.")
 #define FUNC_NAME s_scm_random_hollow_sphere_x
 {
-  SCM_VALIDATE_VECTOR_OR_DVECTOR (1, v);
   if (SCM_UNBNDP (state))
     state = SCM_VARIABLE_REF (scm_var_random_state);
   SCM_VALIDATE_RSTATE (2, state);
   scm_random_normal_vector_x (v, state);
-  vector_scale (v, 1 / sqrt (vector_sum_squares (v)));
+  vector_scale_x (v, 1 / sqrt (vector_sum_squares (v)));
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
@@ -513,24 +539,37 @@ SCM_DEFINE (scm_random_normal_vector_x, "random:normal-vector!", 1, 1, 0,
             "(i.e., with mean 0 and variance 1).")
 #define FUNC_NAME s_scm_random_normal_vector_x
 {
-  int n;
-  SCM_VALIDATE_VECTOR_OR_DVECTOR (1, v);
+  long i;
+  scm_t_array_handle handle;
+  scm_t_array_dim *dim;
+
   if (SCM_UNBNDP (state))
     state = SCM_VARIABLE_REF (scm_var_random_state);
   SCM_VALIDATE_RSTATE (2, state);
-  n = scm_to_int (scm_uniform_vector_length (v));
-  if (SCM_VECTORP (v))
-    while (--n >= 0)
-      SCM_VECTOR_SET (v, n, scm_from_double (scm_c_normal01 (SCM_RSTATE (state))));
+
+  scm_generalized_vector_get_handle (v, &handle);
+  dim = scm_array_handle_dims (&handle);
+
+  if (scm_is_vector (v))
+    {
+      SCM *elts = scm_array_handle_writable_elements (&handle);
+      for (i = dim->lbnd; i <= dim->ubnd; i++, elts += dim->inc)
+	*elts = scm_from_double (scm_c_normal01 (SCM_RSTATE (state)));
+    }
   else
-    while (--n >= 0)
-      ((double *) SCM_VELTS (v))[n] = scm_c_normal01 (SCM_RSTATE (state));
+    {
+      /* must be a f64vector. */
+      double *elts = scm_array_handle_f64_writable_elements (&handle);
+      for (i = dim->lbnd; i <= dim->ubnd; i++, elts += dim->inc)
+	*elts = scm_c_normal01 (SCM_RSTATE (state));
+    }
+
+  scm_array_handle_release (&handle);
+
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 
-#endif /* SCM_HAVE_ARRAYS */
-
 SCM_DEFINE (scm_random_exp, "random:exp", 0, 1, 0, 
             (SCM state),
 	    "Return an inexact real in an exponential distribution with mean\n"
