@@ -1,46 +1,23 @@
-/* Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2005 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001 Free Software Foundation, Inc.
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- * 
- * This program is distributed in the hope that it will be useful,
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301 USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * As a special exception, the Free Software Foundation gives permission
- * for additional uses of the text contained in its release of GUILE.
- *
- * The exception is that, if you link the GUILE library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the GUILE library code into it.
- *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by the
- * Free Software Foundation under the name GUILE.  If you copy
- * code from other Free Software Foundation releases into a copy of
- * GUILE, as the General Public License permits, the exception does
- * not apply to the code that you add in this way.  To avoid misleading
- * anyone as to the status of such modified files, you must delete
- * this exception notice from them.
- *
- * If you write modifications of your own for GUILE, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ */
 
- * This is the C part of the module for delimited I/O.  It's
- * similar to (scsh rdelim) but somewhat incompatible.  */
+#if HAVE_CONFIG_H
+#  include <config.h>
+#endif
 
 #include "libguile/_scm.h"
 
@@ -79,21 +56,24 @@ SCM_DEFINE (scm_read_delimited_x, "%read-delimited!", 3, 3, 0,
 #define FUNC_NAME s_scm_read_delimited_x
 {
   size_t j;
-  char *buf;
   size_t cstart;
   size_t cend;
   int c;
-  char *cdelims;
+  const char *cdelims;
   size_t num_delims;
 
-  SCM_VALIDATE_STRING_COPY (1, delims, cdelims);
-  num_delims = SCM_STRING_LENGTH (delims);
-  SCM_VALIDATE_SUBSTRING_SPEC_COPY (2, str, buf, 5, start, cstart,
-				    6, end, cend);
+  SCM_VALIDATE_STRING (1, delims);
+  cdelims = scm_i_string_chars (delims);
+  num_delims = scm_i_string_length (delims);
+
+  SCM_VALIDATE_STRING (2, str);
+  scm_i_get_substring_spec (scm_i_string_length (str),
+			    start, &cstart, end, &cend);
+
   if (SCM_UNBNDP (port))
-    port = scm_cur_inp;
+    port = scm_current_input_port ();
   else
-    SCM_VALIDATE_OPINPORT (4,port);
+    SCM_VALIDATE_OPINPORT (4, port);
 
   for (j = cstart; j < cend; j++)
     {  
@@ -104,20 +84,20 @@ SCM_DEFINE (scm_read_delimited_x, "%read-delimited!", 3, 3, 0,
 	{
 	  if (cdelims[k] == c)
 	    {
-	      if (SCM_FALSEP (gobble))
+	      if (scm_is_false (gobble))
 		scm_ungetc (c, port);
 
 	      return scm_cons (SCM_MAKE_CHAR (c),
-			       scm_long2num (j - cstart));
+			       scm_from_size_t (j - cstart));
 	    }
 	}
       if (c == EOF)
 	return scm_cons (SCM_EOF_VAL, 
-			 scm_long2num (j - cstart));
+			 scm_from_size_t (j - cstart));
 
-      buf[j] = c;
+      scm_c_string_set_x (str, j, SCM_MAKE_CHAR (c));
     }
-  return scm_cons (SCM_BOOL_F, scm_long2num (j - cstart));
+  return scm_cons (SCM_BOOL_F, scm_from_size_t (j - cstart));
 }
 #undef FUNC_NAME
 
@@ -136,7 +116,7 @@ scm_do_read_line (SCM port, size_t *len_p)
     {
       size_t buf_len = (end + 1) - pt->read_pos;
       /* Allocate a buffer of the perfect size.  */
-      unsigned char *buf = scm_must_malloc (buf_len + 1, "%read-line");
+      unsigned char *buf = scm_malloc (buf_len + 1);
 
       memcpy (buf, pt->read_pos, buf_len);
       pt->read_pos += buf_len;
@@ -155,7 +135,7 @@ scm_do_read_line (SCM port, size_t *len_p)
     size_t buf_size = (len < 50) ? 60 : len * 2;
     /* Invariant: buf always has buf_size + 1 characters allocated;
        the `+ 1' is for the final '\0'.  */
-    unsigned char *buf = scm_must_malloc (buf_size + 1, "%read-line");
+    unsigned char *buf = scm_malloc (buf_size + 1);
     size_t buf_len = 0;
 
     for (;;)
@@ -163,8 +143,7 @@ scm_do_read_line (SCM port, size_t *len_p)
 	if (buf_len + len > buf_size)
 	  {
 	    size_t new_size = (buf_len + len) * 2;
-	    buf = scm_must_realloc (buf, buf_size + 1, new_size + 1,
-				    "%read-line");
+	    buf = scm_realloc (buf, new_size + 1);
 	    buf_size = new_size;
 	  }
 
@@ -197,12 +176,12 @@ scm_do_read_line (SCM port, size_t *len_p)
       }
 
     /* I wonder how expensive this realloc is.  */
-    buf = scm_must_realloc (buf, buf_size + 1, buf_len + 1, "%read-line");
+    buf = scm_realloc (buf, buf_len + 1);
     buf[buf_len] = '\0';
     *len_p = buf_len;
     return buf;
   }
-}  
+}
 
 
 /*
@@ -225,11 +204,11 @@ SCM_DEFINE (scm_read_line, "%read-line", 0, 1, 0,
 {
   scm_t_port *pt;
   char *s;
-  size_t slen = 0;
+  size_t slen;
   SCM line, term;
 
   if (SCM_UNBNDP (port))
-    port = scm_cur_inp;
+    port = scm_current_input_port ();
   SCM_VALIDATE_OPINPORT (1,port);
 
   pt = SCM_PTAB_ENTRY (port);
@@ -246,17 +225,16 @@ SCM_DEFINE (scm_read_line, "%read-line", 0, 1, 0,
 	{
 	  term = SCM_MAKE_CHAR ('\n');
 	  s[slen-1] = '\0';
-	  line = scm_take_str (s, slen-1);
-	  scm_done_free (1);
+	  line = scm_take_locale_stringn (s, slen-1);
 	  SCM_INCLINE (port);
 	}
       else
 	{
 	  /* Fix: we should check for eof on the port before assuming this. */
 	  term = SCM_EOF_VAL;
-	  line = scm_take_str (s, slen);
+	  line = scm_take_locale_stringn (s, slen);
 	  SCM_COL (port) += slen;
-	}	  
+	}
     }
 
   if (pt->rw_random)
