@@ -35,6 +35,7 @@
 #include "libguile/strports.h"
 #include "libguile/vectors.h"
 #include "libguile/validate.h"
+#include "libguile/srfi-4.h"
 
 #include "libguile/read.h"
 
@@ -75,9 +76,9 @@ scm_t_option scm_read_opts[] = {
  */
 
 
-static void
-scm_input_error (char const *function,
-		 SCM port, const char *message, SCM arg)
+void
+scm_i_input_error (char const *function,
+		   SCM port, const char *message, SCM arg)
 {
   SCM fn = (scm_is_string (SCM_FILENAME(port))
 	    ? SCM_FILENAME(port)
@@ -95,7 +96,7 @@ scm_input_error (char const *function,
   string = scm_get_output_string (string_port);
   scm_close_output_port (string_port);
   scm_error_scm (scm_from_locale_symbol ("read-error"),
-		 scm_from_locale_string (function),
+		 function? scm_from_locale_string (function) : SCM_BOOL_F,
 		 string,
 		 arg,
 		 SCM_BOOL_F);
@@ -133,7 +134,7 @@ SCM_DEFINE (scm_read, "read", 0, 1, 0,
   SCM tok_buf, copy;
 
   if (SCM_UNBNDP (port))
-    port = scm_cur_inp;
+    port = scm_current_input_port ();
   SCM_VALIDATE_OPINPORT (1, port);
 
   c = scm_flush_ws (port, (char *) NULL);
@@ -164,7 +165,32 @@ scm_grow_tok_buf (SCM *tok_buf)
   return newdata;
 }
 
+/* Consume an SCSH-style block comment.  Assume that we've already
+   read the initial `#!', and eat characters until we get a
+   exclamation-point/sharp-sign sequence. 
+*/
 
+static void
+skip_scsh_block_comment (SCM port)
+{
+  int bang_seen = 0;
+
+  for (;;)
+    {
+      int c = scm_getc (port);
+      
+      if (c == EOF)
+	scm_i_input_error ("skip_block_comment", port, 
+			   "unterminated `#! ... !#' comment", SCM_EOL);
+
+      if (c == '!')
+	bang_seen = 1;
+      else if (c == '#' && bang_seen)
+	return;
+      else
+	bang_seen = 0;
+    }
+}
 
 int 
 scm_flush_ws (SCM port, const char *eoferr)
@@ -177,10 +203,10 @@ scm_flush_ws (SCM port, const char *eoferr)
       goteof:
 	if (eoferr)
 	  {
-	    scm_input_error (eoferr,
-			     port,
-			     "end of file",
-			     SCM_EOL);
+	    scm_i_input_error (eoferr,
+			       port,
+			       "end of file",
+			       SCM_EOL);
 	  }
 	return c;
       case ';':
@@ -195,6 +221,20 @@ scm_flush_ws (SCM port, const char *eoferr)
 	    break;
 	  }
 	break;
+      case '#':
+	switch (c = scm_getc (port))
+	  {
+	  case EOF:
+	    eoferr = "read_sharp";
+	    goto goteof;
+	  case '!':
+	    skip_scsh_block_comment (port);
+	    break;
+	  default:
+	    scm_ungetc (c, port);
+	    return '#';
+	  }
+	break;
       case SCM_LINE_INCREMENTORS:
       case SCM_SINGLE_SPACES:
       case '\t':
@@ -241,7 +281,7 @@ scm_i_casei_streq (const char *s1, const char *s2, size_t len2)
 static SCM
 recsexpr (SCM obj, long line, int column, SCM filename)
 {
-  if (!SCM_CONSP(obj)) {
+  if (!scm_is_pair(obj)) {
     return obj;
   } else {
     SCM tmp = obj, copy;
@@ -254,7 +294,7 @@ recsexpr (SCM obj, long line, int column, SCM filename)
 	  {
 	    copy = scm_cons (recsexpr (SCM_CAR (obj), line, column, filename),
 			     SCM_UNDEFINED);
-	    while ((tmp = SCM_CDR (tmp)) && SCM_CONSP (tmp))
+	    while ((tmp = SCM_CDR (tmp)) && scm_is_pair (tmp))
 	      {
 		SCM_SETCDR (copy, scm_cons (recsexpr (SCM_CAR (tmp),
 						      line,
@@ -268,7 +308,7 @@ recsexpr (SCM obj, long line, int column, SCM filename)
 	else
 	  {
 	    recsexpr (SCM_CAR (obj), line, column, filename);
-	    while ((tmp = SCM_CDR (tmp)) && SCM_CONSP (tmp))
+	    while ((tmp = SCM_CDR (tmp)) && scm_is_pair (tmp))
 	      recsexpr (SCM_CAR (tmp), line, column, filename);
 	    copy = SCM_UNDEFINED;
 	  }
@@ -284,39 +324,6 @@ recsexpr (SCM obj, long line, int column, SCM filename)
   }
 }
 
-/* Consume an SCSH-style block comment.  Assume that we've already
-   read the initial `#!', and eat characters until we get a
-   newline/exclamation-point/sharp-sign/newline sequence. 
-
-   A carriage return is also reocgnized as a newline. */
-
-static void
-skip_scsh_block_comment (SCM port)
-#define FUNC_NAME "skip_scsh_block_comment"
-{
-  int state = 0;
-
-  for (;;)
-    {
-      int c = scm_getc (port);
-      
-      if (c == EOF)
-	SCM_MISC_ERROR ("unterminated `#! ... !#' comment", SCM_EOL);
-
-      if (state == 1 && c == '!')
-	state = 2;
-      else if (state == 2 && c == '#')
-	state = 3;
-      else if (state == 3 && (c == '\n' || c == '\r'))
-	return;
-      else if (c == '\n' || c == '\r')
-	state = 1;
-      else
-	state = 0;
-    }
-}
-#undef FUNC_NAME
-
 
 static SCM scm_get_hash_procedure(int c);
 static SCM scm_i_lreadparen (SCM *, SCM, char *, SCM *, char);
@@ -334,7 +341,6 @@ scm_lreadr (SCM *tok_buf, SCM port, SCM *copy)
 				  
  tryagain:
   c = scm_flush_ws (port, s_scm_read);
- tryagain_no_flush_ws:
   switch (c)
     {
     case EOF:
@@ -345,7 +351,7 @@ scm_lreadr (SCM *tok_buf, SCM port, SCM *copy)
 	? scm_lreadrecparen (tok_buf, port, s_list, copy)
 	: scm_i_lreadparen (tok_buf, port, s_list, copy, ')');
     case ')':
-      scm_input_error (FUNC_NAME, port,"unexpected \")\"", SCM_EOL);
+      scm_i_input_error (FUNC_NAME, port,"unexpected \")\"", SCM_EOL);
       goto tryagain;
     
 #if SCM_ENABLE_ELISP
@@ -353,7 +359,7 @@ scm_lreadr (SCM *tok_buf, SCM port, SCM *copy)
       if (SCM_ELISP_VECTORS_P)
 	{
 	  p = scm_i_lreadparen (tok_buf, port, s_vector, copy, ']');
-	  return SCM_NULLP (p) ? scm_nullvect : scm_vector (p);
+	  return scm_is_null (p) ? scm_nullvect : scm_vector (p);
 	}
       goto read_token;
 #endif
@@ -415,17 +421,48 @@ scm_lreadr (SCM *tok_buf, SCM port, SCM *copy)
     handle_sharp:
       switch (c)
 	{
+	  /* Vector, arrays, both uniform and not are handled by this
+	     one function.  It also disambiguates between '#f' and
+	     '#f32' and '#f64'.
+	  */
+	case '0': case '1': case '2': case '3': case '4':
+	case '5': case '6': case '7': case '8': case '9':
+	case 'u': case 's': case 'f':
+	case '@':
 	case '(':
-	  p = scm_i_lreadparen (tok_buf, port, s_vector, copy, ')');
-	  return SCM_NULLP (p) ? scm_nullvect : scm_vector (p);
+#if SCM_ENABLE_DEPRECATED
+	  /* See below for 'i' and 'e'. */
+	case 'a':
+	case 'c':
+	case 'y':
+	case 'h':
+	case 'l':
+#endif
+	  return scm_i_read_array (port, c);
 
 	case 't':
 	case 'T':
 	  return SCM_BOOL_T;
-	case 'f':
+
 	case 'F':
+	  /* See above for lower case 'f'. */
 	  return SCM_BOOL_F;
 
+
+	case 'i':
+	case 'e':
+#if SCM_ENABLE_DEPRECATED
+	  {
+	    /* When next char is '(', it really is an old-style
+	       uniform array. */
+	    int next_c = scm_getc (port);
+	    if (next_c != EOF)
+	      scm_ungetc (next_c, port);
+	    if (next_c == '(')
+	      return scm_i_read_array (port, c);
+	    /* Fall through. */
+	  }
+#endif  
 	case 'b':
 	case 'B':
 	case 'o':
@@ -434,31 +471,24 @@ scm_lreadr (SCM *tok_buf, SCM port, SCM *copy)
 	case 'D':
 	case 'x':
 	case 'X':
-	case 'i':
 	case 'I':
-	case 'e':
 	case 'E':
 	  scm_ungetc (c, port);
 	  c = '#';
 	  goto num;
 
 	case '!':
-	  /* start of a shell script.  Parse as a block comment,
-	     terminated by !#, just like SCSH.  */
-	  skip_scsh_block_comment (port);
-	  /* EOF is not an error here */
-	  c = scm_flush_ws (port, (char *)NULL);
-	  goto tryagain_no_flush_ws;
+	  /* should never happen, #!...!# block comments are skipped
+	     over in scm_flush_ws. */
+	  abort ();
 
-#if SCM_HAVE_ARRAYS
 	case '*':
 	  j = scm_read_token (c, tok_buf, port, 0);
-	  p = scm_istr2bve (scm_c_substring_shared (*tok_buf, 1, j-1));
+	  p = scm_istr2bve (scm_c_substring_shared (*tok_buf, 1, j));
 	  if (scm_is_true (p))
 	    return p;
 	  else
 	    goto unkshrp;
-#endif
 
 	case '{':
 	  j = scm_read_token (c, tok_buf, port, 1);
@@ -485,14 +515,12 @@ scm_lreadr (SCM *tok_buf, SCM port, SCM *copy)
 		&& (scm_i_casei_streq (scm_charnames[c],
 				       scm_i_string_chars (*tok_buf), j)))
 	      return SCM_MAKE_CHAR (scm_charnums[c]);
-	  scm_input_error (FUNC_NAME, port, "unknown character name ~a",
-			   scm_list_1 (scm_c_substring (*tok_buf, 0, j)));
+	  scm_i_input_error (FUNC_NAME, port, "unknown character name ~a",
+			     scm_list_1 (scm_c_substring (*tok_buf, 0, j)));
 
 	  /* #:SYMBOL is a syntax for keywords supported in all contexts.  */
 	case ':':
-	  j = scm_read_token ('-', tok_buf, port, 0);
-	  p = scm_string_to_symbol (scm_c_substring_copy (*tok_buf, 0, j));
-	  return scm_make_keyword_from_dash_symbol (p);
+	  return scm_symbol_to_keyword (scm_read (port));
 
 	default:
 	callshrp:
@@ -516,8 +544,8 @@ scm_lreadr (SCM *tok_buf, SCM port, SCM *copy)
 	      }
 	  }
 	unkshrp:
-	scm_input_error (FUNC_NAME, port, "Unknown # object: ~S",
-		     scm_list_1 (SCM_MAKE_CHAR (c)));
+	scm_i_input_error (FUNC_NAME, port, "Unknown # object: ~S",
+			   scm_list_1 (SCM_MAKE_CHAR (c)));
 	}
 
     case '"':
@@ -525,7 +553,9 @@ scm_lreadr (SCM *tok_buf, SCM port, SCM *copy)
       while ('"' != (c = scm_getc (port)))
 	{
 	  if (c == EOF)
-	    str_eof: scm_input_error (FUNC_NAME, port, "end of file in string constant", SCM_EOL);
+	    str_eof: scm_i_input_error (FUNC_NAME, port,
+					"end of file in string constant", 
+					SCM_EOL);
 
 	  while (j + 2 >= scm_i_string_length (*tok_buf))
 	    scm_grow_tok_buf (tok_buf);
@@ -588,15 +618,19 @@ scm_lreadr (SCM *tok_buf, SCM port, SCM *copy)
 		}
 	      default:
 	      bad_escaped:
-		scm_input_error(FUNC_NAME, port,
-				"illegal character in escape sequence: ~S",
-				scm_list_1 (SCM_MAKE_CHAR (c)));
+		scm_i_input_error(FUNC_NAME, port,
+				  "illegal character in escape sequence: ~S",
+				  scm_list_1 (SCM_MAKE_CHAR (c)));
 	      }
 	  scm_c_string_set_x (*tok_buf, j, SCM_MAKE_CHAR (c));
 	  ++j;
 	}
       if (j == 0)
 	return scm_nullstr;
+
+      /* Change this to scm_c_substring_read_only when
+	 SCM_STRING_CHARS has been removed.
+      */
       return scm_c_substring_copy (*tok_buf, 0, j);
 
     case '0': case '1': case '2': case '3': case '4':
@@ -621,17 +655,14 @@ scm_lreadr (SCM *tok_buf, SCM port, SCM *copy)
 	      c = scm_i_string_chars (*tok_buf)[1];
 	      goto callshrp;
 	    }
-	  scm_input_error (FUNC_NAME, port, "unknown # object", SCM_EOL);
+	  scm_i_input_error (FUNC_NAME, port, "unknown # object", SCM_EOL);
 	}
       goto tok;
 
     case ':':
       if (scm_is_eq (SCM_PACK (SCM_KEYWORD_STYLE), scm_keyword_prefix))
-	{
-	  j = scm_read_token ('-', tok_buf, port, 0);
-	  p = scm_string_to_symbol (scm_c_substring (*tok_buf, 0, j));
-	  return scm_make_keyword_from_dash_symbol (p);
-	}
+	return scm_symbol_to_keyword (scm_read (port));
+
       /* fallthrough */
     default:
 #if SCM_ENABLE_ELISP
@@ -759,7 +790,7 @@ scm_i_lreadparen (SCM *tok_buf, SCM port, char *name, SCM *copy, char term_char)
       ans = scm_lreadr (tok_buf, port, copy);
     closeit:
       if (term_char != (c = scm_flush_ws (port, name)))
-	scm_input_error (FUNC_NAME, port, "missing close paren", SCM_EOL);
+	scm_i_input_error (FUNC_NAME, port, "missing close paren", SCM_EOL);
       return ans;
     }
   ans = tl = scm_cons (tmp, SCM_EOL);
@@ -799,13 +830,13 @@ scm_lreadrecparen (SCM *tok_buf, SCM port, char *name, SCM *copy)
     {
       ans = scm_lreadr (tok_buf, port, copy);
       if (')' != (c = scm_flush_ws (port, name)))
-	scm_input_error (FUNC_NAME, port, "missing close paren", SCM_EOL);
+	scm_i_input_error (FUNC_NAME, port, "missing close paren", SCM_EOL);
       return ans;
     }
   /* Build the head of the list structure. */
   ans = tl = scm_cons (tmp, SCM_EOL);
   if (SCM_COPY_SOURCE_P)
-    ans2 = tl2 = scm_cons (SCM_CONSP (tmp)
+    ans2 = tl2 = scm_cons (scm_is_pair (tmp)
 			   ? *copy
 			   : tmp,
 			   SCM_EOL);
@@ -818,12 +849,13 @@ scm_lreadrecparen (SCM *tok_buf, SCM port, char *name, SCM *copy)
 	{
 	  SCM_SETCDR (tl, tmp = scm_lreadr (tok_buf, port, copy));
 	  if (SCM_COPY_SOURCE_P)
-	    SCM_SETCDR (tl2, scm_cons (SCM_CONSP (tmp)
+	    SCM_SETCDR (tl2, scm_cons (scm_is_pair (tmp)
 				       ? *copy
 				       : tmp,
 				       SCM_EOL));
 	  if (')' != (c = scm_flush_ws (port, name)))
-	    scm_input_error (FUNC_NAME, port, "missing close paren", SCM_EOL);
+	    scm_i_input_error (FUNC_NAME, port,
+			       "missing close paren", SCM_EOL);
 	  goto exit;
 	}
 
@@ -833,7 +865,7 @@ scm_lreadrecparen (SCM *tok_buf, SCM port, char *name, SCM *copy)
 
       if (SCM_COPY_SOURCE_P)
 	{
-	  SCM new_tail2 = scm_cons (SCM_CONSP (tmp) ? *copy : tmp, SCM_EOL);
+	  SCM new_tail2 = scm_cons (scm_is_pair (tmp) ? *copy : tmp, SCM_EOL);
 	  SCM_SETCDR (tl2, new_tail2);
 	  tl2 = new_tail2;
 	}
@@ -879,7 +911,7 @@ SCM_DEFINE (scm_read_hash_extend, "read-hash-extend", 2, 0, 0,
   prev = SCM_BOOL_F;
   while (1)
     {
-      if (SCM_NULLP (this))
+      if (scm_is_null (this))
 	{
 	  /* not found, so add it to the beginning.  */
 	  if (scm_is_true (proc))
@@ -926,7 +958,7 @@ scm_get_hash_procedure (int c)
 
   while (1)
     {
-      if (SCM_NULLP (rest))
+      if (scm_is_null (rest))
 	return SCM_BOOL_F;
   
       if (SCM_CHAR (SCM_CAAR (rest)) == c)
