@@ -52,6 +52,7 @@
 #endif
 #endif
 
+#include "libguile/async.h"
 #include "libguile/smob.h"
 #include "libguile/symbols.h"
 #include "libguile/vectors.h"
@@ -162,7 +163,7 @@ SCM_DEFINE (scm_make_regexp, "make-regexp", 1, 0, 1,
      turn off REG_EXTENDED flag (on by default). */
   cflags = REG_EXTENDED;
   flag = flags;
-  while (!SCM_NULLP (flag))
+  while (!scm_is_null (flag))
     {
       if (scm_to_int (SCM_CAR (flag)) == REG_BASIC)
 	cflags &= ~REG_EXTENDED;
@@ -218,15 +219,23 @@ SCM_DEFINE (scm_regexp_exec, "regexp-exec", 2, 2, 0,
 {
   int status, nmatches, offset;
   regmatch_t *matches;
+  char *c_str;
   SCM mvec = SCM_BOOL_F;
+  SCM substr;
 
   SCM_VALIDATE_RGXP (1, rx);
   SCM_VALIDATE_STRING (2, str);
 
   if (SCM_UNBNDP (start))
-    offset = 0;
+    {
+      substr = str;
+      offset = 0;
+    }
   else
-    offset = scm_to_signed_integer (start, 0, scm_i_string_length (str));
+    {
+      substr = scm_substring (str, start, SCM_UNDEFINED);
+      offset = scm_to_int (start);
+    }
 
   if (SCM_UNBNDP (flags))
     flags = SCM_INUM0;
@@ -235,29 +244,31 @@ SCM_DEFINE (scm_regexp_exec, "regexp-exec", 2, 2, 0,
      whole regexp, so add 1 to nmatches. */
 
   nmatches = SCM_RGX(rx)->re_nsub + 1;
-  SCM_DEFER_INTS;
+  SCM_CRITICAL_SECTION_START;
   matches = scm_malloc (sizeof (regmatch_t) * nmatches);
-  status = regexec (SCM_RGX (rx), scm_i_string_chars (str) + offset,
-		    nmatches, matches,
+  c_str = scm_to_locale_string (substr);
+  status = regexec (SCM_RGX (rx), c_str, nmatches, matches,
 		    scm_to_int (flags));
+  free (c_str);
+
   if (!status)
     {
       int i;
       /* The match vector must include a cell for the string that was matched,
 	 so add 1. */
       mvec = scm_c_make_vector (nmatches + 1, SCM_UNSPECIFIED);
-      SCM_VECTOR_SET(mvec,0, str);
+      SCM_SIMPLE_VECTOR_SET(mvec,0, str);
       for (i = 0; i < nmatches; ++i)
 	if (matches[i].rm_so == -1)
-	  SCM_VECTOR_SET(mvec, i+1,
+	  SCM_SIMPLE_VECTOR_SET(mvec, i+1,
 			 scm_cons (scm_from_int (-1), scm_from_int (-1)));
 	else
-	  SCM_VECTOR_SET(mvec, i+1,
+	  SCM_SIMPLE_VECTOR_SET(mvec, i+1,
 			 scm_cons (scm_from_long (matches[i].rm_so + offset),
 				   scm_from_long (matches[i].rm_eo + offset)));
     }
   free (matches);
-  SCM_ALLOW_INTS;
+  SCM_CRITICAL_SECTION_END;
 
   if (status != 0 && status != REG_NOMATCH)
     scm_error_scm (scm_regexp_error_key,
