@@ -399,6 +399,9 @@ SCM_SYMBOL (sym_use_srfis, "use-srfis");
 SCM_SYMBOL (sym_load_path, "%load-path");
 SCM_SYMBOL (sym_set_x, "set!");
 SCM_SYMBOL (sym_cons, "cons");
+SCM_SYMBOL (sym_at, "@");
+SCM_SYMBOL (sym_atat, "@@");
+SCM_SYMBOL (sym_main, "main");
 
 /* Given an array of command-line switches, return a Scheme expression
    to carry out the actions specified by the switches.
@@ -411,6 +414,18 @@ SCM_SYMBOL (sym_cons, "cons");
 
 static char guile[] = "guile";
 
+static int
+all_symbols (SCM list)
+{
+  while (scm_is_pair (list))
+    {
+      if (!scm_is_symbol (SCM_CAR (list)))
+	return 0;
+      list = SCM_CDR (list);
+    }
+  return 1;
+}
+
 SCM
 scm_compile_shell_switches (int argc, char **argv)
 {
@@ -453,7 +468,7 @@ scm_compile_shell_switches (int argc, char **argv)
 	  /* If we specified the -ds option, do_script points to the
 	     cdr of an expression like (load #f); we replace the car
 	     (i.e., the #f) with the script name.  */
-	  if (!SCM_NULLP (do_script))
+	  if (!scm_is_null (do_script))
 	    {
 	      SCM_SETCAR (do_script, scm_from_locale_string (argv[i]));
 	      do_script = SCM_EOL;
@@ -517,7 +532,25 @@ scm_compile_shell_switches (int argc, char **argv)
       else if (! strcmp (argv[i], "-e")) /* entry point */
 	{
 	  if (++i < argc)
-	    entry_point = scm_c_read_string (argv[i]);
+	    {
+	      SCM port 
+		= scm_open_input_string (scm_from_locale_string (argv[i]));
+	      SCM arg1 = scm_read (port);
+	      SCM arg2 = scm_read (port);
+
+	      /* Recognize syntax of certain versions of Guile 1.4 and
+		 transform to (@ MODULE-NAME FUNC).
+	       */
+	      if (scm_is_false (scm_eof_object_p (arg2)))
+		entry_point = scm_list_3 (sym_at, arg1, arg2);
+	      else if (scm_is_pair (arg1)
+		       && !(scm_is_eq (SCM_CAR (arg1), sym_at)
+			    || scm_is_eq (SCM_CAR (arg1), sym_atat))
+		       && all_symbols (arg1))
+		entry_point = scm_list_3 (sym_at, arg1, sym_main);
+	      else
+		entry_point = arg1;
+	    }
 	  else
 	    scm_shell_usage (1, "missing argument to `-e' switch");
 	}
@@ -526,7 +559,7 @@ scm_compile_shell_switches (int argc, char **argv)
 	{
 	  /* We put a dummy "load" expression, and let the -s put the
              filename in.  */
-	  if (!SCM_NULLP (do_script))
+	  if (!scm_is_null (do_script))
 	    scm_shell_usage (1, "the -ds switch may only be specified once");
 	  do_script = scm_cons (SCM_BOOL_F, SCM_EOL);
 	  tail = scm_cons (scm_cons (sym_load, do_script),
@@ -597,7 +630,7 @@ scm_compile_shell_switches (int argc, char **argv)
 	{
 	  /* Print version number.  */
 	  printf ("Guile %s\n"
-		  "Copyright (c) 1995, 1996, 1997, 2000, 2001, 2002 Free Software Foundation\n"
+		  "Copyright (c) 1995, 1996, 1997, 2000, 2001, 2002, 2003, 2004, 2005 Free Software Foundation\n"
 		  "Guile may be distributed under the terms of the GNU General Public Licence;\n"
 		  "certain other uses are permitted as well.  For details, see the file\n"
 		  "`COPYING', which is included in the Guile distribution.\n"
@@ -615,7 +648,7 @@ scm_compile_shell_switches (int argc, char **argv)
     }
 
   /* Check to make sure the -ds got a -s. */
-  if (!SCM_NULLP (do_script))
+  if (!scm_is_null (do_script))
     scm_shell_usage (1, "the `-ds' switch requires the use of `-s' as well");
 
   /* Make any remaining arguments available to the
@@ -626,7 +659,7 @@ scm_compile_shell_switches (int argc, char **argv)
   scm_c_define ("use-emacs-interface", scm_from_bool (use_emacs_interface));
 
   /* Handle the `-e' switch, if it was specified.  */
-  if (!SCM_NULLP (entry_point))
+  if (!scm_is_null (entry_point))
     tail = scm_cons (scm_cons2 (entry_point,
 				scm_cons (sym_command_line, SCM_EOL),
 				SCM_EOL),
@@ -650,7 +683,7 @@ scm_compile_shell_switches (int argc, char **argv)
 
   /* add the user-specified load path here, so it won't be in effect
      during the loading of the user's customization file. */
-  if(!SCM_NULLP(user_load_path)) 
+  if(!scm_is_null(user_load_path)) 
     {
       tail = scm_append_x( scm_cons2(user_load_path, tail, SCM_EOL) );
     }
