@@ -1,4 +1,4 @@
-/* Copyright (C) 1995,1996,1998,1999,2000,2001, 2003 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1998,1999,2000,2001, 2003, 2004 Free Software Foundation, Inc.
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -26,7 +26,9 @@
 
 #include "libguile/_scm.h"
 
+#include "libguile/async.h"
 #include "libguile/objects.h"
+#include "libguile/goops.h"
 #include "libguile/ports.h"
 
 #ifdef HAVE_MALLOC_H
@@ -55,6 +57,13 @@ scm_i_set_smob_flags (SCM x, scm_t_bits data)
   SCM_SET_CELL_WORD_0 (x, (SCM_CELL_WORD_0 (x) & 0xFFFF) | data);
 }
 
+void
+scm_assert_smob_type (scm_t_bits tag, SCM val)
+{
+  if (!SCM_SMOB_PREDICATE (tag, val))
+    scm_wrong_type_arg_msg (NULL, 0, val, scm_smobs[SCM_TC2SMOBNUM(tag)].name);
+}
+
 /* {Mark}
  */
 
@@ -112,9 +121,9 @@ scm_smob_print (SCM exp, SCM port, scm_print_state *pstate SCM_UNUSED)
   scm_puts (SCM_SMOBNAME (n) ? SCM_SMOBNAME (n) : "smob", port);
   scm_putc (' ', port);
   if (scm_smobs[n].size)
-    scm_intprint (SCM_CELL_WORD_1 (exp), 16, port);
+    scm_uintprint (SCM_CELL_WORD_1 (exp), 16, port);
   else
-    scm_intprint (SCM_UNPACK (exp), 16, port);
+    scm_uintprint (SCM_UNPACK (exp), 16, port);
   scm_putc ('>', port);
   return 1;
 }
@@ -242,7 +251,7 @@ scm_smob_apply_2_error (SCM smob, SCM a1 SCM_UNUSED, SCM a2 SCM_UNUSED)
 static SCM
 scm_smob_apply_3_030 (SCM smob, SCM a1, SCM a2, SCM rst)
 {
-  if (!SCM_NULLP (SCM_CDR (rst)))
+  if (!scm_is_null (SCM_CDR (rst)))
     scm_wrong_num_args (smob);
   return SCM_SMOB_APPLY3 (smob, a1, a2, SCM_CAR (rst));
 }
@@ -282,11 +291,11 @@ scm_make_smob_type (char const *name, size_t size)
 {
   long new_smob;
 
-  SCM_ENTER_A_SECTION;  /* scm_numsmob */
+  SCM_CRITICAL_SECTION_START;
   new_smob = scm_numsmob;
   if (scm_numsmob != MAX_SMOB_COUNT)
     ++scm_numsmob;
-  SCM_EXIT_A_SECTION;
+  SCM_CRITICAL_SECTION_END;
 
   if (new_smob == MAX_SMOB_COUNT)
     scm_misc_error (FUNC_NAME, "maximum number of smobs exceeded", SCM_EOL);
