@@ -1,47 +1,27 @@
-/* Copyright (C) 1995,1996,1998,1999,2000,2001 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1998,1999,2000,2001,2002, 2003 Free Software Foundation, Inc.
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- * 
- * This program is distributed in the hope that it will be useful,
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * As a special exception, the Free Software Foundation gives permission
- * for additional uses of the text contained in its release of GUILE.
- *
- * The exception is that, if you link the GUILE library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the GUILE library code into it.
- *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by the
- * Free Software Foundation under the name GUILE.  If you copy
- * code from other Free Software Foundation releases into a copy of
- * GUILE, as the General Public License permits, the exception does
- * not apply to the code that you add in this way.  To avoid misleading
- * anyone as to the status of such modified files, you must delete
- * this exception notice from them.
- *
- * If you write modifications of your own for GUILE, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.  */
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ */
 
 
 
 
+#if HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 #include "libguile/_scm.h"
 
 #include <stdio.h>
@@ -72,6 +52,14 @@
  */
 
 /* NOTES:
+
+   We break the rules set forth by strings.h about accessing the
+   internals of strings here.  We can do this since we can guarantee
+   that the string used as pt->stream is not in use by anyone else.
+   Thus, it's representation will not change asynchronously.
+
+   (Ports aren't thread-safe yet anyway...)
+
    write_buf/write_end point to the ends of the allocated string.
    read_buf/read_end in principle point to the part of the string which
    has been written to, but this is only updated after a flush.
@@ -99,8 +87,10 @@ static void
 st_resize_port (scm_t_port *pt, off_t new_size)
 {
   SCM old_stream = SCM_PACK (pt->stream);
-  SCM new_stream = scm_allocate_string (new_size);
-  unsigned long int old_size = SCM_STRING_LENGTH (old_stream);
+  const char *src = scm_i_string_chars (old_stream);
+  char *dst;
+  SCM new_stream = scm_i_make_string (new_size, &dst);
+  unsigned long int old_size = scm_i_string_length (old_stream);
   unsigned long int min_size = min (old_size, new_size);
   unsigned long int i;
 
@@ -109,12 +99,14 @@ st_resize_port (scm_t_port *pt, off_t new_size)
   pt->write_buf_size = new_size;
 
   for (i = 0; i != min_size; ++i)
-    SCM_STRING_CHARS (new_stream) [i] = SCM_STRING_CHARS (old_stream) [i];
+    dst[i] = src[i];
+
+  scm_remember_upto_here_1 (old_stream);
 
   /* reset buffer. */
   {
     pt->stream = SCM_UNPACK (new_stream);
-    pt->read_buf = pt->write_buf = SCM_STRING_UCHARS (new_stream);
+    pt->read_buf = pt->write_buf = dst;
     pt->read_pos = pt->write_pos = pt->write_buf + index;
     pt->write_end = pt->write_buf + pt->write_buf_size;
     pt->read_end = pt->read_buf + pt->read_buf_size;
@@ -270,29 +262,46 @@ scm_mkstrport (SCM pos, SCM str, long modes, const char *caller)
 {
   SCM z;
   scm_t_port *pt;
-  size_t str_len;
+  size_t str_len, c_pos;
+
+  SCM_ASSERT (scm_is_string (str), str, SCM_ARG1, caller);
+
+  str_len = scm_i_string_length (str);
+  c_pos = scm_to_unsigned_integer (pos, 0, str_len);
 
-  SCM_ASSERT (SCM_INUMP(pos) && SCM_INUM(pos) >= 0, pos, SCM_ARG1, caller);
-  SCM_ASSERT (SCM_STRINGP (str), str, SCM_ARG1, caller);
-  str_len = SCM_STRING_LENGTH (str);
-  if (SCM_INUM (pos) > str_len)
-    scm_out_of_range (caller, pos);
   if (!((modes & SCM_WRTNG) || (modes & SCM_RDNG)))
     scm_misc_error ("scm_mkstrport", "port must read or write", SCM_EOL);
-  SCM_NEWCELL (z);
-  SCM_DEFER_INTS;
-  pt = scm_add_to_port_table (z);
-  SCM_SET_CELL_TYPE (z, scm_tc16_strport | modes);
-  SCM_SETPTAB_ENTRY (z, pt);
+
+  /* XXX
+ 
+     Make a new string to isolate us from changes to the original.
+     This is done so that we can rely on scm_i_string_chars to stay in
+     place even across SCM_TICKs.
+
+     Additionally, when we are going to write to the string, we make a
+     copy so that we can write to it without having to use
+     scm_i_string_writable_chars.
+  */
+
+  if (modes & SCM_WRTNG)
+    str = scm_c_substring_copy (str, 0, str_len);
+  else
+    str = scm_c_substring (str, 0, str_len);
+
+  scm_i_scm_pthread_mutex_lock (&scm_i_port_table_mutex);
+  z = scm_new_port_table_entry (scm_tc16_strport);
+  pt = SCM_PTAB_ENTRY(z);
   SCM_SETSTREAM (z, SCM_UNPACK (str));
-  pt->write_buf = pt->read_buf = SCM_STRING_UCHARS (str);
-  pt->read_pos = pt->write_pos = pt->read_buf + SCM_INUM (pos);
+  SCM_SET_CELL_TYPE(z, scm_tc16_strport|modes);
+  /* see above why we can use scm_i_string_chars here. */
+  pt->write_buf = pt->read_buf = (char *)scm_i_string_chars (str);
+  pt->read_pos = pt->write_pos = pt->read_buf + c_pos;
   pt->write_buf_size = pt->read_buf_size = str_len;
   pt->write_end = pt->read_end = pt->read_buf + pt->read_buf_size;
 
   pt->rw_random = 1;
 
-  SCM_ALLOW_INTS;
+  scm_i_pthread_mutex_unlock (&scm_i_port_table_mutex);
 
   /* ensure write_pos is writable. */
   if ((modes & SCM_WRTNG) && pt->write_pos == pt->write_end)
@@ -305,11 +314,13 @@ SCM scm_strport_to_string (SCM port)
 {
   scm_t_port *pt = SCM_PTAB_ENTRY (port);
   SCM str;
-
+  char *dst;
+  
   if (pt->rw_active == SCM_PORT_WRITE)
     st_flush (port);
 
-  str = scm_mem2string ((char *) pt->read_buf, pt->read_buf_size);
+  str = scm_i_make_string (pt->read_buf_size, &dst);
+  memcpy (dst, (char *) pt->read_buf, pt->read_buf_size);
   scm_remember_upto_here_1 (port);
   return str;
 }
@@ -326,7 +337,7 @@ SCM_DEFINE (scm_object_to_string, "object->string", 1, 1, 0,
   if (!SCM_UNBNDP (printer))
     SCM_VALIDATE_PROC (2, printer);
 
-  str = scm_allocate_string (0);
+  str = scm_c_make_string (0, SCM_UNDEFINED);
   port = scm_mkstrport (SCM_INUM0, str, SCM_OPN | SCM_WRTNG, FUNC_NAME);
 
   if (SCM_UNBNDP (printer))
@@ -338,16 +349,6 @@ SCM_DEFINE (scm_object_to_string, "object->string", 1, 1, 0,
 }
 #undef FUNC_NAME
 
-#if (SCM_DEBUG_DEPRECATED == 0)
-
-SCM
-scm_strprint_obj (SCM obj)
-{
-  return scm_object_to_string (obj, SCM_UNDEFINED);
-}
-
-#endif /* (SCM_DEBUG_DEPRECATED == 0) */
-
 SCM_DEFINE (scm_call_with_output_string, "call-with-output-string", 1, 0, 0, 
            (SCM proc),
 	    "Calls the one-argument procedure @var{proc} with a newly created output\n"
@@ -430,7 +431,7 @@ SCM
 scm_c_read_string (const char *expr)
 {
   SCM port = scm_mkstrport (SCM_INUM0,
-			    scm_makfrom0str (expr),
+			    scm_from_locale_string (expr),
 			    SCM_OPN | SCM_RDNG,
 			    "scm_c_read_string");
   SCM form;
@@ -447,30 +448,15 @@ scm_c_read_string (const char *expr)
 SCM
 scm_c_eval_string (const char *expr)
 {
-  return scm_eval_string (scm_makfrom0str (expr));
-}
-
-#if SCM_DEBUG_DEPRECATED == 0
-
-SCM
-scm_read_0str (char *expr)
-{
-  scm_c_issue_deprecation_warning 
-    ("scm_read_0str is deprecated.  Use scm_c_read_string instead.");
-
-  return scm_c_read_string (expr);
+  return scm_eval_string (scm_from_locale_string (expr));
 }
 
 SCM
-scm_eval_0str (const char *expr)
+scm_c_eval_string_in_module (const char *expr, SCM module)
 {
-  scm_c_issue_deprecation_warning 
-    ("scm_eval_0str is deprecated.  Use scm_c_eval_string instead.");
-
-  return scm_c_eval_string (expr);
+  return scm_eval_string_in_module (scm_from_locale_string (expr), module);
 }
 
-#endif
 
 static SCM
 inner_eval_string (void *data)
@@ -491,21 +477,34 @@ inner_eval_string (void *data)
   return ans;
 }
 
-SCM_DEFINE (scm_eval_string, "eval-string", 1, 0, 0, 
-            (SCM string),
+SCM_DEFINE (scm_eval_string_in_module, "eval-string", 1, 1, 0, 
+            (SCM string, SCM module),
 	    "Evaluate @var{string} as the text representation of a Scheme\n"
 	    "form or forms, and return whatever value they produce.\n"
-	    "Evaluation takes place in the environment returned by the\n"
-	    "procedure @code{interaction-environment}.")
-#define FUNC_NAME s_scm_eval_string
+	    "Evaluation takes place in the given module, or the current\n"
+            "module when no module is given.\n"
+            "While the code is evaluated, the given module is made the\n"
+	    "current one.  The current module is restored when this\n"
+            "procedure returns.")
+#define FUNC_NAME s_scm_eval_string_in_module
 {
   SCM port = scm_mkstrport (SCM_INUM0, string, SCM_OPN | SCM_RDNG,
-			    "eval-string");
-  return scm_c_call_with_current_module (scm_interaction_environment (),
+			    FUNC_NAME);
+  if (SCM_UNBNDP (module))
+    module = scm_current_module ();
+  else
+    SCM_VALIDATE_MODULE (2, module);
+  return scm_c_call_with_current_module (module,
 					 inner_eval_string, (void *)port);
 }
 #undef FUNC_NAME
 
+SCM
+scm_eval_string (SCM string)
+{
+  return scm_eval_string_in_module (string, SCM_UNDEFINED);
+}
+
 static scm_t_bits
 scm_make_stptob ()
 {
