@@ -21,6 +21,7 @@
 #endif
 
 #include "libguile/_scm.h"
+#include "libguile/async.h"
 #include "libguile/chars.h"
 #include "libguile/eval.h"
 #include "libguile/alist.h"
@@ -172,7 +173,7 @@ scm_struct_init (SCM handle, SCM layout, scm_t_bits * mem, int tail_elts, SCM in
 #endif
 
 	case 'u':
-	  if ((prot != 'r' && prot != 'w') || SCM_NULLP (inits))
+	  if ((prot != 'r' && prot != 'w') || scm_is_null (inits))
 	    *mem = 0;
 	  else
 	    {
@@ -182,7 +183,7 @@ scm_struct_init (SCM handle, SCM layout, scm_t_bits * mem, int tail_elts, SCM in
 	  break;
 
 	case 'p':
-	  if ((prot != 'r' && prot != 'w') || SCM_NULLP (inits))
+	  if ((prot != 'r' && prot != 'w') || scm_is_null (inits))
 	    *mem = SCM_UNPACK (SCM_BOOL_F);
 	  else
 	    {
@@ -357,7 +358,7 @@ scm_free_structs (void *dummy1 SCM_UNUSED,
     {
       /* Mark vtables in GC chain.  GC mark set means delay freeing. */
       SCM chain = newchain;
-      while (!SCM_NULLP (chain))
+      while (!scm_is_null (chain))
 	{
 	  SCM vtable = SCM_STRUCT_VTABLE (chain);
 	  if (SCM_STRUCT_GC_CHAIN (vtable) != 0 && vtable != chain)
@@ -367,7 +368,7 @@ scm_free_structs (void *dummy1 SCM_UNUSED,
       /* Free unmarked structs.  */
       chain = newchain;
       newchain = SCM_EOL;
-      while (!SCM_NULLP (chain))
+      while (!scm_is_null (chain))
 	{
 	  SCM obj = chain;
 	  chain = SCM_STRUCT_GC_CHAIN (chain);
@@ -390,7 +391,7 @@ scm_free_structs (void *dummy1 SCM_UNUSED,
 	    }
 	}
     }
-  while (!SCM_NULLP (newchain));
+  while (!scm_is_null (newchain));
   return 0;
 }
 
@@ -429,7 +430,7 @@ SCM_DEFINE (scm_make_struct, "make-struct", 2, 0, 1,
   layout = SCM_PACK (SCM_STRUCT_DATA (vtable) [scm_vtable_index_layout]);
   basic_size = scm_i_symbol_length (layout) / 2;
   tail_elts = scm_to_size_t (tail_array_size);
-  SCM_DEFER_INTS;
+  SCM_CRITICAL_SECTION_START;
   if (SCM_STRUCT_DATA (vtable)[scm_struct_i_flags] & SCM_STRUCTF_ENTITY)
     {
       data = scm_alloc_struct (basic_size + tail_elts,
@@ -446,7 +447,7 @@ SCM_DEFINE (scm_make_struct, "make-struct", 2, 0, 1,
 			     + scm_tc3_struct),
 			    (scm_t_bits) data, 0, 0);
   scm_struct_init (handle, layout, data, tail_elts, init);
-  SCM_ALLOW_INTS;
+  SCM_CRITICAL_SECTION_END;
   return handle;
 }
 #undef FUNC_NAME
@@ -516,7 +517,7 @@ SCM_DEFINE (scm_make_vtable_vtable, "make-vtable-vtable", 2, 0, 1,
   layout = scm_make_struct_layout (fields);
   basic_size = scm_i_symbol_length (layout) / 2;
   tail_elts = scm_to_size_t (tail_array_size);
-  SCM_DEFER_INTS;
+  SCM_CRITICAL_SECTION_START;
   data = scm_alloc_struct (basic_size + tail_elts,
 			   scm_struct_n_extra_words,
 			   "struct");
@@ -524,7 +525,7 @@ SCM_DEFINE (scm_make_vtable_vtable, "make-vtable-vtable", 2, 0, 1,
 			    (scm_t_bits) data, 0, 0);
   data [scm_vtable_index_layout] = SCM_UNPACK (layout);
   scm_struct_init (handle, layout, data, tail_elts, scm_cons (layout, init));
-  SCM_ALLOW_INTS;
+  SCM_CRITICAL_SECTION_END;
   return handle;
 }
 #undef FUNC_NAME
@@ -777,9 +778,9 @@ scm_print_struct (SCM exp, SCM port, scm_print_state *pstate)
       else
 	scm_puts ("struct", port);
       scm_putc (' ', port);
-      scm_intprint (SCM_UNPACK (vtable), 16, port);
+      scm_uintprint (SCM_UNPACK (vtable), 16, port);
       scm_putc (':', port);
-      scm_intprint (SCM_UNPACK (exp), 16, port);
+      scm_uintprint (SCM_UNPACK (exp), 16, port);
       scm_putc ('>', port);
     }
 }
