@@ -388,9 +388,10 @@ typedef unsigned long scm_t_bits;
 /* Checking if a SCM variable holds a pair (for historical reasons, in Guile
  * also known as a cons-cell): This is done by first checking that the SCM
  * variable holds a non-immediate, and second, by checking that tc1==0 holds
- * for the SCM_CELL_TYPE of the SCM variable.  */
-#define SCM_CONSP(x)  (!SCM_IMP (x) && ((1 & SCM_CELL_TYPE (x)) == 0))
-#define SCM_NCONSP(x) (!SCM_CONSP (x))
+ * for the SCM_CELL_TYPE of the SCM variable.  
+*/
+
+#define SCM_I_CONSP(x)  (!SCM_IMP (x) && ((1 & SCM_CELL_TYPE (x)) == 0))
 
 
 
@@ -437,17 +438,15 @@ typedef unsigned long scm_t_bits;
 
 #define scm_tc7_pws		31
 
-#if SCM_HAVE_ARRAYS
-#define scm_tc7_llvect          29
-#define scm_tc7_uvect		37
-#define scm_tc7_fvect		45
-#define scm_tc7_dvect		47
-#define scm_tc7_cvect		53
-#define scm_tc7_svect		55
-#define scm_tc7_bvect		71
-#define scm_tc7_byvect		77
-#define scm_tc7_ivect		79
-#endif
+#define scm_tc7_unused_1        29
+#define scm_tc7_unused_2	37
+#define scm_tc7_unused_3	45
+#define scm_tc7_unused_4	47
+#define scm_tc7_unused_5	53
+#define scm_tc7_unused_6	55
+#define scm_tc7_unused_7	71
+#define scm_tc7_unused_8	77
+#define scm_tc7_unused_9	79
 
 #define scm_tc7_dsubr		61
 #define scm_tc7_cclo		63
