@@ -1,4 +1,4 @@
-/* Copyright (C) 1995,1996,1997,1998,2000,2001, 2003 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1997,1998,2000,2001, 2003, 2004 Free Software Foundation, Inc.
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,6 +20,7 @@
 
 #include <stdio.h>
 #include "libguile/_scm.h"
+#include "libguile/async.h"
 #include "libguile/smob.h"
 #include "libguile/alist.h"
 #include "libguile/eval.h"
@@ -33,9 +34,9 @@
 #include "libguile/fluids.h"
 #include "libguile/ports.h"
 #include "libguile/lang.h"
-
 #include "libguile/validate.h"
 #include "libguile/throw.h"
+#include "libguile/init.h"
 
 
 /* the jump buffer data structure */
@@ -59,7 +60,7 @@ jmpbuffer_print (SCM exp, SCM port, scm_print_state *pstate SCM_UNUSED)
 {
   scm_puts ("#<jmpbuffer ", port);
   scm_puts (JBACTIVE(exp) ? "(active) " : "(inactive) ", port);
-  scm_intprint((long) JBJMPBUF (exp), 16, port);
+  scm_uintprint((scm_t_bits) JBJMPBUF (exp), 16, port);
   scm_putc ('>', port);
   return 1 ;
 }
@@ -68,13 +69,13 @@ static SCM
 make_jmpbuf (void)
 {
   SCM answer;
-  SCM_REDEFER_INTS;
+  SCM_CRITICAL_SECTION_START;
   {
     SCM_NEWSMOB2 (answer, tc16_jmpbuffer, 0, 0);
     SETJBJMPBUF(answer, (jmp_buf *)0);
     DEACTIVATEJB(answer);
   }
-  SCM_REALLOW_INTS;
+  SCM_CRITICAL_SECTION_END;
   return answer;
 }
 
@@ -145,9 +146,9 @@ scm_internal_catch (SCM tag, scm_t_catch_body body, void *body_data, scm_t_catch
 
   jmpbuf = make_jmpbuf ();
   answer = SCM_EOL;
-  scm_dynwinds = scm_acons (tag, jmpbuf, scm_dynwinds);
+  scm_i_set_dynwinds (scm_acons (tag, jmpbuf, scm_i_dynwinds ()));
   SETJBJMPBUF(jmpbuf, &jbr.buf);
-  SCM_SETJBDFRAME(jmpbuf, scm_last_debug_frame);
+  SCM_SETJBDFRAME(jmpbuf, scm_i_last_debug_frame ());
   if (setjmp (jbr.buf))
     {
       SCM throw_tag;
@@ -156,10 +157,10 @@ scm_internal_catch (SCM tag, scm_t_catch_body body, void *body_data, scm_t_catch
 #ifdef STACK_CHECKING
       scm_stack_checking_enabled_p = SCM_STACK_CHECKING_P;
 #endif
-      SCM_REDEFER_INTS;
+      SCM_CRITICAL_SECTION_START;
       DEACTIVATEJB (jmpbuf);
-      scm_dynwinds = SCM_CDR (scm_dynwinds);
-      SCM_REALLOW_INTS;
+      scm_i_set_dynwinds (SCM_CDR (scm_i_dynwinds ()));
+      SCM_CRITICAL_SECTION_END;
       throw_args = jbr.retval;
       throw_tag = jbr.throw_tag;
       jbr.throw_tag = SCM_EOL;
@@ -170,10 +171,10 @@ scm_internal_catch (SCM tag, scm_t_catch_body body, void *body_data, scm_t_catch
     {
       ACTIVATEJB (jmpbuf);
       answer = body (body_data);
-      SCM_REDEFER_INTS;
+      SCM_CRITICAL_SECTION_START;
       DEACTIVATEJB (jmpbuf);
-      scm_dynwinds = SCM_CDR (scm_dynwinds);
-      SCM_REALLOW_INTS;
+      scm_i_set_dynwinds (SCM_CDR (scm_i_dynwinds ()));
+      SCM_CRITICAL_SECTION_END;
     }
   return answer;
 }
@@ -241,15 +242,15 @@ scm_internal_lazy_catch (SCM tag, scm_t_catch_body body, void *body_data, scm_t_
   c.handler_data = handler_data;
   lazy_catch = make_lazy_catch (&c);
 
-  SCM_REDEFER_INTS;
-  scm_dynwinds = scm_acons (tag, lazy_catch, scm_dynwinds);
-  SCM_REALLOW_INTS;
+  SCM_CRITICAL_SECTION_START;
+  scm_i_set_dynwinds (scm_acons (tag, lazy_catch, scm_i_dynwinds ()));
+  SCM_CRITICAL_SECTION_END;
 
   answer = (*body) (body_data);
 
-  SCM_REDEFER_INTS;
-  scm_dynwinds = SCM_CDR (scm_dynwinds);
-  SCM_REALLOW_INTS;
+  SCM_CRITICAL_SECTION_START;
+  scm_i_set_dynwinds (SCM_CDR (scm_i_dynwinds ()));
+  SCM_CRITICAL_SECTION_END;
 
   return answer;
 }
@@ -385,20 +386,30 @@ static void
 handler_message (void *handler_data, SCM tag, SCM args)
 {
   char *prog_name = (char *) handler_data;
-  SCM p = scm_cur_errp;
+  SCM p = scm_current_error_port ();
 
-  if (scm_ilength (args) >= 3)
+  if (scm_ilength (args) == 4)
     {
       SCM stack   = scm_make_stack (SCM_BOOL_T, SCM_EOL);
       SCM subr    = SCM_CAR (args);
       SCM message = SCM_CADR (args);
       SCM parts   = SCM_CADDR (args);
-      SCM rest    = SCM_CDDDR (args);
+      SCM rest    = SCM_CADDDR (args);
 
       if (SCM_BACKTRACE_P && scm_is_true (stack))
 	{
+	  SCM highlights;
+
+	  if (scm_is_eq (tag, scm_arg_type_key)
+	      || scm_is_eq (tag, scm_out_of_range_key))
+	    highlights = rest;
+	  else
+	    highlights = SCM_EOL;
+
 	  scm_puts ("Backtrace:\n", p);
-	  scm_display_backtrace (stack, p, SCM_UNDEFINED, SCM_UNDEFINED);
+	  scm_display_backtrace_with_highlights (stack, p,
+						 SCM_BOOL_F, SCM_BOOL_F,
+						 highlights);
 	  scm_newline (p);
 	}
       scm_i_display_error (stack, p, subr, message, parts, rest);
@@ -445,12 +456,10 @@ SCM
 scm_handle_by_message (void *handler_data, SCM tag, SCM args)
 {
   if (scm_is_true (scm_eq_p (tag, scm_from_locale_symbol ("quit"))))
-    {
-      exit (scm_exit_status (args));
-    }
+    exit (scm_exit_status (args));
 
   handler_message (handler_data, tag, args);
-  exit (2);
+  scm_i_pthread_exit (NULL);
 }
 
 
@@ -461,6 +470,9 @@ scm_handle_by_message (void *handler_data, SCM tag, SCM args)
 SCM
 scm_handle_by_message_noexit (void *handler_data, SCM tag, SCM args)
 {
+  if (scm_is_true (scm_eq_p (tag, scm_from_locale_symbol ("quit"))))
+    exit (scm_exit_status (args));
+
   handler_message (handler_data, tag, args);
 
   return SCM_BOOL_F;
@@ -575,12 +587,18 @@ scm_ithrow (SCM key, SCM args, int noreturn SCM_UNUSED)
   SCM dynpair = SCM_UNDEFINED;
   SCM winds;
 
+  if (scm_i_critical_section_level)
+    {
+      fprintf (stderr, "throw from within critical section.\n");
+      abort ();
+    }
+
   /* Search the wind list for an appropriate catch.
      "Waiter, please bring us the wind list." */
-  for (winds = scm_dynwinds; SCM_CONSP (winds); winds = SCM_CDR (winds))
+  for (winds = scm_i_dynwinds (); scm_is_pair (winds); winds = SCM_CDR (winds))
     {
       dynpair = SCM_CAR (winds);
-      if (SCM_CONSP (dynpair))
+      if (scm_is_pair (dynpair))
 	{
 	  SCM this_key = SCM_CAR (dynpair);
 
@@ -592,19 +610,19 @@ scm_ithrow (SCM key, SCM args, int noreturn SCM_UNUSED)
   /* If we didn't find anything, print a message and abort the process
      right here.  If you don't want this, establish a catch-all around
      any code that might throw up. */
-  if (SCM_NULLP (winds))
+  if (scm_is_null (winds))
     {
       scm_handle_by_message (NULL, key, args);
       abort ();
     }
 
   /* If the wind list is malformed, bail.  */
-  if (!SCM_CONSP (winds))
+  if (!scm_is_pair (winds))
     abort ();
       
   jmpbuf = SCM_CDR (dynpair);
   
-  for (wind_goal = scm_dynwinds;
+  for (wind_goal = scm_i_dynwinds ();
        !scm_is_eq (SCM_CDAR (wind_goal), jmpbuf);
        wind_goal = SCM_CDR (wind_goal))
     ;
@@ -615,12 +633,12 @@ scm_ithrow (SCM key, SCM args, int noreturn SCM_UNUSED)
     {
       struct lazy_catch *c = (struct lazy_catch *) SCM_CELL_WORD_1 (jmpbuf);
       SCM handle, answer;
-      scm_dowinds (wind_goal, (scm_ilength (scm_dynwinds)
+      scm_dowinds (wind_goal, (scm_ilength (scm_i_dynwinds ())
 			       - scm_ilength (wind_goal)));
-      SCM_REDEFER_INTS;
-      handle = scm_dynwinds;
-      scm_dynwinds = SCM_CDR (scm_dynwinds);
-      SCM_REALLOW_INTS;
+      SCM_CRITICAL_SECTION_START;
+      handle = scm_i_dynwinds ();
+      scm_i_set_dynwinds (SCM_CDR (handle));
+      SCM_CRITICAL_SECTION_END;
       answer = (c->handler) (c->handler_data, key, args);
       scm_misc_error ("throw", "lazy-catch handler did return.", SCM_EOL);
     }
@@ -629,12 +647,12 @@ scm_ithrow (SCM key, SCM args, int noreturn SCM_UNUSED)
   else if (SCM_JMPBUFP (jmpbuf))
     {
       struct jmp_buf_and_retval * jbr;
-      scm_dowinds (wind_goal, (scm_ilength (scm_dynwinds)
+      scm_dowinds (wind_goal, (scm_ilength (scm_i_dynwinds ())
 			       - scm_ilength (wind_goal)));
       jbr = (struct jmp_buf_and_retval *)JBJMPBUF (jmpbuf);
       jbr->throw_tag = key;
       jbr->retval = args;
-      scm_last_debug_frame = SCM_JBDFRAME (jmpbuf);
+      scm_i_set_last_debug_frame (SCM_JBDFRAME (jmpbuf));
       longjmp (*JBJMPBUF (jmpbuf), 1);
     }
 
