@@ -1,45 +1,20 @@
 ;;; installed-scm-file
 
-;;;; 	Copyright (C) 1998, 1999, 2000, 2001, 2003 Free Software Foundation, Inc.
+;;;; Copyright (C) 1998,1999,2000,2001,2002, 2003 Free Software Foundation, Inc.
 ;;;; 
-;;;; This program is free software; you can redistribute it and/or modify
-;;;; it under the terms of the GNU General Public License as published by
-;;;; the Free Software Foundation; either version 2, or (at your option)
-;;;; any later version.
+;;;; This library is free software; you can redistribute it and/or
+;;;; modify it under the terms of the GNU Lesser General Public
+;;;; License as published by the Free Software Foundation; either
+;;;; version 2.1 of the License, or (at your option) any later version.
 ;;;; 
-;;;; This program is distributed in the hope that it will be useful,
+;;;; This library is distributed in the hope that it will be useful,
 ;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;;;; GNU General Public License for more details.
+;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+;;;; Lesser General Public License for more details.
 ;;;; 
-;;;; You should have received a copy of the GNU General Public License
-;;;; along with this software; see the file COPYING.  If not, write to
-;;;; the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-;;;; Boston, MA 02110-1301 USA
-;;;;
-;;;; As a special exception, the Free Software Foundation gives permission
-;;;; for additional uses of the text contained in its release of GUILE.
-;;;;
-;;;; The exception is that, if you link the GUILE library with other files
-;;;; to produce an executable, this does not by itself cause the
-;;;; resulting executable to be covered by the GNU General Public License.
-;;;; Your use of that executable is in no way restricted on account of
-;;;; linking the GUILE library code into it.
-;;;;
-;;;; This exception does not however invalidate any other reasons why
-;;;; the executable file might be covered by the GNU General Public License.
-;;;;
-;;;; This exception applies only to the code released by the
-;;;; Free Software Foundation under the name GUILE.  If you copy
-;;;; code from other Free Software Foundation releases into a copy of
-;;;; GUILE, as the General Public License permits, the exception does
-;;;; not apply to the code that you add in this way.  To avoid misleading
-;;;; anyone as to the status of such modified files, you must delete
-;;;; this exception notice from them.
-;;;;
-;;;; If you write modifications of your own for GUILE, it is your choice
-;;;; whether to permit this exception to apply to your modifications.
-;;;; If you do not wish that, delete this exception notice.
+;;;; You should have received a copy of the GNU Lesser General Public
+;;;; License along with this library; if not, write to the Free Software
+;;;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 ;;;; 
 
 
@@ -53,12 +28,18 @@
 (define-module (oop goops)
   :export-syntax (define-class class
 		  define-generic define-accessor define-method
+		  define-extended-generic define-extended-generics
 		  method)
-  :export (goops-version is-a?
+  :export (goops-version is-a? class-of
            ensure-metaclass ensure-metaclass-with-supers
 	   make-class
 	   make-generic ensure-generic
+	   make-extended-generic
 	   make-accessor ensure-accessor
+	   process-class-pre-define-generic
+	   process-class-pre-define-accessor
+	   process-define-generic
+	   process-define-accessor
 	   make-method add-method!
 	   object-eqv? object-equal?
 	   class-slot-ref class-slot-set! slot-unbound slot-missing 
@@ -95,7 +76,7 @@
 	   primitive-generic-generic enable-primitive-generic!
 	   method-procedure accessor-method-slot-definition
 	   slot-exists? make find-method get-keyword)
-  :re-export (class-of)  ;; from (guile)
+  :replace (<class> <operator-class> <entity-class> <entity>)
   :no-backtrace)
 
 ;; First initialize the builtin part of GOOPS
@@ -177,15 +158,31 @@
 (define (define-class-pre-definition keyword exp env)
   (case keyword
     ((#:getter #:setter)
-     (if (defined? exp env)
-	 `(define ,exp (ensure-generic ,exp ',exp))
-	 `(define ,exp (make-generic ',exp))))
+     `(process-class-pre-define-generic ',exp))
     ((#:accessor)
-     (if (defined? exp env)
-	 `(define ,exp (ensure-accessor ,exp ',exp))
-	 `(define ,exp (make-accessor ',exp))))
+     `(process-class-pre-define-accessor ',exp))
     (else #f)))
 
+(define (process-class-pre-define-generic name)
+  (let ((var (module-variable (current-module) name)))
+    (if (not (and var
+		  (variable-bound? var)
+		  (is-a? (variable-ref var) <generic>)))
+	(process-define-generic name))))
+
+(define (process-class-pre-define-accessor name)
+  (let ((var (module-variable (current-module) name)))
+    (cond ((or (not var)
+	       (not (variable-bound? var)))
+	   (process-define-accessor name))
+	  ((or (is-a? (variable-ref var) <accessor>)
+	       (is-a? (variable-ref var) <extended-generic-with-setter>)))
+	  ((is-a? (variable-ref var) <generic>)
+	   ;;*fixme* don't mutate an imported object!
+	   (variable-set! var (ensure-accessor (variable-ref var) name)))
+	  (else
+	   (process-define-accessor name)))))
+
 ;;; This code should be implemented in C.
 ;;;
 (define define-class
@@ -224,7 +221,7 @@
 	   (name cadr)
 	   (slots cdddr))
     
-    (procedure->macro
+    (procedure->memoizing-macro
       (lambda (exp env)
 	(cond ((not (top-level-env? env))
 	       (goops-error "define-class: Only allowed at top level"))
@@ -235,25 +232,20 @@
 		 `(begin
 		    ;; define accessors
 		    ,@(pre-definitions (slots exp) env)
-		 
-		    ,(if (defined? name env)
-		      
-			 ;; redefine an old class
-			 `(define ,name
-			    (let ((old ,name)
-				  (new (class ,@(cddr exp) #:name ',name)))
-			      (if (and (is-a? old <class>)
-				       ;; Prevent redefinition of non-objects
-				       (memq <object>
-					     (class-precedence-list old)))
-				  (class-redefinition old new)
-				  new)))
-		      
-			 ;; define a new class
-			 `(define ,name
-			    (class ,@(cddr exp) #:name ',name)))))))))))
+		    ;; update the current-module
+		    (let* ((class (class ,@(cddr exp) #:name ',name))
+			   (var (module-ensure-local-variable!
+				 (current-module) ',name))
+			   (old (and (variable-bound? var)
+				     (variable-ref var))))
+		      (if (and old
+			       (is-a? old <class>)
+			       (memq <object> (class-precedence-list old)))
+			  (variable-set! var (class-redefinition old class))
+			  (variable-set! var class)))))))))))
 
-(define standard-define-class define-class)
+(defmacro standard-define-class args
+  `(define-class ,@args))
 
 ;;; (class (SUPER ...) SLOT-DEFINITION ... OPTION ...)
 ;;;
@@ -361,23 +353,95 @@
 ;;;
 
 (define define-generic
-  (procedure->macro
+  (procedure->memoizing-macro
     (lambda (exp env)
       (let ((name (cadr exp)))
 	(cond ((not (symbol? name))
 	       (goops-error "bad generic function name: ~S" name))
-	      ((defined? name env)
-	       `(define ,name
-		  (if (is-a? ,name <generic>)
-		      (make <generic> #:name ',name)
-		      (ensure-generic ,name ',name))))
+	      ((top-level-env? env)
+	       `(process-define-generic ',name))
 	      (else
 	       `(define ,name (make <generic> #:name ',name))))))))
 
+(define (process-define-generic name)
+  (let ((var (module-ensure-local-variable! (current-module) name)))
+    (if (or (not var)
+	    (not (variable-bound? var))
+	    (is-a? (variable-ref var) <generic>))
+	;; redefine if NAME isn't defined previously, or is another generic
+	(variable-set! var (make <generic> #:name name))
+	;; otherwise try to upgrade the object to a generic
+	(variable-set! var (ensure-generic (variable-ref var) name)))))
+
+(define define-extended-generic
+  (procedure->memoizing-macro
+    (lambda (exp env)
+      (let ((name (cadr exp)))
+	(cond ((not (symbol? name))
+	       (goops-error "bad generic function name: ~S" name))
+	      ((null? (cddr exp))
+	       (goops-error "missing expression"))
+	      (else
+	       `(define ,name (make-extended-generic ,(caddr exp) ',name))))))))
+(define define-extended-generics
+  (procedure->memoizing-macro
+    (lambda (exp env)
+      (let ((names (cadr exp))
+	    (prefixes (get-keyword #:prefix (cddr exp) #f)))
+	(if prefixes
+	    `(begin
+	       ,@(map (lambda (name)
+			`(define-extended-generic ,name
+			   (list ,@(map (lambda (prefix)
+					  (symbol-append prefix name))
+					prefixes))))
+		      names))
+	    (goops-error "no prefixes supplied"))))))
+
 (define (make-generic . name)
   (let ((name (and (pair? name) (car name))))
     (make <generic> #:name name)))
 
+(define (make-extended-generic gfs . name)
+  (let* ((name (and (pair? name) (car name)))
+	 (gfs (if (pair? gfs) gfs (list gfs)))
+	 (gws? (any (lambda (gf) (is-a? gf <generic-with-setter>)) gfs)))
+    (let ((ans (if gws?
+		   (let* ((sname (and name (make-setter-name name)))
+			  (setters
+			   (apply append
+				  (map (lambda (gf)
+					 (if (is-a? gf <generic-with-setter>)
+					     (list (ensure-generic (setter gf)
+								   sname))
+					     '()))
+				       gfs)))
+			  (es (make <extended-generic-with-setter>
+				#:name name
+				#:extends gfs
+				#:setter (make <extended-generic>
+					   #:name sname
+					   #:extends setters))))
+		     (extended-by! setters (setter es))
+		     es)
+		   (make <extended-generic>
+		     #:name name
+		     #:extends gfs))))
+      (extended-by! gfs ans)
+      ans)))
+
+(define (extended-by! gfs eg)
+  (for-each (lambda (gf)
+	      (slot-set! gf 'extended-by
+			 (cons eg (slot-ref gf 'extended-by))))
+	    gfs))
+
+(define (not-extended-by! gfs eg)
+  (for-each (lambda (gf)
+	      (slot-set! gf 'extended-by
+			 (delq! eg (slot-ref gf 'extended-by))))
+	    gfs))
+
 (define (ensure-generic old-definition . name)
   (let ((name (and (pair? name) (car name))))
     (cond ((is-a? old-definition <generic>) old-definition)
@@ -391,40 +455,48 @@
 	  (else (make <generic> #:name name)))))
 
 (define define-accessor
-  (procedure->macro
+  (procedure->memoizing-macro
     (lambda (exp env)
       (let ((name (cadr exp)))
 	(cond ((not (symbol? name))
 	       (goops-error "bad accessor name: ~S" name))
-	      ((defined? name env)
-	       `(define ,name
-		  (if (and (is-a? ,name <generic-with-setter>)
-			   (is-a? (setter ,name) <generic>))
-		      (make-accessor ',name)
-		      (ensure-accessor ,name ',name))))
+	      ((top-level-env? env)
+	       `(process-define-accessor ',name))
 	      (else
 	       `(define ,name (make-accessor ',name))))))))
 
+(define (process-define-accessor name)
+  (let ((var (module-ensure-local-variable! (current-module) name)))
+    (if (or (not var)
+	    (not (variable-bound? var))
+	    (is-a? (variable-ref var) <accessor>)
+	    (is-a? (variable-ref var) <extended-generic-with-setter>))
+	;; redefine if NAME isn't defined previously, or is another accessor
+	(variable-set! var (make-accessor name))
+	;; otherwise try to upgrade the object to an accessor
+	(variable-set! var (ensure-accessor (variable-ref var) name)))))
+
 (define (make-setter-name name)
   (string->symbol (string-append "setter:" (symbol->string name))))
 
 (define (make-accessor . name)
   (let ((name (and (pair? name) (car name))))
-    (make <generic-with-setter>
+    (make <accessor>
 	  #:name name
 	  #:setter (make <generic>
 		         #:name (and name (make-setter-name name))))))
 
 (define (ensure-accessor proc . name)
   (let ((name (and (pair? name) (car name))))
-    (cond ((is-a? proc <generic-with-setter>)
-	   (if (is-a? (setter proc) <generic>)
-	       proc
-	       (upgrade-generic-with-setter proc (setter proc))))
+    (cond ((and (is-a? proc <accessor>)
+		(is-a? (setter proc) <generic>))
+	   proc)
+	  ((is-a? proc <generic-with-setter>)
+	   (upgrade-accessor proc (setter proc)))
 	  ((is-a? proc <generic>)
-	   (upgrade-generic-with-setter proc (make-generic name)))
+	   (upgrade-accessor proc (make-generic name)))
 	  ((procedure-with-setter? proc)
-	   (make <generic-with-setter>
+	   (make <accessor>
 		 #:name name
 		 #:default (procedure proc)
 		 #:setter (ensure-generic (setter proc) name)))
@@ -433,11 +505,19 @@
 	  (else
 	   (make-accessor name)))))
 
-(define (upgrade-generic-with-setter generic setter)
-  (let ((methods (generic-function-methods generic))
-	(gws (make <generic-with-setter>
+(define (upgrade-accessor generic setter)
+  (let ((methods (slot-ref generic 'methods))
+	(gws (make (if (is-a? generic <extended-generic>)
+		       <extended-generic-with-setter>
+		       <accessor>)
 		   #:name (generic-function-name generic)
+		   #:extended-by (slot-ref generic 'extended-by)
 		   #:setter setter)))
+    (if (is-a? generic <extended-generic>)
+	(let ((gfs (slot-ref generic 'extends)))
+	  (not-extended-by! gfs generic)
+	  (slot-set! gws 'extends gfs)
+	  (extended-by! gfs gws)))
     ;; Steal old methods
     (for-each (lambda (method)
 		(slot-set! method 'generic-function gws))
@@ -553,7 +633,7 @@
 
 (define (compute-new-list-of-methods gf new)
   (let ((new-spec (method-specializers new))
-	(methods  (generic-function-methods gf)))
+	(methods  (slot-ref gf 'methods)))
     (let loop ((l methods))
       (if (null? l)
 	  (cons new methods)
@@ -658,7 +738,7 @@
 ;;; Methods to compare objects
 ;;;
 
-(define-method (eqv? x y)   #f)
+(define-method (eqv? x y) #f)
 (define-method (equal? x y) (eqv? x y))
 
 ;;; These following two methods are for backward compatibility only.
@@ -762,6 +842,73 @@
 (define-method (display o file) 
   (write-object o file))
 
+;;;
+;;; Handling of duplicate bindings in the module system
+;;;
+
+(define-method (merge-generics (module <module>)
+			       (name <symbol>)
+			       (int1 <module>)
+			       (val1 <top>)
+			       (int2 <module>)
+			       (val2 <top>)
+			       (var <top>)
+			       (val <top>))
+  #f)
+
+(define-method (merge-generics (module <module>)
+			       (name <symbol>)
+			       (int1 <module>)
+			       (val1 <generic>)
+			       (int2 <module>)
+			       (val2 <generic>)
+			       (var <top>)
+			       (val <boolean>))
+  (and (not (eq? val1 val2))
+       (make-variable (make-extended-generic (list val2 val1) name))))
+
+(define-method (merge-generics (module <module>)
+			       (name <symbol>)
+			       (int1 <module>)
+			       (val1 <generic>)
+			       (int2 <module>)
+			       (val2 <generic>)
+			       (var <top>)
+			       (gf <extended-generic>))
+  (and (not (memq val2 (slot-ref gf 'extends)))
+       (begin
+	 (slot-set! gf
+		    'extends
+		    (cons val2 (delq! val2 (slot-ref gf 'extends))))
+	 (slot-set! val2
+		    'extended-by
+		    (cons gf (delq! gf (slot-ref val2 'extended-by))))
+	 var)))
+
+(module-define! duplicate-handlers 'merge-generics merge-generics)
+
+(define-method (merge-accessors (module <module>)
+				(name <symbol>)
+				(int1 <module>)
+				(val1 <top>)
+				(int2 <module>)
+				(val2 <top>)
+				(var <top>)
+				(val <top>))
+  #f)
+
+(define-method (merge-accessors (module <module>)
+				(name <symbol>)
+				(int1 <module>)
+				(val1 <accessor>)
+				(int2 <module>)
+				(val2 <accessor>)
+				(var <top>)
+				(val <top>))
+  (merge-generics module name int1 val1 int2 val2 var val))
+
+(module-define! duplicate-handlers 'merge-accessors merge-accessors)
+
 ;;;
 ;;; slot access
 ;;;
@@ -936,7 +1083,7 @@
     ;; So we can work as if we had only proper lists.
     (if (pair? l)       	  
 	(begin
-	  (if (eq? (car l) old)  
+	  (if (eqv? (car l) old)  
 	      (set-car! l new))
 	  (loop (cdr l))))))
 
@@ -1199,7 +1346,7 @@
 					     can-go-in-now))))
 		(loop
 		 (filter (lambda (x) (not (eq? x choice)))
-			     elements)
+			 elements)
 		 constraints
 		 (append result (list choice)))))))))
 
@@ -1283,7 +1430,7 @@
 	   (set (get-keyword #:slot-set! (slot-definition-options s) #f))
 	   (env (class-environment class)))
        (if (not (and get set))
-	   (goops-error "You must supply a :slot-ref and a :slot-set! in ~S"
+	   (goops-error "You must supply a #:slot-ref and a #:slot-set! in ~S"
 			s))
        (list get set)))
     (else    (next-method))))
@@ -1387,6 +1534,10 @@
 	(set-procedure-property! generic 'name name))
     ))
 
+(define-method (initialize (eg <extended-generic>) initargs)
+  (next-method)
+  (slot-set! eg 'extends (get-keyword #:extends initargs '())))
+
 (define dummy-procedure (lambda args *unspecified*))
 
 (define-method (initialize (method <method>) initargs)
