@@ -39,11 +39,11 @@ srfi1_ilength (SCM sx)
 
   do {
     if (SCM_NULL_OR_NIL_P(hare)) return i;
-    if (!SCM_CONSP (hare)) return -2;
+    if (!scm_is_pair (hare)) return -2;
     hare = SCM_CDR(hare);
     i++;
     if (SCM_NULL_OR_NIL_P(hare)) return i;
-    if (!SCM_CONSP (hare)) return -2;
+    if (!scm_is_pair (hare)) return -2;
     hare = SCM_CDR(hare);
     i++;
     /* For every two steps the hare takes, the tortoise takes one.  */
@@ -72,13 +72,13 @@ SCM_REGISTER_PROC (s_srfi1_concatenate_x, "concatenate!", 1, 0, 0, scm_append_x)
 
 
 SCM_DEFINE (scm_srfi1_count, "count", 2, 0, 1,
-            (SCM pred, SCM lst1, SCM rest),
+            (SCM pred, SCM list1, SCM rest),
 	    "Return a count of the number of times @var{pred} returns true\n"
 	    "when called on elements from the given lists.\n"
 	    "\n"
 	    "@var{pred} is called with @var{N} parameters @code{(@var{pred}\n"
 	    "@var{elem1} @dots{} @var{elemN})}, each element being from the\n"
-	    "corresponding @var{lst1} @dots{} @var{lstN}.  The first call is\n"
+	    "corresponding @var{list1} @dots{} @var{lstN}.  The first call is\n"
 	    "with the first element of each list, the second with the second\n"
 	    "element from each, and so on.\n"
 	    "\n"
@@ -91,44 +91,44 @@ SCM_DEFINE (scm_srfi1_count, "count", 2, 0, 1,
 
   count = 0;
 
-  if (SCM_NULLP (rest))
+  if (scm_is_null (rest))
     {
       /* one list */
       scm_t_trampoline_1 pred_tramp;
       pred_tramp = scm_trampoline_1 (pred);
       SCM_ASSERT (pred_tramp, pred, SCM_ARG1, FUNC_NAME);
 
-      for ( ; SCM_CONSP (lst1); lst1 = SCM_CDR (lst1))
-        count += scm_is_true (pred_tramp (pred, SCM_CAR (lst1)));
+      for ( ; scm_is_pair (list1); list1 = SCM_CDR (list1))
+        count += scm_is_true (pred_tramp (pred, SCM_CAR (list1)));
 
     end_lst1:
-      SCM_ASSERT_TYPE (SCM_NULL_OR_NIL_P (lst1), lst1, SCM_ARG2, FUNC_NAME,
+      SCM_ASSERT_TYPE (SCM_NULL_OR_NIL_P (list1), list1, SCM_ARG2, FUNC_NAME,
                        "list");
     }
-  else if (SCM_CONSP (rest) && SCM_NULLP (SCM_CDR (rest)))
+  else if (scm_is_pair (rest) && scm_is_null (SCM_CDR (rest)))
     {
       /* two lists */
       scm_t_trampoline_2 pred_tramp;
-      SCM lst2;
+      SCM list2;
 
       pred_tramp = scm_trampoline_2 (pred);
       SCM_ASSERT (pred_tramp, pred, SCM_ARG1, FUNC_NAME);
 
-      lst2 = SCM_CAR (rest);
+      list2 = SCM_CAR (rest);
       for (;;)
         {
-          if (! SCM_CONSP (lst1))
+          if (! scm_is_pair (list1))
             goto end_lst1;
-          if (! SCM_CONSP (lst2))
+          if (! scm_is_pair (list2))
             {
-              SCM_ASSERT_TYPE (SCM_NULL_OR_NIL_P (lst2), lst2, SCM_ARG3,
+              SCM_ASSERT_TYPE (SCM_NULL_OR_NIL_P (list2), list2, SCM_ARG3,
                                FUNC_NAME, "list");
               break;
             }
           count += scm_is_true (pred_tramp
-				(pred, SCM_CAR (lst1), SCM_CAR (lst2)));
-          lst1 = SCM_CDR (lst1);
-          lst2 = SCM_CDR (lst2);
+				(pred, SCM_CAR (list1), SCM_CAR (list2)));
+          list1 = SCM_CDR (list1);
+          list2 = SCM_CDR (list2);
         }
     }
   else
@@ -138,12 +138,12 @@ SCM_DEFINE (scm_srfi1_count, "count", 2, 0, 1,
       int  argnum;
 
       /* lstlst is a list of the list arguments */
-      lstlst = scm_cons (lst1, rest);
+      lstlst = scm_cons (list1, rest);
 
       /* args is the argument list to pass to pred, same length as lstlst,
          re-used for each call */
       args = SCM_EOL;
-      for (l = lstlst; SCM_CONSP (l); l = SCM_CDR (l))
+      for (l = lstlst; scm_is_pair (l); l = SCM_CDR (l))
         args = scm_cons (SCM_BOOL_F, args);
 
       for (;;)
@@ -151,11 +151,11 @@ SCM_DEFINE (scm_srfi1_count, "count", 2, 0, 1,
           /* first elem of each list in lstlst into args, and step those
              lstlst entries onto their next element */
           for (l = lstlst, a = args, argnum = 2;
-               SCM_CONSP (l);
+               scm_is_pair (l);
                l = SCM_CDR (l), a = SCM_CDR (a), argnum++)
             {
               lst = SCM_CAR (l);  /* list argument */
-              if (! SCM_CONSP (lst))
+              if (! scm_is_pair (lst))
                 {
                   SCM_ASSERT_TYPE (SCM_NULL_OR_NIL_P (lst), lst,
                                    argnum, FUNC_NAME, "list");
@@ -216,7 +216,7 @@ SCM_DEFINE (scm_srfi1_delete, "delete", 2, 1, 0,
   ret = SCM_EOL;
   p = &ret;
 
-  for ( ; SCM_CONSP (lst); lst = SCM_CDR (lst))
+  for ( ; scm_is_pair (lst); lst = SCM_CDR (lst))
     {
       if (scm_is_true (equal_p (pred, x, SCM_CAR (lst))))
         {
@@ -274,7 +274,7 @@ SCM_DEFINE (scm_srfi1_delete_x, "delete!", 2, 1, 0,
   SCM_ASSERT (equal_p, pred, SCM_ARG3, FUNC_NAME);
 
   for (prev = &lst, walk = lst;
-       SCM_CONSP (walk);
+       scm_is_pair (walk);
        walk = SCM_CDR (walk))
     {
       if (scm_is_true (equal_p (pred, x, SCM_CAR (walk))))
@@ -338,7 +338,7 @@ SCM_DEFINE (scm_srfi1_delete_duplicates, "delete-duplicates", 1, 1, 0,
 
   /* skip to end if an empty list (or something invalid) */
   ret = lst;
-  if (SCM_CONSP (lst))
+  if (scm_is_pair (lst))
     {
       if (SCM_UNBNDP (pred))
         equal_p = equal_trampoline;
@@ -355,7 +355,7 @@ SCM_DEFINE (scm_srfi1_delete_duplicates, "delete-duplicates", 1, 1, 0,
       for (;;)
         {
           lst = SCM_CDR (lst);
-          if (! SCM_CONSP (lst))
+          if (! scm_is_pair (lst))
             break;
           item = SCM_CAR (lst);
 
@@ -423,7 +423,7 @@ SCM_DEFINE (scm_srfi1_delete_duplicates_x, "delete-duplicates!", 1, 1, 0,
 
   /* skip to end if an empty list (or something invalid) */
   ret = lst;
-  if (SCM_CONSP (lst))
+  if (scm_is_pair (lst))
     {
       if (SCM_UNBNDP (pred))
         equal_p = equal_trampoline;
@@ -439,7 +439,7 @@ SCM_DEFINE (scm_srfi1_delete_duplicates_x, "delete-duplicates!", 1, 1, 0,
       for (;;)
         {
           lst = SCM_CDR (lst);
-          if (! SCM_CONSP (lst))
+          if (! scm_is_pair (lst))
             break;
           item = SCM_CAR (lst);
 
@@ -505,7 +505,7 @@ SCM_DEFINE (scm_srfi1_list_copy, "list-copy", 1, 0, 0,
   fill_here = &newlst;
   from_here = lst;
 
-  while (SCM_CONSP (from_here))
+  while (scm_is_pair (from_here))
     {
       SCM c;
       c = scm_cons (SCM_CAR (from_here), SCM_CDR (from_here));
@@ -531,23 +531,23 @@ check_map_args (SCM argv,
 		SCM args,
 		const char *who)
 {
-  SCM const *ve = SCM_VELTS (argv);
   long i;
 
-  for (i = SCM_VECTOR_LENGTH (argv) - 1; i >= 1; i--)
+  for (i = SCM_SIMPLE_VECTOR_LENGTH (argv) - 1; i >= 1; i--)
     {
+      SCM elt = SCM_SIMPLE_VECTOR_REF (argv, i);
       long elt_len;
 
-      if (!(SCM_NULLP (ve[i]) || SCM_CONSP (ve[i])))
+      if (!(scm_is_null (elt) || scm_is_pair (elt)))
 	{
 	check_map_error:
 	  if (gf)
 	    scm_apply_generic (gf, scm_cons (proc, args));
 	  else
-	    scm_wrong_type_arg (who, i + 2, ve[i]);
+	    scm_wrong_type_arg (who, i + 2, elt);
 	}
 	
-      elt_len = srfi1_ilength (ve[i]);
+      elt_len = srfi1_ilength (elt);
       if (elt_len < -1)
 	goto check_map_error;
 
@@ -579,14 +579,13 @@ scm_srfi1_map (SCM proc, SCM arg1, SCM args)
   long i, len;
   SCM res = SCM_EOL;
   SCM *pres = &res;
-  SCM const *ve = &args;		/* Keep args from being optimized away. */
 
   len = srfi1_ilength (arg1);
-  SCM_GASSERTn ((SCM_NULLP (arg1) || SCM_CONSP (arg1)) && len >= -1,
+  SCM_GASSERTn ((scm_is_null (arg1) || scm_is_pair (arg1)) && len >= -1,
 		g_srfi1_map,
 		scm_cons2 (proc, arg1, args), SCM_ARG2, s_srfi1_map);
   SCM_VALIDATE_REST_ARGUMENT (args);
-  if (SCM_NULLP (args))
+  if (scm_is_null (args))
     {
       scm_t_trampoline_1 call = scm_trampoline_1 (proc);
       SCM_GASSERT2 (call, g_srfi1_map, proc, arg1, SCM_ARG1, s_srfi1_map);
@@ -599,7 +598,7 @@ scm_srfi1_map (SCM proc, SCM arg1, SCM args)
 	}
       return res;
     }
-  if (SCM_NULLP (SCM_CDR (args)))
+  if (scm_is_null (SCM_CDR (args)))
     {
       SCM arg2 = SCM_CAR (args);
       int len2 = srfi1_ilength (arg2);
@@ -608,7 +607,7 @@ scm_srfi1_map (SCM proc, SCM arg1, SCM args)
 		    scm_cons2 (proc, arg1, args), SCM_ARG1, s_srfi1_map);
       if (len < 0 || (len2 >= 0 && len2 < len))
 	len = len2;
-      SCM_GASSERTn ((SCM_NULLP (arg2) || SCM_CONSP (arg2))
+      SCM_GASSERTn ((scm_is_null (arg2) || scm_is_pair (arg2))
 		    && len >= 0 && len2 >= -1,
 		    g_srfi1_map,
 		    scm_cons2 (proc, arg1, args),
@@ -625,15 +624,15 @@ scm_srfi1_map (SCM proc, SCM arg1, SCM args)
       return res;
     }
   args = scm_vector (arg1 = scm_cons (arg1, args));
-  ve = SCM_VELTS (args);
   len = check_map_args (args, len, g_srfi1_map, proc, arg1, s_srfi1_map);
   while (len > 0)
     {
       arg1 = SCM_EOL;
-      for (i = SCM_VECTOR_LENGTH (args) - 1; i >= 0; i--)
+      for (i = SCM_SIMPLE_VECTOR_LENGTH (args) - 1; i >= 0; i--)
 	{
-	  arg1 = scm_cons (SCM_CAR (ve[i]), arg1);
-	  SCM_VECTOR_SET (args, i, SCM_CDR (ve[i]));
+	  SCM elt = SCM_SIMPLE_VECTOR_REF (args, i);
+	  arg1 = scm_cons (SCM_CAR (elt), arg1);
+	  SCM_SIMPLE_VECTOR_SET (args, i, SCM_CDR (elt));
 	}
       *pres = scm_list_1 (scm_apply (proc, arg1, SCM_EOL));
       pres = SCM_CDRLOC (*pres);
@@ -651,14 +650,13 @@ SCM
 scm_srfi1_for_each (SCM proc, SCM arg1, SCM args)
 #define FUNC_NAME s_srfi1_for_each
 {
-  SCM const *ve = &args;		/* Keep args from being optimized away. */
   long i, len;
   len = srfi1_ilength (arg1);
-  SCM_GASSERTn ((SCM_NULLP (arg1) || SCM_CONSP (arg1)) && len >= -1,
+  SCM_GASSERTn ((scm_is_null (arg1) || scm_is_pair (arg1)) && len >= -1,
 		g_srfi1_for_each, scm_cons2 (proc, arg1, args),
 		SCM_ARG2, s_srfi1_for_each);
   SCM_VALIDATE_REST_ARGUMENT (args);
-  if (SCM_NULLP (args))
+  if (scm_is_null (args))
     {
       scm_t_trampoline_1 call = scm_trampoline_1 (proc);
       SCM_GASSERT2 (call, g_srfi1_for_each, proc, arg1,
@@ -672,7 +670,7 @@ scm_srfi1_for_each (SCM proc, SCM arg1, SCM args)
 	}
       return SCM_UNSPECIFIED;
     }
-  if (SCM_NULLP (SCM_CDR (args)))
+  if (scm_is_null (SCM_CDR (args)))
     {
       SCM arg2 = SCM_CAR (args);
       int len2 = srfi1_ilength (arg2);
@@ -681,7 +679,7 @@ scm_srfi1_for_each (SCM proc, SCM arg1, SCM args)
 		    scm_cons2 (proc, arg1, args), SCM_ARG1, s_srfi1_for_each);
       if (len < 0 || (len2 >= 0 && len2 < len))
 	len = len2;
-      SCM_GASSERTn ((SCM_NULLP (arg2) || SCM_CONSP (arg2))
+      SCM_GASSERTn ((scm_is_null (arg2) || scm_is_pair (arg2))
 		    && len >= 0 && len2 >= -1,
 		    g_srfi1_for_each,
 		    scm_cons2 (proc, arg1, args),
@@ -697,16 +695,16 @@ scm_srfi1_for_each (SCM proc, SCM arg1, SCM args)
       return SCM_UNSPECIFIED;
     }
   args = scm_vector (arg1 = scm_cons (arg1, args));
-  ve = SCM_VELTS (args);
   len = check_map_args (args, len, g_srfi1_for_each, proc, arg1,
 			s_srfi1_for_each);
   while (len > 0)
     {
       arg1 = SCM_EOL;
-      for (i = SCM_VECTOR_LENGTH (args) - 1; i >= 0; i--)
+      for (i = SCM_SIMPLE_VECTOR_LENGTH (args) - 1; i >= 0; i--)
 	{
-	  arg1 = scm_cons (SCM_CAR (ve[i]), arg1);
-	  SCM_VECTOR_SET (args, i, SCM_CDR (ve[i]));
+	  SCM elt = SCM_SIMPLE_VECTOR_REF (args, i);
+	  arg1 = scm_cons (SCM_CAR (elt), arg1);
+	  SCM_SIMPLE_VECTOR_SET (args, i, SCM_CDR (elt));
 	}
       scm_apply (proc, arg1, SCM_EOL);
       --len;
@@ -718,14 +716,21 @@ scm_srfi1_for_each (SCM proc, SCM arg1, SCM args)
 
 SCM_DEFINE (scm_srfi1_member, "member", 2, 1, 0,
            (SCM x, SCM lst, SCM pred),
-	    "Return the first sublist of @var{lst} whose car is\n"
-	    "@var{equal?} to @var{x} where the sublists of @var{lst} are\n"
-	    "the non-empty lists returned by @code{(list-tail @var{lst}\n"
-	    "@var{k})} for @var{k} less than the length of @var{lst}.  If\n"
-	    "@var{x} does not occur in @var{lst}, then @code{#f} (not the\n"
-	    "empty list) is returned.  If optional third argument @var{equal?}\n"
-	    "isn't given, @code{equal?} is used for comparison.\n"
-	    "(Extended from R5RS.)\n")
+	    "Return the first sublist of @var{lst} whose @sc{car} is equal\n"
+	    "to @var{x}.  If @var{x} does not appear in @var{lst}, return\n"
+	    "@code{#f}.\n"
+	    "\n"
+	    "Equality is determined by @code{equal?}, or by the equality\n"
+	    "predicate @var{=} if given.  @var{=} is called @code{(= @var{x}\n"
+	    "elem)}, ie.@: with the given @var{x} first, so for example to\n"
+	    "find the first element greater than 5,\n"
+	    "\n"
+	    "@example\n"
+	    "(member 5 '(3 5 1 7 2 9) <) @result{} (7 2 9)\n"
+	    "@end example\n"
+	    "\n"
+	    "This version of @code{member} extends the core @code{member} by\n"
+	    "accepting an equality predicate.")
 #define FUNC_NAME s_scm_srfi1_member
 {
   scm_t_trampoline_2 equal_p;
@@ -739,7 +744,7 @@ SCM_DEFINE (scm_srfi1_member, "member", 2, 1, 0,
     }
   for (; !SCM_NULL_OR_NIL_P (lst); lst = SCM_CDR (lst))
     {
-      if (scm_is_true (equal_p (pred, SCM_CAR (lst), x)))
+      if (scm_is_true (equal_p (pred, x, SCM_CAR (lst))))
 	return lst;
     }
   return SCM_BOOL_F;
@@ -762,10 +767,10 @@ SCM_DEFINE (scm_srfi1_assoc, "assoc", 2, 1, 0,
       equal_p = scm_trampoline_2 (pred);
       SCM_ASSERT (equal_p, pred, 3, FUNC_NAME);
     }
-  for(; SCM_CONSP (ls); ls = SCM_CDR (ls)) 
+  for(; scm_is_pair (ls); ls = SCM_CDR (ls)) 
     {
       SCM tmp = SCM_CAR (ls);
-      SCM_ASSERT_TYPE (SCM_CONSP (tmp), alist, SCM_ARG2, FUNC_NAME,
+      SCM_ASSERT_TYPE (scm_is_pair (tmp), alist, SCM_ARG2, FUNC_NAME,
 		       "association list");
       if (scm_is_true (equal_p (pred, SCM_CAR (tmp), key)))
 	return tmp;
@@ -816,6 +821,37 @@ SCM_DEFINE (scm_srfi1_partition, "partition", 2, 0, 0,
 }
 #undef FUNC_NAME
 
+SCM_DEFINE (scm_srfi1_remove, "remove", 2, 0, 0,
+	    (SCM pred, SCM list),
+	    "Return a list containing all elements from @var{lst} which do\n"
+	    "not satisfy the predicate @var{pred}.  The elements in the\n"
+	    "result list have the same order as in @var{lst}.  The order in\n"
+	    "which @var{pred} is applied to the list elements is not\n"
+	    "specified.")
+#define FUNC_NAME s_scm_srfi1_remove
+{
+  scm_t_trampoline_1 call = scm_trampoline_1 (pred);
+  SCM walk;
+  SCM *prev;
+  SCM res = SCM_EOL;
+  SCM_ASSERT (call, pred, 1, FUNC_NAME);
+  SCM_VALIDATE_LIST (2, list);
+  
+  for (prev = &res, walk = list;
+       scm_is_pair (walk);
+       walk = SCM_CDR (walk))
+    {
+      if (scm_is_false (call (pred, SCM_CAR (walk))))
+	{
+	  *prev = scm_cons (SCM_CAR (walk), SCM_EOL);
+	  prev = SCM_CDRLOC (*prev);
+	}
+    }
+
+  return res;
+}
+#undef FUNC_NAME
+
 void
 scm_init_srfi_1 (void)
 {
